/**
 * This file is part of FatturazioneElettronica - Core Extensions - Comune di Modena
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core Extensions - Comune di Modena.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.par.comunemodena;

import it.fe.core.par.invoice.AdditionalData;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;


/**
 * The persistent class for the conticontrattuali database table.
 * 
 */
@Entity
@Table(name="ContiContrattuali")
public class ContoContrattuale extends AdditionalData implements Serializable {
	private static final long serialVersionUID = 1L;

	/*@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long idContoContrattuale;*/

	@Column(nullable=false, length=30)
	private String codiceCliente;

	@Column(nullable=false, length=50, unique=true)
	private String contoContrattuale;

    @Column
	private Integer codiceComune;

	@Column
	private Integer codiceVia;
    
    @Column
	private String descrizione;
	
	@Column
	private Integer numeroCivico;
	
	@Column(length=2)
	private String esponente;
	
	@Column
	private Integer interno;
	
	@Column(length=1)
	private String esponenteInterno;

    @Temporal( TemporalType.DATE)
	@Column
	private Date dataFineValidita;

    @Temporal( TemporalType.DATE)
	@Column
	private Date dataInizioValidita;
	
	@Column
	private Integer codiceSettore;
	
	@Column
	private Integer codiceDivisione;
	
	@Column
	private Integer codiceUfficio;
    
	@OneToMany(	fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="idContoContrattuale", referencedColumnName="id")
	private List<AreaCommercialeCC> areecommercialiccs;

	@OneToMany(	fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="idContoContrattuale", referencedColumnName="id")
	private List<CapitoloArticoloCC> capitoloarticoloccs;

	@OneToMany(	fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="idContoContrattuale", referencedColumnName="id")
	private List<CentroDiCostoCC> centridicostoccs;

    public ContoContrattuale() {
    }

	/*public long getIdContoContrattuale() {
		return this.idContoContrattuale;
	}

	public void setIdContoContrattuale(long idContoContrattuale) {
		this.idContoContrattuale = idContoContrattuale;
	}*/

	public String getCodiceCliente() {
		return this.codiceCliente;
	}

	public void setCodiceCliente(String codiceCliente) {
		this.codiceCliente = codiceCliente;
	}

	public Integer getCodiceComune() {
		return this.codiceComune;
	}

	public void setCodiceComune(Integer codiceComune) {
		this.codiceComune = codiceComune;
	}

	public Integer getCodiceVia() {
		return this.codiceVia;
	}

	public void setCodiceVia(Integer codiceVia) {
		this.codiceVia = codiceVia;
	}

	public String getContoContrattuale() {
		return this.contoContrattuale;
	}

	public void setContoContrattuale(String contoContrattuale) {
		this.contoContrattuale = contoContrattuale;
	}

	public Date getDataFineValidita() {
		return this.dataFineValidita;
	}

	public void setDataFineValidita(Date dataFineValidita) {
		this.dataFineValidita = dataFineValidita;
	}

	public Date getDataInizioValidita() {
		return this.dataInizioValidita;
	}

	public void setDataInizioValidita(Date dataInizioValidita) {
		this.dataInizioValidita = dataInizioValidita;
	}

	public String getDescrizione() {
		return this.descrizione;
	}

	public void setDescrizione(String descrizione) {
		this.descrizione = descrizione;
	}

	public String getEsponente() {
		return this.esponente;
	}

	public void setEsponente(String esponente) {
		this.esponente = esponente;
	}

	public String getEsponenteInterno() {
		return this.esponenteInterno;
	}

	public void setEsponenteInterno(String esponenteInterno) {
		this.esponenteInterno = esponenteInterno;
	}

	public Integer getInterno() {
		return this.interno;
	}

	public void setInterno(Integer interno) {
		this.interno = interno;
	}

	public Integer getNumeroCivico() {
		return this.numeroCivico;
	}

	public void setNumeroCivico(Integer numeroCivico) {
		this.numeroCivico = numeroCivico;
	}
	
	public Integer getCodiceSettore() {
		return codiceSettore;
	}

	public void setCodiceSettore(Integer codiceSettore) {
		this.codiceSettore = codiceSettore;
	}

	public Integer getCodiceDivisione() {
		return codiceDivisione;
	}

	public void setCodiceDivisione(Integer codiceDivisione) {
		this.codiceDivisione = codiceDivisione;
	}

	public Integer getCodiceUfficio() {
		return codiceUfficio;
	}

	public void setCodiceUfficio(Integer codiceUfficio) {
		this.codiceUfficio = codiceUfficio;
	}

	public List<AreaCommercialeCC> getAreecommercialiccs() {
		return this.areecommercialiccs;
	}

	public void setAreecommercialiccs(List<AreaCommercialeCC> areecommercialiccs) {
		this.areecommercialiccs = areecommercialiccs;
	}
	
	public List<CapitoloArticoloCC> getCapitoloarticoloccs() {
		return this.capitoloarticoloccs;
	}

	public void setCapitoloarticoloccs(List<CapitoloArticoloCC> capitoloarticoloccs) {
		this.capitoloarticoloccs = capitoloarticoloccs;
	}
	
	public List<CentroDiCostoCC> getCentridicostoccs() {
		return this.centridicostoccs;
	}

	public void setCentridicostoccs(List<CentroDiCostoCC> centridicostoccs) {
		this.centridicostoccs = centridicostoccs;
	}
	
}