/**
 * This file is part of FatturazioneElettronica - Core Extensions - Comune di Modena
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core Extensions - Comune di Modena is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core Extensions - Comune di Modena.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.par.comunemodena;

import it.fe.core.par.invoice.AdditionalData;

import java.io.Serializable;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;

@Entity
@Table(name="InvoiceLineCoMoAdditionalData")
public class InvoiceLineAdditionalData extends AdditionalData implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	//Tipologia riga fattura (se Iva Irpef ecc.). Codificata in SIB
	@Column
	private String causaleRiga;
	//Codice aliquota Iva o tipo esenzione, ecc. Codificata in SIB
	@Column
	private String subCausaleRiga;
	//Tipologia Iva: I=Istituzionale/C=Commerciale
	@Column
	private String flagTipo;
	//Individua per cosa si spende ai fini Iva. Codificata in SIB (BLTA209)
	@Column
	private Integer areaCommerciale;
	//Lista delle aree commerciali su cui dividere l'importo
	@OneToMany(	fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="id", referencedColumnName="id")
	private List<AreaCommercialeInvoiceLine> areeCommerciali;
	
	@OneToMany(	fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="id", referencedColumnName="id")
	private List<CapitoloArticoloInvoiceLine> capitoliArticoli;
	
	@OneToMany(	fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="id", referencedColumnName="id")
	private List<CentroDiCostoInvoiceLine> centriDiCosto;
	 
	public String getCausaleRiga() {
		return causaleRiga;
	}

	public void setCausaleRiga(String causaleRiga) {
		this.causaleRiga = causaleRiga;
	}

	public String getSubCausaleRiga() {
		return subCausaleRiga;
	}

	public void setSubCausaleRiga(String subCausaleRiga) {
		this.subCausaleRiga = subCausaleRiga;
	}

	public String getFlagTipo() {
		return flagTipo;
	}

	public void setFlagTipo(String flagTipo) {
		this.flagTipo = flagTipo;
	}

	public Integer getAreaCommerciale() {
		return areaCommerciale;
	}

	public void setAreaCommerciale(Integer areaCommerciale) {
		this.areaCommerciale = areaCommerciale;
	}

	public List<AreaCommercialeInvoiceLine> getAreeCommerciali() {
		return areeCommerciali;
	}

	public void setAreeCommerciali(List<AreaCommercialeInvoiceLine> areeCommerciali) {
		this.areeCommerciali = areeCommerciali;
	}

	public List<CapitoloArticoloInvoiceLine> getCapitoliArticoli() {
		return capitoliArticoli;
	}

	public void setCapitoliArticoli(
			List<CapitoloArticoloInvoiceLine> capitoliArticoli) {
		this.capitoliArticoli = capitoliArticoli;
	}

	public List<CentroDiCostoInvoiceLine> getCentriDiCosto() {
		return centriDiCosto;
	}

	public void setCentriDiCosto(List<CentroDiCostoInvoiceLine> centriDiCosto) {
		this.centriDiCosto = centriDiCosto;
	}

}
