/**
 * This file is part of FatturazioneElettronica - Core Extensions - UniMoRe
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core Extensions - UniMoRe is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core Extensions - UniMoRe is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core Extensions - UniMoRe.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.unimore.sia;

import it.fe.core.common.utils.InvoiceUtils;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.common.utils.ReflectionUtils;
import it.fe.core.common.utils.db.oracle.OracleConnectionFactory;
import it.fe.core.ejb.unimore.cia.UnimoreCIAGenericDAO;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.unimore.cia.Anagrafico;
import it.fe.core.par.unimore.cia.Banca;
import it.fe.core.par.unimore.cia.BeneServizio;
import it.fe.core.par.unimore.cia.Causale;
import it.fe.core.par.unimore.cia.CentroCosto;
import it.fe.core.par.unimore.cia.ContoContabile;
import it.fe.core.par.unimore.cia.Indirizzo;
import it.fe.core.par.unimore.cia.ModalitaPagamento;
import it.fe.core.par.unimore.cia.UnitaOrganizzativa;
import it.fe.core.par.unimore.sia.ImpFattura;
import it.fe.core.par.unimore.sia.ImpFatturaDett;

import java.beans.PropertyDescriptor;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.log4j.Logger;

/**
 * @author fulvio
 * 
*
 *
 */
public class UnimoreSiaDao {
	
	private final static Logger logger = Logger.getLogger(UnimoreSiaDao.class);
	
	private final static String SEQUENCE_CURRENT = "SELECT SIA.S_IMP_FATT_SEQ.currval FROM dual";
	private final static String SEQUENCE_NEXT = "SELECT SIA.S_IMP_FATT_SEQ.nextval FROM dual";
	private final static String COLUMN_NAME_NEXTVAL = "NEXTVAL";
	
	private static final String TI_IMPORT = "I";
	private static final String CODICE_VALUTA = "EUR";
	private static final String STATO_APPROVAZIONE = "Y";
	private static final String FL_STATO_IMPORT = "N";
	private static final String FL_ERR_IMPORT = "N";
	private static final String FL_WAR_IMPORT = "N";
	private static final String LIQUIDAZIONE_DIFFERITA = "N";
	private static final String UTUV = "FE_TEST";
	private static final String TIPO_FATTURA = "i";
	private static final long INDIRIZZO_DEFAULT = 1;
	
	private static final String DATE_FORMAT_PATTERN = "yy-MMM-dd";
	
	private static final String IMP_FATTURA_DETT_INSERT_STMT = 
		"INSERT INTO IMP_FATTURA_DETT " +
		"(ABBUONO,CAUSALE,CENTRO_COSTO,CHIAVE_IMPEGNO,CHIAVE_PRENOTAZIONE,CODICE_ARTICOLO,CODICE_BENE,CODICE_IVA,COMMESSA,CONTO_CONTABILE,DA_NON_INVENTARIARE,DACR,DATA_COLLAUDO,DESCRIZIONE_BENE,DIFFERENZA_CAMBIO,DISCREPANTE_ORDINE,DUVA,ESERCIZIO_ORDINE,FINE,FLAG_SOLO_IVA,FUNZIONE_OBIETTIVO,ID_IMP_FATTURA,IMPORTO_IVA,IMPORTO_SCONTO,IMPORTO_TOTALE_IT,INIZIO,NUMERO_DETTAGLIO,NUMERO_DETTAGLIO_O,NUMERO_ORDINE,PERCENTUALE_IVA,PREZZO_TOTALE,PREZZO_UNITARIO,QUANTITA,RESPONSABILE_COLLAUDO,SCONTO_FINANZIARIO,SPESA_COMMISSIONE_BANCARIA,TIPO_FATTURA,TIPOLOGIA_SPESA,UNITA_ORGANIZZATIVA,UTUV) " +
		"VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )";
	
	
	/**
	 * sequence S_IMP_FATT_SEQ.CURRVAL must be defined in the current session
	 * select NEXTVAL from the sequence before selecting CURRVAL
	 * 
	 * @return
	 */
	public static long getSequenceCurrentValue() {
		long result = -1;
		Connection conn = OracleConnectionFactory.createConnection();
		try {
			PreparedStatement preparedStmt = conn.prepareStatement(SEQUENCE_CURRENT);
			ResultSet resultSet = preparedStmt.executeQuery();
			if (resultSet.next()) {
				result = resultSet.getLong(COLUMN_NAME_NEXTVAL);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			return result;
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return result;
	}
	
	/**
	 * @return
	 */
	public static long getSequenceNextValue() {
		long result = -1;
		Connection conn = OracleConnectionFactory.createConnection();
		try {
			PreparedStatement preparedStmt = conn.prepareStatement(SEQUENCE_NEXT);
			ResultSet resultSet = preparedStmt.executeQuery();
			if (resultSet.next()) {
				result = resultSet.getLong(COLUMN_NAME_NEXTVAL);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			return result;
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return result;
	}
	
	/**
	 * @param impFattura
	 * 
	 * @deprecated replaced by insert(Object object)
	 */
	@Deprecated
	public static void insertImpFattura(ImpFattura impFattura) {
		Set<PropertyDescriptor> propertyDescriptors = 
			new HashSet<PropertyDescriptor>(
					Arrays.asList(
							PropertyUtils.getPropertyDescriptors(
									impFattura.getClass() ) ) );
		
		String insertQuery = "";
		insertQuery += "INSERT INTO ";
		
		String tableName = " ";
		try {
			tableName = (String) impFattura.getClass().getField("ciaViewName").get(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		
		insertQuery += tableName;
		
		String columnNames = "(";
		String values = " VALUES (";
		
		String columnName = "";
		Object value = null;
		
		Iterator<PropertyDescriptor> i = propertyDescriptors.iterator();
		
		while (i.hasNext()) {
			
			PropertyDescriptor propertyDescriptor = i.next();
			
			if (propertyDescriptor.getName().equals("class"))
				continue;
			
			if (propertyDescriptor.getName().equals("null"))
				continue;
			
			if (propertyDescriptor.getName().equals("id"))
				continue;
			
			if (propertyDescriptor.getName().equals("id_imp_fattura")) {
				columnName = propertyDescriptor.getName().toUpperCase();
				value = impFattura.getId_imp_fattura();
			}
			
			else {
				Class<?> type = ReflectionUtils.getType(impFattura, propertyDescriptor.getName());
				try {
					if (type.equals(Class.forName("java.util.Date")) || type.equals(Class.forName("java.sql.Date"))) {
						columnName = propertyDescriptor.getName().toUpperCase();
						java.util.Date date = (Date) ReflectionUtils.getPropertyValue(impFattura, propertyDescriptor.getName());
						if (date != null) {
							// es: 30-MAY-02
							SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_PATTERN);
							value = "to_date('"+sdf.format(date)+"', 'YY-MON-DD')";
						}
						else {
							value = "NULL";
						}
						
					}
					else {
						columnName = propertyDescriptor.getName().toUpperCase();
						value = ReflectionUtils.getPropertyValue(impFattura, propertyDescriptor.getName());
						if (value != null) {
							value = "'"+value+"'";
						}
						else {
							value = "NULL";
						}
					}
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}
			}
			
			columnNames += columnName;
			values += value;
			
			if(i.hasNext()) {
				columnNames += ", ";
				values += ", ";
			}
			else {
				columnNames += " )";
				values += " )";
			}			
			
		}
		
		insertQuery += columnNames;
		insertQuery += values;
		
		Connection conn = OracleConnectionFactory.createConnection();
		
		try {
			PreparedStatement stmt = conn.prepareStatement(insertQuery);
			stmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		finally {
			OracleConnectionFactory.closeConnection(conn);
		}
	}
	
	/**
	 * Method to insert an ImpFatturaDett into the database
	 * 
	 * @param impFatturaDett
	 * 
	* @deprecated replaced by insert(Object object)
	 */
	@Deprecated
	public static void insertImpFatturaDett(ImpFatturaDett impFatturaDett) {
		
		Set<PropertyDescriptor> propertyDescriptors = 
			new HashSet<PropertyDescriptor>(
					Arrays.asList(
							PropertyUtils.getPropertyDescriptors(
									impFatturaDett.getClass() ) ) );
		
		String insertQuery = "";
		insertQuery += "INSERT INTO ";
		
		String tableName = " ";
		try {
			tableName = (String) impFatturaDett.getClass().getField("ciaViewName").get(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		
		insertQuery += tableName;
		
		String columnNames = "(";
		String values = " VALUES (";
		
		String columnName = "";
		Object value = null;
		
		Iterator<PropertyDescriptor> i = propertyDescriptors.iterator();
		
		while (i.hasNext()) {
			
			PropertyDescriptor propertyDescriptor = i.next();
			
			if (propertyDescriptor.getName().equals("class"))
				continue;
			
			if (propertyDescriptor.getName().equals("null"))
				continue;
			
			if (propertyDescriptor.getName().equals("id"))
				continue;
			
			if (propertyDescriptor.getName().equals("id_imp_fattura")) {
				columnName = propertyDescriptor.getName().toUpperCase();
				value = impFatturaDett.getId_imp_fattura();
			}
			
			else {
				Class<?> type = ReflectionUtils.getType(impFatturaDett, propertyDescriptor.getName());
				try {
					if (type.equals(Class.forName("java.util.Date")) || type.equals(Class.forName("java.sql.Date"))) {
						columnName = propertyDescriptor.getName().toUpperCase();
						java.util.Date date = (Date) ReflectionUtils.getPropertyValue(impFatturaDett, propertyDescriptor.getName());
						if (date != null) {
							// es: 30-MAY-02
							SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_PATTERN);
							value = "to_date('"+sdf.format(date)+"', 'YY-MON-DD')";
						}
						else {
							value = "NULL";
						}
					}
					else {
						columnName = propertyDescriptor.getName().toUpperCase();
						value = ReflectionUtils.getPropertyValue(impFatturaDett, propertyDescriptor.getName());
						if (value != null) {
							value = "'"+value+"'";
						}
						else {
							value = "NULL";
						}
					}
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}
	
			}
			
			columnNames += columnName;
			values += value;
			
			if(i.hasNext()) {
				columnNames += ", ";
				values += ", ";
			}
			else {
				columnNames += " )";
				values += " )";
			}			
			
		}
		
		insertQuery += columnNames;
		insertQuery += values;
		
		Connection conn = OracleConnectionFactory.createConnection();
		
		try {
			PreparedStatement stmt = conn.prepareStatement(insertQuery);
			stmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		
	}
	
	public static String prepareInsertQuery(Object object) {
		Set<PropertyDescriptor> propertyDescriptors = 
			new HashSet<PropertyDescriptor>(
					Arrays.asList(
							PropertyUtils.getPropertyDescriptors(
									object.getClass() ) ) );
		
		String insertQuery = "";
		insertQuery += "INSERT INTO ";
		
		String tableName = " ";
		try {
			tableName = (String) object.getClass().getField("ciaViewName").get(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		
		insertQuery += tableName;
		
		String columnNames = "(";
		String values = " VALUES (";
		
		String columnName = "";
		Object value = null;
		
		Iterator<PropertyDescriptor> i = propertyDescriptors.iterator();
		
		while (i.hasNext()) {
			
			PropertyDescriptor propertyDescriptor = i.next();
			
			if (propertyDescriptor.getName().equals("class"))
				continue;
			
			if (propertyDescriptor.getName().equals("null"))
				continue;
			
			if (propertyDescriptor.getName().equals("id"))
				continue;
			
			if (propertyDescriptor.getName().equals("id_imp_fattura")) {
				columnName = propertyDescriptor.getName().toUpperCase();
				value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
			}
			
			else {
				Class<?> type = ReflectionUtils.getType(object, propertyDescriptor.getName());
				try {
					if (type.equals(Class.forName("java.util.Date")) || type.equals(Class.forName("java.sql.Date"))) {
						columnName = propertyDescriptor.getName().toUpperCase();
						java.util.Date date = (Date) ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
						if (date != null) {
							// es: 30-MAY-02
							SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_PATTERN);
							value = "to_date('"+sdf.format(date)+"', 'YY-MON-DD')";
						}
						else {
							value = "NULL";
						}
					}
					else {
						columnName = propertyDescriptor.getName().toUpperCase();
						value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
						if (value != null) {
							value = "'"+value+"'";
						}
						else {
							value = "NULL";
						}
					}
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}
	
			}
			
			columnNames += columnName;
			values += value;
			
			if(i.hasNext()) {
				columnNames += ", ";
				values += ", ";
			}
			else {
				columnNames += " )";
				values += " )";
			}			
			
		}
		
		insertQuery += columnNames;
		insertQuery += values;
		
		return insertQuery;
	}
	
	/**
	 * Generic method to insert ImpFattura and ImpFattura DTOs into the database
	 * 
	 * @param object
	 */
	public static void insert(Object object) {
		Set<PropertyDescriptor> propertyDescriptors = 
			new HashSet<PropertyDescriptor>(
					Arrays.asList(
							PropertyUtils.getPropertyDescriptors(
									object.getClass() ) ) );
		
		String insertQuery = "";
		insertQuery += "INSERT INTO ";
		
		String tableName = " ";
		try {
			tableName = (String) object.getClass().getField("ciaViewName").get(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		
		insertQuery += tableName;
		
		String columnNames = "(";
		String values = " VALUES (";
		
		String columnName = "";
		Object value = null;
		
		Iterator<PropertyDescriptor> i = propertyDescriptors.iterator();
		
		while (i.hasNext()) {
			
			PropertyDescriptor propertyDescriptor = i.next();
			
			if (propertyDescriptor.getName().equals("class"))
				continue;
			
			if (propertyDescriptor.getName().equals("null"))
				continue;
			
			if (propertyDescriptor.getName().equals("id"))
				continue;
			
			if (propertyDescriptor.getName().equals("id_imp_fattura")) {
				columnName = propertyDescriptor.getName().toUpperCase();
				value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
			}
			
			else {
				Class<?> type = ReflectionUtils.getType(object, propertyDescriptor.getName());
				try {
					if (type.equals(Class.forName("java.util.Date")) || type.equals(Class.forName("java.sql.Date"))) {
						columnName = propertyDescriptor.getName().toUpperCase();
						java.util.Date date = (Date) ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
						if (date != null) {
							// es: 30-MAY-02
							SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_PATTERN);
							value = "to_date('"+sdf.format(date)+"', 'YY-MON-DD')";
						}
						else {
							value = "NULL";
						}
					}
					else {
						columnName = propertyDescriptor.getName().toUpperCase();
						value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
						if (value != null) {
							value = "'"+value+"'";
						}
						else {
							value = "NULL";
						}
					}
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}
	
			}
			
			columnNames += columnName;
			values += value;
			
			if(i.hasNext()) {
				columnNames += ", ";
				values += ", ";
			}
			else {
				columnNames += " )";
				values += " )";
			}			
			
		}
		
		insertQuery += columnNames;
		insertQuery += values;
		
		Connection conn = OracleConnectionFactory.createConnection();
		
		try {
			PreparedStatement stmt = conn.prepareStatement(insertQuery);
			stmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		finally {
			OracleConnectionFactory.closeConnection(conn);
		}
	}
	
	/**
	 * Generic find method
	 * 
	 * @param <T>
	 * @param clazz
	 * @param parameters
	 * @return
	 */
	public static  <T> List<T> findByParameters(Class<T> clazz, Map<String, Object> parameters) {
		
		List<T> results = new ArrayList<T>();
		Connection conn = OracleConnectionFactory.createConnection();
		
		try {
			
			PropertyDescriptor[] propertyDescriptors = PropertyUtils.getPropertyDescriptors(clazz);
			String query = "";
			query +="SELECT * FROM ";
			
			// estraggo il nome della tabella corrispondente
			//			Entity entityAnnotation = clazz.getAnnotation(Entity.class);
			//			String tableName = entityAnnotation.name();
			//			query += tableName;
			
			String tableName = (String) clazz.getField("ciaViewName").get(null);
			query += tableName;
			
			//aggiungo condizioni in base al numero di parametri passati in input alla funzione
			Set<Entry<String, Object>> rawParameters = parameters.entrySet();
			
			if (rawParameters.size() > 0) {
				query += " WHERE ";
				Iterator<Entry<String, Object>> i = rawParameters.iterator();
				while (i.hasNext()) {
					Entry<String, Object> entry = i.next();
					String columnName = entry.getKey().toUpperCase();
					Object value = entry.getValue();
					query += columnName + "='"+value+"'";
					if (i.hasNext()) {
						query += " AND ";
					}
				}
			}					
			
			PreparedStatement stmt = conn.prepareStatement(query);
			stmt.clearParameters();
			
			
			ResultSet resultSet = stmt.executeQuery();
			
			while(resultSet.next()) {
				Object obj = clazz.newInstance();
				for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
					String propertyName = propertyDescriptor.getName();
					if (propertyName.equals("class"))
						continue;
					if (propertyName.equals("null"))
						continue;
					if (propertyName.equals("id"))
						continue;
					if (propertyName.equals("ciaViewName"))
						continue;
					
					Object value = resultSet.getObject(propertyName.toUpperCase());
					
					if (value != null) {
						if (value instanceof BigDecimal) {
							value = ((BigDecimal)value).longValue();
						}
						PropertyUtils.setSimpleProperty(obj, propertyName, value);
					}
						
				}
				results.add((T) obj);
			}
			resultSet.close();
			stmt.close();
		} catch (Exception e) {
			logger.error(e.getMessage());
			return null;
		}
		finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return results;
	}
	
	public static <T> boolean deleteById_imp_fattura(Class<T> clazz, Long id_imp_fattura) {
		boolean result = false;
		if ( id_imp_fattura == null || id_imp_fattura < 0 )  {
			logger.error("Cannot delete an entry with an invalid id ");
			return result;
		}
		Connection conn = OracleConnectionFactory.createConnection();
		String deleteQuery = "";
		deleteQuery += "DELETE FROM ";
		String tableName = "";
		try {
			tableName = (String) clazz.getField("ciaViewName").get(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		
		deleteQuery += tableName;
		deleteQuery += " WHERE ID_IMP_FATTURA='"+id_imp_fattura+"'";
		
		try {
			PreparedStatement stmt = conn.prepareStatement(deleteQuery);
			stmt.executeUpdate();
			result = true;
			stmt.close();
		} catch (SQLException e) {
			logger.error(e.getMessage());
			result = false;
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return result;
	}
	
	/**
	 * @param id_imp_fattura
	 * @return
	 * 
	 * @deprecated replaced by deleteById_imp_fattura(Class<T> clazz, Long id_imp_fattura)
	 */
	@Deprecated
	public static boolean deleteImpFattura(Long id_imp_fattura) {
		boolean result = false;
		if ( id_imp_fattura == null || id_imp_fattura < 0 )  {
			logger.error("Cannot delete an entry with an invalid id ");
			return result;
		}
		Connection conn = OracleConnectionFactory.createConnection();
		String deleteQuery = "";
		deleteQuery += "DELETE FROM ";
		String tableName = "";
		try {
			tableName = (String) ImpFattura.class.getField("ciaViewName").get(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		
		deleteQuery += tableName;
		deleteQuery += " WHERE ID_IMP_FATTURA="+id_imp_fattura+"'";
		
		try {
			PreparedStatement stmt = conn.prepareStatement(deleteQuery);
			stmt.executeUpdate();
			result = true;
			stmt.close();
		} catch (SQLException e) {
			logger.error(e.getMessage());
			result = false;
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return result;
	}
	
	/**
	 * @param id_imp_fattura
	 * @return
	 * 
	 * @deprecated replaced by deleteById_imp_fattura(Class<T> clazz, Long id_imp_fattura)
	 */
	@Deprecated
	public static boolean deleteImpFatturaDett(Long id_imp_fattura) {
		boolean result = false;
		if ( id_imp_fattura == null || id_imp_fattura < 0 )  {
			logger.error("Cannot delete an entry with an invalid id ");
			return result;
		}
		Connection conn = OracleConnectionFactory.createConnection();
		String deleteQuery = "";
		deleteQuery += "DELETE FROM ";
		String tableName = "";
		try {
			tableName = (String) ImpFattura.class.getField("ciaViewName").get(null);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
		
		deleteQuery += tableName;
		deleteQuery += " WHERE ID_IMP_FATTURA="+id_imp_fattura+"'";
		
		try {
			PreparedStatement stmt = conn.prepareStatement(deleteQuery);
			stmt.executeUpdate();
			result = true;
			stmt.close();
		} catch (SQLException e) {
			logger.error(e.getMessage());
			result = false;
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		return result;
	}

	public static void insertImpFatturaDetts(List<InvoiceLine> invoiceLines, long id_imp_fattura, Map<String, Object> params) {
		
		Connection conn = OracleConnectionFactory.createConnection();
		
		try {
			
			long lineCount = 1;
			
			PreparedStatement ps = conn.prepareStatement(IMP_FATTURA_DETT_INSERT_STMT);
			
			for (InvoiceLine invoiceLine : invoiceLines) {
				if ( !(invoiceLine.getLineExtensionAmount() == 0.0) ) {
					ps = UnimoreSiaDao.addBatchinsert(invoiceLineToImpFatturaDett(invoiceLine, id_imp_fattura, lineCount, params), ps);
					lineCount++;
				}
			}
			
			ps.executeBatch();
			ps.close();
		
		} catch (SQLException e) {
			logger.error(e.getMessage());
		} finally {
			OracleConnectionFactory.closeConnection(conn);
		}
		
	}
	
	private static PreparedStatement addBatchinsert(Object object, PreparedStatement ps) {
		
		// alphabetical order
		List<PropertyDescriptor> propertyDescriptors = Arrays.asList(PropertyUtils.getPropertyDescriptors( object.getClass() ) ) ;
		
		Object value = null;
		
		Iterator<PropertyDescriptor> i = propertyDescriptors.iterator();
		
		int parameterIndex = 1;
		
		while (i.hasNext()) {
			
			try {
			
				PropertyDescriptor propertyDescriptor = i.next();
				
				if (propertyDescriptor.getName().equals("class"))
					continue;
				
				if (propertyDescriptor.getName().equals("null"))
					continue;
				
				if (propertyDescriptor.getName().equals("id"))
					continue;
				
				if (propertyDescriptor.getName().equals("id_imp_fattura")) {
					value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
					ps.setLong(parameterIndex, (Long) value);
				}
				
				if (propertyDescriptor.getName().equals("numero_dettaglio_o")) {
					value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
					ps.setNull(parameterIndex, java.sql.Types.BIGINT);
				}
				
				else {
					Class<?> type = ReflectionUtils.getType(object, propertyDescriptor.getName());
					
						if (type.equals(Class.forName("java.util.Date")) || type.equals(Class.forName("java.sql.Date"))) {
							java.util.Date date = (Date) ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
							if (date != null) {
								ps.setDate(parameterIndex, (java.sql.Date) date);
							}
							else {
								ps.setNull(parameterIndex, java.sql.Types.DATE);
							}
						}
						else if (type.equals(Class.forName("java.lang.String"))) {
							value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
							if (value != null) {
								ps.setString(parameterIndex, (String) value);
							}
							else {
								ps.setNull(parameterIndex, java.sql.Types.VARCHAR);
							}
						}
						else if (type.equals(java.lang.Long.TYPE)) {
							value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
							if (value != null) {
								ps.setLong(parameterIndex, (Long) value);
							}
							else {
								ps.setNull(parameterIndex, java.sql.Types.BIGINT);
							}
						}
						else if (type.equals(java.lang.Double.TYPE)) {
							value = ReflectionUtils.getPropertyValue(object, propertyDescriptor.getName());
							if (value != null) {
								ps.setDouble(parameterIndex, (Double) value);
							}
							else {
								ps.setNull(parameterIndex, java.sql.Types.DOUBLE);
							}
						}
					
				}
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (SQLException e) {
				e.printStackTrace();
			}
			
			parameterIndex++;
			
		}
		try {
			ps.addBatch();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return ps;
		
	}

	/**
	 * @param invoiceLine
	 * @param id_imp_fattura
	 * @param detailNumber
	 * @param params TODO
	 * @return
	 */
	private static ImpFatturaDett invoiceLineToImpFatturaDett(InvoiceLine invoiceLine, long id_imp_fattura, long detailNumber, Map<String,Object> params) {

		ImpFatturaDett impFatturaDett = new ImpFatturaDett();
		
		// id_imp_fattura
		impFatturaDett.setId_imp_fattura(id_imp_fattura);
		
		// numero_dettaglio
		impFatturaDett.setNumero_dettaglio(detailNumber);
		
		// descrizione_bene - varchar2(250)
		
		impFatturaDett.setDescrizione_bene(
				invoiceLine.getItemDescription().length() <= 250 ? 
						invoiceLine.getItemDescription() : invoiceLine.getItemDescription().substring(0,250));
			
		// QUANTITA / PREZZO_UNITARIO / IMPORTO_TOTALE_IT
		
		impFatturaDett.setPrezzo_unitario(invoiceLine.getPriceAmount());
		impFatturaDett.setQuantita(invoiceLine.getInvoicedQuantity());
		impFatturaDett.setImporto_totale_it(invoiceLine.getLineExtensionAmount());
	
		if (bankersRounding(invoiceLine.getPriceAmount() * invoiceLine.getInvoicedQuantity()) != invoiceLine.getLineExtensionAmount()) {
			// ECCEZIONE 1. QUOTE FISSE HERA
			//	In alcuni dettagli di riga il prodotto fra quantità e prezzo unitario non coincide con il prezzo totale della riga.
			// 	Il motivo è dovuto al fatto che il prezzo unitario è espresso su base annua e deve essere pertanto calcolato 
			//	preliminarmente il prezzo unitario giornaliero.
			if (bankersRounding( (invoiceLine.getPriceAmount()/365) * invoiceLine.getInvoicedQuantity()) == invoiceLine.getLineExtensionAmount()) {
				impFatturaDett.setPrezzo_unitario(invoiceLine.getPriceAmount()/365);
				impFatturaDett.setQuantita(invoiceLine.getInvoicedQuantity());
				impFatturaDett.setImporto_totale_it(invoiceLine.getLineExtensionAmount());
			}
			// ECCEZIONE 2. quantita == 0.0 
			//	Da Consulenza CIA: "Sconsigliamo di inserire come quantità il valore zero, al limite se la quantità non è nota o non è significativa 
			// 	(come nel caso di acquisizione di servizi), consigliamo di inserire come quantità '1' e imporre che il prezzo unitario 
			// 	sia uguale a quello totale.
			else if (invoiceLine.getInvoicedQuantity() == 0.0) {
				impFatturaDett.setQuantita(1.0);
				impFatturaDett.setPrezzo_unitario(invoiceLine.getLineExtensionAmount());
				impFatturaDett.setImporto_totale_it(invoiceLine.getLineExtensionAmount());
			}
		}
			
		// importo_totale_it (controvalore in euro dell'imponibile della fattura) -> DEVE ESSERE PRESO SENZA ALCUNA MANIPOLAZIONE DAL FLUSSO
		// InvoiceLine.lineExtensionAmount rappresenta l'imponibile (netto) di riga
		//impFatturaDett.setImporto_totale_it( bankersRounding(impFatturaDett.getPrezzo_unitario() * impFatturaDett.getQuantita()) );
		//impFatturaDett.setImporto_totale_it(impFatturaDett.getQuantita() * impFatturaDett.getPrezzo_unitario());
		//impFatturaDett.setImporto_totale_it(invoiceLine.getTaxDetail().getTaxableAmount());
		//impFatturaDett.setImporto_totale_it(invoiceLine.getLineExtensionAmount());
		
		// flag_solo_iva
		/*if (impFatturaDett.getImporto_totale_it() == 0 )
			impFatturaDett.setFlag_solo_iva("Y");*/
		
		// prezzo_totale (se codice valura = EUR è uguale a importo_totale_it, altrimenti è l'imponibile totale del dettaglio in valuta straniera
		if ("EUR".equals(invoiceLine.getCurrencyID())) {
			impFatturaDett.setPrezzo_totale(invoiceLine.getLineExtensionAmount());
		}
			
		// percentuale_iva
		impFatturaDett.setPercentuale_iva(invoiceLine.getTaxDetail().getTaxPercent());
		
		// importo_iva (deve essere importo_totale_it*percentuale_iva/100)
		// Corrisponde a: InvoiceLine.TaxDetail.taxAmount
		
		// Importo IVA calcolato al volo
		impFatturaDett.setImporto_iva(impFatturaDett.getImporto_totale_it() * impFatturaDett.getPercentuale_iva() / 100);
		
		// Importo IVA preso da FE
		//impFatturaDett.setImporto_iva(invoiceLine.getTaxDetail().getTaxAmount());
		
		// Secondo documentazione CIA deve essere: importo_totale_it*100/percentuale_iva (non ha senso)
		//impFatturaDett.setImporto_iva(impFatturaDett.getImporto_totale_it()*100/impFatturaDett.getPercentuale_iva());
		
		// codice_iva - VARCHAR2(10)
		// TODO: ?PATCH? per risolvere warning nell'applicativo CIA
		String codiceIva = "";
		switch ((int)invoiceLine.getTaxDetail().getTaxPercent()) {
		case 0:
			codiceIva = "I114";
			break;
		case 4:
			codiceIva = "I101";
			break;
		case 10:
			codiceIva = "I102";
			break;
		case 20:
			codiceIva = "I103";
			break;
		default:
			break;
		}
		
		impFatturaDett.setCodice_iva(codiceIva.length() <= 10 ?
				codiceIva : codiceIva.substring(0, 10));
		
		// inizio
		impFatturaDett.setInizio(invoiceLine.getConsumption().getStartConsumptionDate());
		
		// fine
		impFatturaDett.setFine(invoiceLine.getConsumption().getEndConsumptionDate());
		
		// dacr
		impFatturaDett.setDacr(new java.sql.Date(new java.util.Date().getTime()));
		
		// utuv: stringa da concordare varchar2(20)
		impFatturaDett.setUtuv(
				UTUV.length() <= 20 ?
				UTUV : UTUV.substring(0, 20));
		
		// duva
		impFatturaDett.setDuva(new java.sql.Date(new java.util.Date().getTime()));
		
		// DATO AGGIUNTIVO unita_organizzativa varchar2(60)
		if ( InvoiceUtils.invoiceHasSavedAdditionalData(invoiceLine.getInvoice(), UnitaOrganizzativa.class) ) {
			List<UnitaOrganizzativa> savedAdditionalData = InvoiceUtils.getInvoiceStoredAdditionalData(invoiceLine.getInvoice(), UnitaOrganizzativa.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia una sola UO
				impFatturaDett.setUnita_organizzativa(
						savedAdditionalData.get(0).getChiave_completa().length() <= 60 ?
								savedAdditionalData.get(0).getChiave_completa() : savedAdditionalData.get(0).getChiave_completa().substring(0, 60));
			}
		}
		
		// DATO AGGIUNTIVO causale varchar2(10)
		if ( InvoiceUtils.invoiceLineHasSavedAdditionalData(invoiceLine, Causale.class) ) {
			List<Causale> savedAdditionalData = InvoiceUtils.getInvoiceLineStoredAdditionalData(invoiceLine, Causale.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia una sola causale
				impFatturaDett.setCausale(
						savedAdditionalData.get(0).getCodice().length() <= 10 ?
								savedAdditionalData.get(0).getCodice() : savedAdditionalData.get(0).getCodice().substring(0,10));
			}
		}
		
		// DATO AGGIUNTIVO centro_costo varchar2(60)
		if (InvoiceUtils.invoiceLineHasSavedAdditionalData(invoiceLine, CentroCosto.class)) {
			List<CentroCosto> savedAdditionalData = InvoiceUtils.getInvoiceLineStoredAdditionalData(invoiceLine, CentroCosto.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia un solo centro di costo
				impFatturaDett.setCentro_costo(
						savedAdditionalData.get(0).getChiave_completa().length() <= 60 ?
								savedAdditionalData.get(0).getChiave_completa() : savedAdditionalData.get(0).getChiave_completa().substring(0,60));
			}
		}
		
		// BeneServizio: codice_bene - varchar2(5)
		if (InvoiceUtils.invoiceLineHasSavedAdditionalData(invoiceLine, BeneServizio.class)) {
			List<BeneServizio> savedAdditionalData = InvoiceUtils.getInvoiceLineStoredAdditionalData(invoiceLine, BeneServizio.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia un solo bene servizio
				impFatturaDett.setCodice_bene(
						savedAdditionalData.get(0).getCodice_bene().length() <= 5 ?
								savedAdditionalData.get(0).getCodice_bene() : savedAdditionalData.get(0).getCodice_bene().substring(0,5));
			}
		}
		
		// ContoContabile - varchar2(45)
		if (InvoiceUtils.invoiceLineHasSavedAdditionalData(invoiceLine, ContoContabile.class)) {
			List<ContoContabile> savedAdditionalData = InvoiceUtils.getInvoiceLineStoredAdditionalData(invoiceLine, ContoContabile.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia un solo conto contabile
				impFatturaDett.setConto_contabile(
						savedAdditionalData.get(0).getChiave_completa().length() <= 45 ?
								savedAdditionalData.get(0).getChiave_completa() : savedAdditionalData.get(0).getChiave_completa().substring(0, 45));
			}
		}
		
		// DATO AGGIUNTIVO tipo_fattura varchar2(10)
		impFatturaDett.setTipo_fattura(
				TIPO_FATTURA.length() <= 10 ?
						TIPO_FATTURA : TIPO_FATTURA.substring(0, 10));
		
		return impFatturaDett;
	}
	
	/**
	 * Performs banker's double rounding
	 * 
	 * @param value
	 * @return
	 */
	private static double bankersRounding(double value) {
		return BigDecimal.valueOf(value).setScale(2, RoundingMode.HALF_EVEN).doubleValue();
	}

	/**
	 * @param invoice
	 * @param id_imp_fattura
	 * @return
	 */
	public static ImpFattura invoiceToImpFattura(Invoice invoice, long id_imp_fattura) {
		ImpFattura impFattura = new ImpFattura();
		// id_imp_fattura
		impFattura.setId_imp_fattura(id_imp_fattura);
		// ti_import
		impFattura.setTi_import(TI_IMPORT);
		// numero
		impFattura.setNumero((new Long(id_imp_fattura)).toString());
		// DATO AGGIUNTIVO bilancio, esercizio, unità organizzativa
		if (InvoiceUtils.invoiceHasSavedAdditionalData(invoice, UnitaOrganizzativa.class)) {
			List<UnitaOrganizzativa> savedAdditionalData = InvoiceUtils.getInvoiceStoredAdditionalData(invoice, UnitaOrganizzativa.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia una sola unità organizzativa
				impFattura.setEsercizio(
						savedAdditionalData.get(0).getEsercizio().length() <= 5 ?
								savedAdditionalData.get(0).getEsercizio() : savedAdditionalData.get(0).getEsercizio().substring(0, 5));
				impFattura.setBilancio(
						savedAdditionalData.get(0).getIdentificativo_bilancio().length() <= 60 ?
								savedAdditionalData.get(0).getIdentificativo_bilancio() : savedAdditionalData.get(0).getIdentificativo_bilancio().substring(0, 60));
				impFattura.setUnita_organizzativa(
						savedAdditionalData.get(0).getChiave_completa().length() <= 60 ? 
								savedAdditionalData.get(0).getChiave_completa() : savedAdditionalData.get(0).getChiave_completa().substring(0, 60));
			}
		}
		// codice_valuta
		impFattura.setCodice_valuta(CODICE_VALUTA);
		// TODO: DATO AGGIUNTIVO tipo_fattura (FATALE), per ora "i"
		impFattura.setTipo_fattura(TIPO_FATTURA);
		// numero_fattura_fornitore
		impFattura.setNumero_fattura_fornitore(
				invoice.getInvoiceNumber().length() <= 20 ?
						invoice.getInvoiceNumber() : invoice.getInvoiceNumber().substring(0, 20));
		// data_fattura
		impFattura.setData_fattura(invoice.getIssueDate());
		// data_scadenza
		impFattura.setData_scadenza(invoice.getPaymentDetail().getPaymentDueDate());
		// DATO AGGIUNTIVO modalità_pagamento
		if (InvoiceUtils.invoiceHasSavedAdditionalData(invoice, ModalitaPagamento.class)) {
			List<ModalitaPagamento> savedAdditionalData = InvoiceUtils.getInvoiceStoredAdditionalData(invoice, ModalitaPagamento.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia una sola modalità di pagamento
				impFattura.setModalita_pagamento(
						savedAdditionalData.get(0).getCodice_rif().length() <= 10 ?
								savedAdditionalData.get(0).getCodice_rif() : savedAdditionalData.get(0).getCodice_rif().substring(0, 10));
			}
		}
		
		//invoice = JNDIUtils.getInvoiceManager().fetchTaxDetails(invoice);
		
		List<TaxDetail> taxDetails = JNDIUtils.getInvoiceManager().fetchTaxDetails(invoice);
		
		if ( (taxDetails!=null) && (!taxDetails.isEmpty()) ) {
			
			double taxDetailsTotal = 0;
			double taxableAmountTotal = 0;
			
			for (TaxDetail taxDetail : taxDetails) {
				taxDetailsTotal += taxDetail.getTaxAmount();
				taxableAmountTotal += taxDetail.getTaxableAmount();
			}
			// totale_iva
			impFattura.setTotale_iva(taxDetailsTotal);
			// totale_imponibile
			impFattura.setTotale_imponibile(taxableAmountTotal);
			// TODO: importo_valuta (come estraggo il codice valuta dall'oggetto Invoice?)
			impFattura.setImporto_valuta(impFattura.getTotale_imponibile());
		}
	
		// ABI / CAB / NUMEROCONTO
		if ("CC".equals(impFattura.getModalita_pagamento())) {
			if (InvoiceUtils.invoiceHasSavedAdditionalData(invoice, Banca.class)) {
				List<Banca> savedAdditionalData = InvoiceUtils.getInvoiceStoredAdditionalData(invoice, Banca.class);
				if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
					// Mi aspetto che ci sia una sola banca
					impFattura.setAbi(
							savedAdditionalData.get(0).getAbi().length() <= 5 ? 
									savedAdditionalData.get(0).getAbi() : savedAdditionalData.get(0).getAbi().substring(0, 5));
					impFattura.setCab(
							savedAdditionalData.get(0).getCab().length() <= 5 ? 
									savedAdditionalData.get(0).getCab() : savedAdditionalData.get(0).getCab().substring(0, 5));
					impFattura.setNumero_conto(
							savedAdditionalData.get(0).getNumeroconto().length() <= 15 ? 
									savedAdditionalData.get(0).getNumeroconto() : savedAdditionalData.get(0).getNumeroconto().substring(0, 15));
				}
			}
			
		}
		
		
		//  DATO AGGIUNTIVO codice_fornitore
		if (InvoiceUtils.invoiceHasSavedAdditionalData(invoice, Anagrafico.class)) {
			List<Anagrafico> savedAdditionalData = InvoiceUtils.getInvoiceStoredAdditionalData(invoice, Anagrafico.class);
			if ( (savedAdditionalData !=null) && (!savedAdditionalData.isEmpty())) {
				// Mi aspetto che ci sia un solo fornitore
				impFattura.setCodice_fornitore(
						savedAdditionalData.get(0).getCodice().length() <= 10 ? 
								savedAdditionalData.get(0).getCodice() : savedAdditionalData.get(0).getCodice().substring(0, 10));
				
				// indirizzo
				String addressQuery = 	"select * from epoca_indirizzo " +
								"where codice='"+savedAdditionalData.get(0).getCodice()+"' and via like upper('%"+invoice.getSupplier().getPostalAddress().getStreetName()+"%')";
				List<Indirizzo> supplierAddresses = UnimoreCIAGenericDAO.findByNativeQuery(Indirizzo.class, addressQuery);
				if (supplierAddresses != null && !supplierAddresses.isEmpty() && supplierAddresses.size() == 1) {
					impFattura.setIndirizzo(supplierAddresses.get(0).getNumero_indirizzo().longValue());
				}
				else {
					impFattura.setIndirizzo(INDIRIZZO_DEFAULT);
				}
				
				
			}
		}

		// stato_approvazione (fisso a Y)
		impFattura.setStato_approvazione(STATO_APPROVAZIONE);
		// dacr
		impFattura.setDacr(new java.sql.Date(new java.util.Date().getTime()));
		// utuv: stringa da concordare
		impFattura.setUtuv(UTUV);
		// duva
		impFattura.setDuva(new java.sql.Date(new java.util.Date().getTime()));
		
		// liquidazione_differita: default N, può essere Y o N
		impFattura.setLiquidazione_differita(LIQUIDAZIONE_DIFFERITA);
		
		// data_protocollatura: Se compilato: data di protocollo della fattura che si sta importando
		impFattura.setData_protocollatura(new java.sql.Date(new java.util.Date().getTime()));
		
		// sezionale: i sezionali non sono gestiti; il valore da utilizzare è il valore di BILANCIO senza i punti
		if (!"".equals(impFattura.getBilancio())) {
			impFattura.setSezionale(getSezionale(impFattura.getBilancio()));
		}
		
		// se ci sono righe con quantità == 0 (causa errore nella fattura ricevuta), lo segnalo nelle note
		String note = "";
		for (InvoiceLine line : invoice.getInvoiceLines()) {
			if (line.getInvoicedQuantity() == 0.0) {
				note += "AVVERTENZA FE: Riga "+line.getLineNumber()+" ("+line.getItemDescription()+") quantità=0 e importo_unitario="+line.getPriceAmount()+line.getCurrencyID()+". Quantità impostata a 1 e prezzo_unitario=prezzo_totale.             ";
			}
		}
		impFattura.setNote(
				note.length() <= 500 ? 
						note : note.substring(0, 500));
		
		impFattura.setFl_stato_import(FL_STATO_IMPORT);
		impFattura.setFl_err_import(FL_ERR_IMPORT);
		impFattura.setFl_war_import(FL_WAR_IMPORT);
		
		return impFattura;
	}
	
	/**
	 * @param importoTotaleIt
	 * @param percentualeIva
	 * @return
	 */
	private static double getImportoIva(double importoTotaleIt, double percentualeIva) {
		if (percentualeIva == 0)
			return 0;
		else
			return importoTotaleIt*percentualeIva/100;
	}
	
	/**
	 * @param bilancio
	 * @return
	 */
	private static String getSezionale(String bilancio) {
		return bilancio.substring(0, bilancio.lastIndexOf('.'))+bilancio.substring(bilancio.lastIndexOf('.')+1);
	}
	
	
}
