/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.common.utils;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;

/**
 * @author fulvio
 * 
*
 * 
 * Generic configuration interface that enables the reading/writing of properties on 
 * properties file.
 *
 */
public class FEPropertiesUtil {
	
	private static String PROPERTIES_FILE = "fe-core.properties";
	private static org.apache.commons.configuration.Configuration configuration;
	private static Logger logger = Logger.getLogger(FEPropertiesUtil.class.getName());
	
	private static org.apache.commons.configuration.Configuration getConfiguration() {
		if (configuration == null) {
			try {
				configuration = new PropertiesConfiguration(PROPERTIES_FILE);
			} catch (ConfigurationException e) {
				logger.error(e.getMessage());
			}
		}
		return configuration;
	}
	
	public static String getJndiHost() {
		return getConfiguration().getString("jndi.host");
	}
	
	public static String getJndiPort() {
		return getConfiguration().getString("jndi.port");
	}
	
	public static String getProcessEngineJndiName() {
		return getConfiguration().getString("processengine.jndiname");
	}
	
	public static boolean useMock() {
		String useMock = getConfiguration().getString("mock");
		boolean result = true;
		if (useMock.equals("true"))
			result = true;
		else if (useMock.equals("false"))
			result = false;
		return result;
	}
	
	public static boolean useComoWsMock() {
		return getConfiguration().getBoolean("como.ws.mock");
	}
	
	public static String getFELiferayOrganizationId() {
		return getConfiguration().getString("fe.liferay.organizationid");
	}
	
	public static String getFELiferayGroupId() {
		return getConfiguration().getString("fe.liferay.groupId");
	}
	
	public static boolean isTest() {
		return getConfiguration().getBoolean("test");
	}
	
	public static String getInvoiceManualUploadDir() {
		return getConfiguration().getString("invoice.manual.upload.dir");
	}
	
	public static String getCiaJdbcDriver() {
		return getConfiguration().getString("cia.db.jdbc.driver.oracle");
	}
	
	public static String getCiaDbConnectionUrl() {
		return getConfiguration().getString("cia.db.url");
	}
	
	public static String getCiaDbConnectionPort() {
		return getConfiguration().getString("cia.db.port");
	}
	
	public static String getCiaDbInstance() {
		return getConfiguration().getString("cia.db.instance");
	}
	
	public static String getCiaDbUsername() {
		return getConfiguration().getString("cia.db.username");
	}
	
	public static String getCiaDbPassword() {
		return getConfiguration().getString("cia.db.password");
	}
	
	public static String getMailTransportProtocol() {
		return getConfiguration().getString("mail.transport.protocol");
	}
	
	public static String getMailHost() {
		return getConfiguration().getString("mail.host");
	}
	
	public static int getMailPort() {
		return getConfiguration().getInt("mail.port");
	}
	
	public static String getMailUser() {
		return getConfiguration().getString("mail.user");
	}
	
	public static String getMailPassword() {
		return getConfiguration().getString("mail.password");
	}
	
	public static String getMailFrom() {
		return getConfiguration().getString("mail.from");
	}
	
	public static String getUoDenomination() {
		return getConfiguration().getString("web.portlet.useradministration.uodenomination");
	}
	
	public static boolean useProcessMockAccountableSystem() {
		return getConfiguration().getBoolean("jbpm.active.process.accountable.system.mock");
	}
	
	public static String getActiveProcessName() {
		return getConfiguration().getString("jbpm.active.process.name");
	}
	
	public static String getActiveProcessKey() {
		return getConfiguration().getString("jbpm.active.process.key");
	}
	
	public static String getFeJdbcDriver() {
		return getConfiguration().getString("fe.db.jdbc.driver");
	}
	
	public static String getFeDbUrl() {
		return getConfiguration().getString("fe.db.url");
	}
	
	public static String getFeDbUsername() {
		return getConfiguration().getString("fe.db.username");
	}
	
	public static String getFeDbPassword() {
		return getConfiguration().getString("fe.db.password");
	}
	
	
	public static String getFeTestJdbcDriver() {
		return getConfiguration().getString("feTest.db.jdbc.driver");
	}
	
	public static String getFeTestDbUrl() {
		return getConfiguration().getString("feTest.db.url");
	}
	
	public static String getFeTestDbUsername() {
		return getConfiguration().getString("feTest.db.username");
	}
	
	public static String getFeTestDbPassword() {
		return getConfiguration().getString("feTest.db.password");
	}
	
	public static String getDefaultAdminScreenName() {
		return getConfiguration().getString("default.admin.screen.name");
	}

	public static String getDefaultAdminPassword() {
		return getConfiguration().getString("default.admin.password");
	}

	public static String getPA_Backend_WSDL_URL() {
		return getConfiguration().getString("fe.ws.PA_Backend_WSDL_URL");
	}

	public static String getPA_Login_Backend_WSDL_URL() {
		return getConfiguration().getString("fe.ws.PA_Login_Backend_WSDL_URL");
	}
	
	public static String getInvoiceManagerName() {
		return getConfiguration().getString("fe.invoiceManagerName");
	}
	
}
