/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.common.utils;

import it.fe.core.par.common.PostalAddress;
import it.fe.core.par.invoice.AllowanceCharge;
import it.fe.core.par.invoice.Invoice;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.log4j.Logger;
import org.apache.xml.utils.PrefixResolver;
import org.apache.xml.utils.PrefixResolverDefault;
import org.apache.xpath.NodeSet;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public abstract class XmlParser {
	
	protected static Logger logger = Logger.getLogger(XmlParser.class.getName());
	
	protected static XPath xpath = null;
	
	protected static Document doc = null;
	
	protected static SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
	
//	public abstract void parse(Invoice invoice, Element rootNode, XPath xpath, Document doc, PrintWriter out) throws XPathExpressionException;

	public abstract void parse(Invoice invoice, Element rootNode, XPath xpath, Document doc) throws XPathExpressionException;
	
	protected void setupXmlDocument(XPath xpath, Document doc) {
		
		this.xpath = xpath;
		this.doc = doc;
		
	}
	
	public static NodeList getNodes(Node parentN, String prefix, String nodeName) throws XPathExpressionException{	
		NodeList result = new NodeSet();
		
		if(parentN == null){
			return result;
		}
		if(nodeName == null){
			return result;
		}

		result = (NodeList)xpath.evaluate(nodeName, parentN, XPathConstants.NODESET);
		if(result.getLength() <= 0){
			prefix = prefix + ":";
			
			result = (NodeList)xpath.evaluate(prefix + nodeName, parentN, XPathConstants.NODESET);
		}

	    return result;
	}
	
	public static Node getNode(Node parentN, String prefix, String nodeName) throws XPathExpressionException{
		if(parentN == null){
			return null;
		}
		if(nodeName == null){
			return null;
		}

		Node result = (Node)xpath.evaluate(nodeName, parentN, XPathConstants.NODE);
		if(result == null){
			prefix = prefix + ":";
			result = (Node)xpath.evaluate(prefix + nodeName, parentN, XPathConstants.NODE);
			logger.debug(prefix + nodeName);
		}

	    return result;
	}

	public static String getNodeValue(Element e, String prefix, String nodeName) throws XPathExpressionException{
		if(e == null){
			return null;
		}
		if(nodeName == null){
			if(e.getFirstChild() != null){
				return e.getFirstChild().getNodeValue();
			}
			return null;
		}
		
		//controlla la presenza del namespace
		if(!(xpath.evaluate(nodeName, e).equals(""))) {
			return xpath.evaluate(nodeName, e);
		}
		else if (!xpath.evaluate(prefix + ":" + nodeName, e).equals("")){
			return xpath.evaluate(prefix + ":" + nodeName, e);
		}
		
		return null;
	}

	public static String getNumericNodeValue(Element e, String prefix, String nodeName) throws XPathExpressionException{
		String s = getNodeValue(e, prefix, nodeName);
		if(s == null){
			return "0";
		}
		
		return s;
	}

	public static String getNodeValueByExpr(Document doc, Node parent, Node nodeToFind, Node nodeQuery, String value){
		if(parent == null){
			return null;
		}
		if(nodeToFind == null){
			return null;
		}
		if(nodeQuery == null){
			return null;
		}
		
	    String expression = "[" + nodeQuery.getNodeName() + "='" + value + "']";
	    XPathExpression ex;
		try {
			ex = xpath.compile("//" + parent.getNodeName() + expression + "/" + nodeToFind.getNodeName() + "/text()");
			return (String) ex.evaluate(doc, XPathConstants.STRING);
			
		} catch (XPathExpressionException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	
	//Helper method
//	public static List<AllowanceCharge> readAllowanceCharge(Element element, boolean forInformationOnly, PrintWriter out) throws XPathExpressionException{
	public static List<AllowanceCharge> readAllowanceCharge(Element element, boolean forInformationOnly) throws XPathExpressionException{
		NodeList allowanceChargeNodes = getNodes(element, "cac", "AllowanceCharge");
		List<AllowanceCharge> allowanceChargeList = new ArrayList<AllowanceCharge>();
		
//		out.println("\t<cac:AllowanceCharge>");
		
		for (int z = 0; z < allowanceChargeNodes.getLength(); z++) {
			Element lineAllowanceChargeNode = (Element)allowanceChargeNodes.item(z);
			AllowanceCharge allowanceCharge = new AllowanceCharge();
			
			double lineeAllowanceChargeAmount = Double.parseDouble(getNumericNodeValue(lineAllowanceChargeNode, "cbc", "Amount"));
			
//			out.println("\t\t<cbc:Amount>" + lineeAllowanceChargeAmount + "</cbc:Amount>");
			
			String chargeIndicator = getNumericNodeValue(lineAllowanceChargeNode, "cbc", "ChargeIndicator");
			
//			out.println("\t\t<cbc:ChargeIndicator>" + chargeIndicator + "</cbc:ChargeIndicator>");
			
			if("false".equalsIgnoreCase(chargeIndicator)){
				lineeAllowanceChargeAmount = lineeAllowanceChargeAmount * (-1);
			}
			allowanceCharge.setAmount(lineeAllowanceChargeAmount);
			allowanceCharge.setAllowanceChargeReason(getNodeValue(lineAllowanceChargeNode, "cbc", "AllowanceChargeReason"));
			allowanceCharge.setMultiplierFactorNumeric(Double.parseDouble(getNumericNodeValue(lineAllowanceChargeNode, "cbc", "MultiplierFactorNumeric")));
			allowanceCharge.setBaseAmount(Double.parseDouble(getNumericNodeValue(lineAllowanceChargeNode, "cbc", "BaseAmount")));
			allowanceCharge.setForInformationOnly(forInformationOnly);

//			out.println("\t\t<cbc:AllowanceChargeReason>" + allowanceCharge.getAllowanceChargeReason() + "</cbc:AllowanceChargeReason>");
//			out.println("\t\t<cbc:MultiplierFactorNumeric>" + allowanceCharge.getMultiplierFactorNumeric() + "</cbc:MultiplierFactorNumeric>");
//			out.println("\t\t<cbc:BaseAmount>" + allowanceCharge.getBaseAmount() + "</cbc:BaseAmount>");
			
			allowanceChargeList.add(allowanceCharge);
		}
		
//		out.println("\t</cac:AllowanceCharge>");
		
		return allowanceChargeList;
		
	}
	
//	protected static PostalAddress readAddress(Element addressElement, PrintWriter out) throws XPathExpressionException {
	protected static PostalAddress readAddress(Element addressElement) throws XPathExpressionException {
		PostalAddress address = new PostalAddress();
		address.setStreetName(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "StreetName")));
		address.setBuildingNumber(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "BuildingNumber")));
		address.setFloor(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "Floor")));
		address.setRoom(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "Room")));
		address.setPostalZone(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "PostalZone")));
		address.setCityName(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "CityName")));
		
		Node addressCountryNode = getNode(addressElement, "cac", "Country");
		address.setCountry(SpecialCharacter.xmlEntityToString(getNodeValue((Element)addressCountryNode, "cbc", "IdentificationCode")));
		address.setCountryName(SpecialCharacter.xmlEntityToString(getNodeValue((Element)addressCountryNode, "cbc", "Name")));
		
		address.setPostBox(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "Postbox")));
		address.setAdditionalStreetName(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "AdditionalStreetName")));
		address.setDepartment(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "Department")));
		address.setCountrySubentityCode(SpecialCharacter.xmlEntityToString(getNodeValue(addressElement, "cbc", "CountrySubentityCode")));
		
//		out.println("\t\t\t<cbc:StreetName>" + address.getStreetName() + "</cbc:StreetName>");
//		out.println("\t\t\t<cbc:BuildingNumber>" + address.getBuildingNumber() + "</cbc:BuildingNumber>");
//		out.println("\t\t\t<cbc:Floor>" + address.getFloor() + "</cbc:Floor>");
//		out.println("\t\t\t<cbc:Room>" + address.getRoom() + "</cbc:Room>");
//		out.println("\t\t\t<cbc:CityName>" + address.getCityName() + "</cbc:CityName>");
//		out.println("\t\t\t<cbc:PostalZone>" + address.getPostalZone() + "</cbc:PostalZone>");
//		out.println("\t\t\t<cac:Country>");
//		out.println("\t\t\t\t<cbc:IdentificationCode>" + address.getCountry() + "</cbc:IdentificationCode>");
//		out.println("\t\t\t\t<cbc:Name>" + address.getCountryName() + "</cbc:Name>");
//		out.println("\t\t\t</cac:Country>");
//		out.println("\t\t\t<cbc:Postbox>" + address.getPostBox() + "</cbc:Postbox>");
//		out.println("\t\t\t<cbc:AdditionalStreetName>" + address.getAdditionalStreetName() + "</cbc:AdditionalStreetName>");
//		out.println("\t\t\t<cbc:Department>" + address.getDepartment() + "</cbc:Department>");
//		out.println("\t\t\t<cbc:CountrySubentityCode>" + address.getCountrySubentityCode() + "</cbc:CountrySubentityCode>");
		return address;
	}
	

	public static String readSupplierName(String xml, Element rootNode) {
		// Documento XML
		
		try {
			DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		DocumentBuilder builder;
		Document doc = null;
		domFactory.setNamespaceAware(true);

		
			builder = domFactory.newDocumentBuilder();
			doc = builder
					.parse(new java.io.ByteArrayInputStream(xml.getBytes()));

	

			final PrefixResolver resolver = new PrefixResolverDefault(doc
					.getDocumentElement());
			NamespaceContext ctx = new NamespaceContext() {
				public String getNamespaceURI(String prefix) {
					return resolver.getNamespaceForPrefix(prefix);
				}
	
				public String getPrefix(String arg0) {
					return null;
				}
	
				public Iterator getPrefixes(String arg0) {
					return null;
				}
			};
	
			xpath.setNamespaceContext(ctx);
			rootNode = doc.getDocumentElement();
	
			Node supplierInfoNode = getNode(rootNode, "cac", "AccountingSupplierParty");
			Node supplierPartyNode = getNode(supplierInfoNode, "cac", "Party");
			Node supplierPartyNameNode = getNode(supplierPartyNode, "cac", "PartyName");
			Node supplierPartyLegalEntityNode = getNode(supplierPartyNode, "cac", "PartyLegalEntity");
	
			// SupplierName
			String supplierName = getNodeValue((Element) supplierPartyNameNode, "cbc", "Name");
			supplierName = SpecialCharacter.xmlEntityToString(supplierName);
			
			return supplierName;
		} catch (SAXParseException spe) {
			logger.error("Parse error: " + spe.getMessage());
		} catch (SAXException se) {
			logger.error("SAXException: " + se.getMessage());
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		return "";
	}
}
