/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
*
 *
 */

package it.fe.core.common.utils.db.oracle;

import it.fe.core.common.utils.FEPropertiesUtil;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.sql.PooledConnection;

import oracle.jdbc.driver.OracleConnection;
import oracle.jdbc.pool.OracleConnectionPoolDataSource;

import org.apache.log4j.Logger;


/**
 * @author fulvio di marco
 *
 *	Class providing Oracle connectivity to UniMoRe
 *
 */
public class OracleConnectionFactory {
	
	private static final Logger logger = Logger.getLogger(OracleConnectionFactory.class);
	
	/**
	 * Name of the class that holds the jdbc driver implementation for the hsqldb database
	 */
	public static final String DRIVER = FEPropertiesUtil.getCiaJdbcDriver();
	
	/**
	 * URI of the database to connect to
	 */
	public static final String DBURL = "jdbc:oracle:thin:@"+FEPropertiesUtil.getCiaDbConnectionUrl()+":"+FEPropertiesUtil.getCiaDbConnectionPort()+":"+FEPropertiesUtil.getCiaDbInstance();

	public static final String USERNAME = FEPropertiesUtil.getCiaDbUsername();
	public static final String PASSWORD = FEPropertiesUtil.getCiaDbPassword();
	
	private static OracleConnectionPoolDataSource ocpds;
	private static PooledConnection pooledConnection;

	// --------------------------------------------

	static {
		try {
			Class.forName(DRIVER).newInstance();
			ocpds = new OracleConnectionPoolDataSource();
			ocpds.setURL(DBURL);
			ocpds.setUser(USERNAME);
			ocpds.setPassword(PASSWORD);
			pooledConnection = ocpds.getPooledConnection();
		} 
		catch (Exception e) {
			logger.error(e.getMessage());
			e.printStackTrace();
		}
	}
	
	
	public static OracleConnection createConnection() {
		try {
			OracleConnection conn = (OracleConnection) DriverManager.getConnection (DBURL, USERNAME, PASSWORD);
			return conn;
		} 
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public static void closeConnection(Connection conn) {
		try {
			conn.close();
		}
		catch (Exception e) {
			logger.error("Failed closing connection\n"+e.getMessage());
			e.printStackTrace();
		}
	}
	
	public static Connection getConnection() {
		try {
			return pooledConnection.getConnection();
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}

}
