/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.generic;

import java.util.List;
import java.util.Map;

import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;

public interface CrudManager {

	public <T> T create(T t);

	public <T> T find(Object id, Class<T> type);

	//public <T> void delete(Object id, Class<T> t);
	public void delete(Object t);

	public <T> T update(T t);

	public <T> List<T> findByQuery(String query);

	public <T> List<T> findByQuery(String query,
			Map<String, Object> parameters);

	public <T> List<T> findByQuery(String query, int resultLimit);

	public <T> List<T> findByQuery(String query,
			Map<String, Object> parameters, int resultLimit);

	public <T> List<T> findByQuery(String query,
			Map<String, Object> parameters, int startingRow, int resultLimit);

	/**
	 * Should be invoked by passing a count query 
	 *  
	 * @param namedQueryName
	 * @param parameters
	 * @return
	 */
	public int countItems(String query, Map<String, Object> parameters);
	
	public <T> List<T> findByCriteria(Class<T> clazz,int startRow, int pageSize, List<Criterion> restrictions, Order orderBy) ;
	public int countByCriteria(Class clazz, List<Criterion> restrictions) ;
	
	public <T>List<T> findByDetachedCriteria(int startRow, int pageSize, DetachedCriteria detachedCriteria) ;
	public int countByDetachedCriteria(DetachedCriteria detachedCriteria);

	public <T>List<T> findByNativeQuery(String nativeQuery);
	
	public <T>List<T> findByNativeQuery(String nativeQuery, Class<T> resultClass);
	
	public <T>List<T> findByNativeQuery(String nativeQuery, String resultSetMapping);
	
	public int executeNativeQuery(String nativeQuery);
	
}
