/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.invoice;


import it.fe.core.par.invoice.Client;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.log4j.Logger;

@Stateless
@Remote(ClientManager.class)
public class ClientManagerBean implements ClientManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(ClientManagerBean.class.getName());

	
	public Client getClientByVatId(String vatId) {
		Client client;
		Query query = entityManager.createQuery("SELECT c FROM Client c WHERE c.vatId = :vatId");
		query.setParameter("vatId", vatId);
		
		try {
			logger.info("Find supplier by " + vatId);
			client = (Client) query.getSingleResult();
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			client = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			client = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			client = null;
		}
		
		return client;
	}
	
	public int insertClient(Client client) {
		int result;
		
		try {
			 entityManager.persist(client);
			 result = client.getClientId().intValue();
			 entityManager.flush();
		} catch (EntityExistsException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalStateException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (PersistenceException e) {
			logger.error(e.getMessage());
			result = -1;
		}

		return result;
	}
}
