/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.invoice;

import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.TaxDetail;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;

import java.util.List;
import java.util.Map;

public interface InvoiceManager {
	
	public List<Invoice> getInvoices();
	
	public List<Invoice> getPagedInvoices(int first, int pageSize);
	
	public int findTotalInvoices();
	
	public Invoice getInvoiceById(long id);
	
	public long insertInvoice(Invoice invoice);
	
	public int removeInvoiceById(long id);
	
	public int checkInvoice(Invoice invoice);
	
	public Invoice updateProcessInstanceId(long id, String processInstanceId);
	
	public Invoice updateInvoiceState(long id, InvoiceState state);

	public Invoice findInvoiceByNumberAndVatId(String invoiceNumber, String vatId);
	
	public List<PreviousDocument> fetchPreviousDocuments(Invoice invoice);
	
	public List<OriginalDocument> fetchOriginalDocuments(Invoice invoice);
	
	public List<InvoiceLine> fetchInvoiceLines(Invoice invoice);

	public List<MeterSummary> fetchMeterSummaries(Invoice invoice);
	
	public List<AdditionalData> fetchInvoiceAdditionalData(Invoice invoice);
	
	public List<InvoiceLine> fetchInvoiceLinesWithAdditionalData(Invoice invoice);
	
	public List<AdditionalData> fetchInvoiceLineAdditionalData(InvoiceLine invoiceLine);
	
	public Invoice fetchXMLFile(Invoice invoice);
	
	public List<TaxDetail> fetchTaxDetails(Invoice invoice);
	
	public Invoice updateInvoiceErrorState(long id, boolean isInError, String errorMessage);
	
	public Invoice updateInvoice(Invoice invoice);
	
	public boolean invoiceExists(long id);

	public List<Reading> fetchReadings(Invoice Invoice);

	public List<Object[]> fetchInvoiceLineAccountingCosts(Invoice invoice);
}
