/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.invoice;

import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.Supplier;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityExistsException;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.log4j.Logger;

@Stateless
@Remote(PreviousDocumentManager.class)
public class PreviousDocumentManagerBean implements PreviousDocumentManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	@EJB
	public SupplierManager supplierManager;
	
	
	protected static Logger logger = Logger.getLogger(PreviousDocumentManagerBean.class.getName());
	
	
	public PreviousDocument findContractByDocumentCode(String documentCode, Supplier supplier) {
		PreviousDocument previousDocument;
		String documentType = PreviousDocument.DocumetType.CONTRATTO.toString();
		
		Query query = entityManager.createQuery("SELECT c FROM PreviousDocument c WHERE (c.documentType = ?1 AND c.documentCode = ?2 AND c.supplier = ?3)");
		query.setParameter(1, documentType);		
		query.setParameter(2, documentCode);
		query.setParameter(3, supplier);
		
		try {
			logger.info("Find contract num. " + documentCode);
			previousDocument = (PreviousDocument) query.getSingleResult();
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		}
		
		return previousDocument;
	}

	public PreviousDocument findPreviousDocumentByDocumentCode(String documentCode, Supplier supplier) {		
		PreviousDocument previousDocument;
		
		Query query = entityManager.createQuery("SELECT c FROM PreviousDocument c WHERE c.documentCode = ?1 AND c.supplier = ?2");
		query.setParameter(1, documentCode);
		query.setParameter(2, supplier);
		
		try {
			logger.info("Find previous document num. " + documentCode);
			previousDocument = (PreviousDocument) query.getSingleResult();
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		}
		
		return previousDocument;
	}

	public PreviousDocument findPreviousDocumentById(long id) {
		PreviousDocument previousDocument;
		
		try {
			previousDocument = entityManager.find(PreviousDocument.class, id);
		} catch (NoResultException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		} catch (NonUniqueResultException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		} catch (EntityNotFoundException e) {
			logger.error(e.getMessage());
			previousDocument = null;
		}
		
		return previousDocument;
	}

	public long insertPreviousDocument(PreviousDocument previousDocument) {
		long result = 0;
		Supplier supplierDb;
		
		if(previousDocument.getSupplier() != null){
			supplierDb = supplierManager.getSupplierByVatId(previousDocument.getSupplier().getVatId());	
			if (supplierDb != null) {
				logger.info("Fornitore già presente nel db");
				previousDocument.setSupplier(supplierDb);
			}
			else {
				logger.info("Inserimento nuovo fornitore");
				long supplierId = supplierManager.insertSupplier(previousDocument.getSupplier());
				previousDocument.getSupplier().setSupplierId(supplierId);
			}
		}
		else {
			logger.error("DOCUMENTO SENZA FORNITORE");
		}
		
		try {
			 entityManager.persist(previousDocument);
			 result = previousDocument.getPreviousDocumentId();
			 entityManager.flush();
		} catch (EntityExistsException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalStateException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (IllegalArgumentException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (PersistenceException e) {
			logger.error(e.getMessage());
			result = -1;
		}

		return result;
	}

	public int removePreviousDocumentById(long id) {
		// TODO Auto-generated method stub
		return 0;
	}

	
	
	//--------------- @EJB Getters/Setters --------------------------
	
	public SupplierManager getSupplierManager() {
		return supplierManager;
	}

	public void setSupplierManager(SupplierManager supplierManager) {
		this.supplierManager = supplierManager;
	}

}
