/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.mail;

import it.fe.core.common.utils.FEPropertiesUtil;

import java.util.Hashtable;
import java.util.Map;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.mail.Message;
import javax.mail.Message.RecipientType;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

@Stateless
@Remote(MailService.class)
public class MailServiceBean implements MailService {
	
	private static String hostPort = FEPropertiesUtil.getJndiHost()+":"+FEPropertiesUtil.getJndiPort();
	
	protected static Logger logger = Logger.getLogger(MailServiceBean.class.getName());

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.mail.MailService#sendEmail(java.util.Map)
	 */
	public int sendEmail(Map<String,Object> parameters) {
		
		int result;
		
		try {
			Session mailSession = (Session) getInitialContext(hostPort).lookup("java:/Mail");
			
			MimeMessage message = new MimeMessage(mailSession);
			
			// from
			Object from = parameters.get(MailService.FROM);
			if (from != null && from instanceof String) {
				message.setFrom(new InternetAddress((String) from));
			}
			
			// subject
			Object subject = parameters.get(MailService.SUBJECT);
			if (subject != null && subject instanceof String) {
				message.setSubject((String) subject);
			}
			
			// recipients
			Object recipients = parameters.get(MailService.RECIPIENTS);
			if (recipients != null && recipients instanceof String) {
				message.setRecipients(RecipientType.TO, InternetAddress.parse((String) recipients, false));
			}
			
			// body
			Object body = parameters.get(MailService.BODY);
			if (body != null && body instanceof String) {
				//message.setText((String) body);
				message.setContent((String) body, "text/plain");
			}
			
			message.saveChanges();
			
			Transport transport = mailSession.getTransport(FEPropertiesUtil.getMailTransportProtocol());
			transport.connect(
					FEPropertiesUtil.getMailHost(), 
					FEPropertiesUtil.getMailPort(), 
					FEPropertiesUtil.getMailUser(), 
					FEPropertiesUtil.getMailPassword());
			transport.sendMessage(message, message.getRecipients(Message.RecipientType.TO));
			transport.close();
			result = 1;
		} catch (NamingException e) {
			logger.error(e.getMessage());
			result = -1;
		} catch (Exception e) {
			logger.error(e.getMessage());
			result = -1;
		}
		
		return result;
	}
	
	@SuppressWarnings("unchecked")
	private static InitialContext getInitialContext(String hostPort)
			throws Exception {
		Hashtable props = getInitialContextProperties(hostPort);
		return new InitialContext(props);
	}

	@SuppressWarnings("unchecked")
	private static Hashtable getInitialContextProperties(String hostPort) {
		Hashtable props = new Hashtable();
		props.put("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");
		props.put("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");
		props.put("java.naming.provider.url", hostPort);

		return props;
	}

}
