/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.parer;

import it.fe.core.ejb.parer.wsupload.*;
import it.fe.core.ejb.user.OrganizationManager;
import it.fe.core.ejb.user.OrganizationPropertyManager;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.parer.*;
import it.fe.core.par.parer.TentativoVersamento.Esito;
import it.fe.core.par.user.Organization;
import it.fe.core.common.utils.ParerPropertiesUtil;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.net.ssl.HttpsURLConnection;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.bind.Unmarshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.commons.codec.binary.Base64;
import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;

@Stateless
@Remote(ParerManager.class)
public class ParerManagerBean implements ParerManager {

	static org.apache.commons.configuration.Configuration configuration = ParerPropertiesUtil.getConfiguration();
	
	private final static String restServer = configuration.getString("parerRestServer");
	private final static String uploadDatiFiscaliFatPassiva = configuration.getString("uploadDatiFiscaliFatturazionePassiva");
	private final static String uploadDatiFiscaliFatAttiva = configuration.getString("uploadDatiFiscaliFatturazioneAttiva");
	private final static String uploadFatturaDem = configuration.getString("uploadFatturaDematerializzata");
	private final static String registroRecupero = configuration.getString("registroRecuperoVersamentoFattura");
	private final static String tipoUnitaDocumentaria = configuration.getString("tipoUnitaDocumentariaVersamentoFattura");
	private final static int versioneDatiSpecifici = 1;

	protected static Logger logger = Logger.getLogger(ParerManagerBean.class.getName());
	
	@EJB
	TentativoVersamentoManager tentativoInvioManager;
	
	@EJB
	OrganizationManager organizationManager;
	
	@EJB
	OrganizationPropertyManager organizationPropertyManager;

	public Risposta uploadDatiFiscaliFatturaPassiva(
			Invoice invoice,
			String loginName, 
			String password, 
			String numeroFatturaRecupero, // peppolMessageId 
			int annoRecupero, // issue date da UBL
			String numerFatturaVersamento,
			String registroVersamento,
			Date dataFattura,
			String mittenteDestinatario, // supplier UBL
			String tipoMittenteDestinatario, 
			String supplierId, 
			String tipoSupplierId,
			Organization organization 
			) {
		
		// Storico invii
		TentativoVersamento tentativoInvio = new TentativoVersamento();
		tentativoInvio.setData(new Date());
		tentativoInvio.setInvoice(invoice);
		
		// Da dati utente
		//TODO lanciare eccezione in caso di propertyNotFound
		String ambiente = organizationPropertyManager.getOrganizationPropertyByOrganizationAndLabel(organization, "ambiente").getValue();
		String ente = organizationPropertyManager.getOrganizationPropertyByOrganizationAndLabel(organization, "ente").getValue();
		String struttura = organizationPropertyManager.getOrganizationPropertyByOrganizationAndLabel(organization, "struttura").getValue();
		String participantId = organizationPropertyManager.getOrganizationPropertyByOrganizationAndLabel(organization, "participantId").getValue();
//		String userId = organizationPropertyManager.getOrganizationPropertyByOrganizationAndLabel(organization, "userId").getValue();
		String userId = loginName;
		
		final String tipoDocumento = invoice.getDocumentType().equals("CreditNote") ? configuration.getString("tipoDocumentoVersamentoNotaDiCredito") : configuration.getString("tipoDocumentoVersamentoFattura");

		String peppolUsername = organizationPropertyManager.getOrganizationPropertyByOrganizationAndLabel(organization, "peppolUsername").getValue();
		String peppolPassword = organizationPropertyManager.getOrganizationPropertyByOrganizationAndLabel(organization, "peppolPassword").getValue();
		
		
		try {
			String authString = peppolUsername + ":" + peppolPassword;
			byte[] authEncBytes = Base64.encodeBase64(authString.getBytes());
			String authStringEnc = new String(authEncBytes);
			
			URL url = new URL(restServer + uploadDatiFiscaliFatPassiva);
			HttpURLConnection conn = (HttpsURLConnection) url.openConnection();
			conn.setDoOutput(true);
			conn.setRequestMethod("POST");
			conn.setRequestProperty("Authorization", "Basic "+ authStringEnc);
			conn.setRequestProperty("Content-Type", "application/xml");

			String input = null;
			try {
				DatiFattura datiFattura = createDatiFatturaObject(
						numeroFatturaRecupero, 
						annoRecupero, 
						registroRecupero, 
						registroVersamento, 
						numerFatturaVersamento, 
						tipoDocumento, 
						ambiente, 
						ente, 
						struttura, 
						participantId,
						userId, 
						loginName, 
						password, 
						tipoUnitaDocumentaria, 
						mittenteDestinatario,
						tipoMittenteDestinatario,
						supplierId,
						tipoSupplierId,
						dataFattura,
						invoice,
						false);
				
				input = marshall(datiFattura).toString();
logger.info("XML Versamento:"+input);
				tentativoInvio.setXmlVersamento(input);
				
			} catch (Exception e) {
				e.printStackTrace();
				
				tentativoInvio.setEsito(Esito.NEGATIVO);
				tentativoInvio.setMessaggioErrore("Errore nel marshaling dei DatiFattura");
				tentativoInvio.setStackTrace(e.getLocalizedMessage());
				
				tentativoInvioManager.insert(tentativoInvio);
				
				Risposta risposta = new Risposta();
				EsitoGeneraleType esitoGeneraleType = new EsitoGeneraleType();
				esitoGeneraleType.setEsito(EsitoType.NEGATIVO);
				esitoGeneraleType.setMessaggio("Errore nel marshaling dei DatiFattura");
				risposta.setEsitoGenerale(esitoGeneraleType);
				
				return risposta;
			}

			OutputStream os = conn.getOutputStream();
			os.write(input.getBytes());
			os.flush();

			//			if (conn.getResponseCode() != HttpsURLConnection.HTTP_CREATED) {
			//				throw new RuntimeException("Failed : HTTP error code : " + conn.getResponseCode());
			//			}

			BufferedReader br = new BufferedReader(new InputStreamReader((conn.getInputStream())));

			String output;
			String resEntity = null;

			while ((output = br.readLine()) != null) {
				resEntity = output;
			}
			logger.info("Server Response:\n " + resEntity);

			Risposta esito = null;

			if (resEntity != null) {
				try {
					JAXBContext jaxbContext = JAXBContext.newInstance(Risposta.class);
					Unmarshaller jaxbUnmarshaller = jaxbContext.createUnmarshaller();
					esito = (Risposta) jaxbUnmarshaller.unmarshal(new StringReader(resEntity));
				} catch (JAXBException e) {
					e.printStackTrace();
					
					tentativoInvio.setEsito(Esito.NEGATIVO);
					tentativoInvio.setMessaggioErrore("Errore nell'unmarshaling della risposta di Parer");
					tentativoInvio.setStackTrace(e.getLocalizedMessage());
					
					tentativoInvioManager.insert(tentativoInvio);
					
					Risposta risposta = new Risposta();
					EsitoGeneraleType esitoGeneraleType = new EsitoGeneraleType();
					esitoGeneraleType.setEsito(EsitoType.NEGATIVO);
					esitoGeneraleType.setMessaggio("Errore nell'unmarshaling della risposta di Parer");
					risposta.setEsitoGenerale(esitoGeneraleType);
					
					return risposta;
				}
			}
			else {
				logger.error("No response from Parer");
				
				tentativoInvio.setEsito(Esito.NEGATIVO);
				tentativoInvio.setMessaggioErrore("Nessuna risposta da Parer");
				
				tentativoInvioManager.insert(tentativoInvio);
				
				Risposta risposta = new Risposta();
				EsitoGeneraleType esitoGeneraleType = new EsitoGeneraleType();
				esitoGeneraleType.setEsito(EsitoType.NEGATIVO);
				esitoGeneraleType.setMessaggio("Nessuna risposta da Parer");
				risposta.setEsitoGenerale(esitoGeneraleType);
				
				return risposta;
			}

			logger.info("Esito: " + esito.getEsitoGenerale().getEsito().toString());
			logger.info("Codice esito: " + esito.getEsitoGenerale().getCodiceEsito());
			logger.info("Messaggio: " + esito.getEsitoGenerale().getMessaggio());
			
			conn.disconnect();
			
			tentativoInvio.setCodiceEsito(esito.getEsitoGenerale().getCodiceEsito());
			tentativoInvio.setEsito(Esito.valueOf(esito.getEsitoGenerale().getEsito().toString()));
			tentativoInvio.setMessaggioErrore(esito.getEsitoGenerale().getMessaggio());
			
			tentativoInvioManager.insert(tentativoInvio);
			
			return esito;
			
		} catch (Exception e) {
			logger.error("Errore di connessione");
			e.printStackTrace();
			
			tentativoInvio.setEsito(Esito.NEGATIVO);
			tentativoInvio.setMessaggioErrore("Errore di connessione");
			tentativoInvio.setStackTrace(e.getLocalizedMessage());
			
			tentativoInvioManager.insert(tentativoInvio);
			
			Risposta risposta = new Risposta();
			EsitoGeneraleType esitoGeneraleType = new EsitoGeneraleType();
			esitoGeneraleType.setEsito(EsitoType.NEGATIVO);
			esitoGeneraleType.setMessaggio("Errore di connessione");
			risposta.setEsitoGenerale(esitoGeneraleType);
			
			return risposta;
		}
	}

	public void uploadDatiFiscaliFatturaAttiva() {

	}

	public void uploadFatturaDematerializzata() {

	}

	private StringWriter marshall(DatiFattura datiFattura) throws JAXBException, PropertyException {

		JAXBContext jaxbContext2 = JAXBContext.newInstance(DatiFattura.class);
		Marshaller jaxbMarshaller = jaxbContext2.createMarshaller();
		jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);

		StringWriter xmlSip = new StringWriter();
		jaxbMarshaller.marshal(datiFattura, System.out);
		jaxbMarshaller.marshal(datiFattura, xmlSip);
		return xmlSip;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private DatiFattura createDatiFatturaObject(
			String numeroFatturaRecupero, 
			int annoRecupero, 
			String registroRecupero, 
			String registroVersamento, 
			String numeroFatturaVersamento,  
			String tipoDocumento, 
			String ambiente, 
			String ente, 
			String struttura, 
			String participantId,
			String userId, 
			String loginName, 
			String password, 
			String tipoUnitaDocumentaria, 
			String mittenteDestinatario, 
			String tipoMittenteDestinatario,
			String supplierId,
			String tipoSupplierId,
			Date dataFattura, 
			Invoice invoice,
			boolean fatturaAttiva) throws DatatypeConfigurationException {

		DatiFattura datiFattura = new DatiFattura();

		// CREDENZIALI
		CredenzialiType credenzialiType = new CredenzialiType();
		credenzialiType.setLoginName(loginName);
		credenzialiType.setPassword(password);
		credenzialiType.setPeppolPartecipantId(participantId);

		// VERSATORE
		VersatoreType versatoreType = new VersatoreType();
		versatoreType.setAmbiente(ambiente);
		versatoreType.setEnte(ente);
		versatoreType.setStruttura(struttura);
		versatoreType.setUserId(userId);

		credenzialiType.setVersatore(versatoreType);
		datiFattura.setCredenziali(credenzialiType);

		// CHIAVE RECUPERO
		ChiaveType chiaveType = new ChiaveType();
		chiaveType.setAnno(annoRecupero);
		chiaveType.setNumero(numeroFatturaRecupero);
		chiaveType.setTipoRegistro(registroRecupero);
		datiFattura.setChiaveRecupero(chiaveType);

		// CHIAVE VERSAMENTO
		ChiaveVersamentoType chiaveVersamentoType = new ChiaveVersamentoType();
		chiaveVersamentoType.setNumeroFattura(numeroFatturaVersamento);
		chiaveVersamentoType.setRegistro(registroVersamento);
		datiFattura.setChiaveVersamento(chiaveVersamentoType);

		datiFattura.setMittenteDestinatario(mittenteDestinatario);

		GregorianCalendar c = new GregorianCalendar();
		c.setTime(dataFattura);
		XMLGregorianCalendar date = null;
		date = DatatypeFactory.newInstance().newXMLGregorianCalendar(c);
		datiFattura.setDataFattura(date);

		datiFattura.setTipoDocumento(tipoDocumento);
		datiFattura.setTipologiaUnitaDocumentaria(tipoUnitaDocumentaria);

		// DATI SPECIFICI / FISCALI
		DatiSpecificiType datiSpecificiType = new DatiSpecificiType();

		if(fatturaAttiva){
			JAXBElement<String> VersioneDatiSpecifici = new JAXBElement(QName.valueOf("VersioneDatiSpecifici"), Integer.class, versioneDatiSpecifici);
			JAXBElement<String> FA_NumeroEmissione = new JAXBElement(QName.valueOf("FA_NumeroEmissione"), String.class, "1");
			JAXBElement<String> FA_RegistroEmissione = new JAXBElement(QName.valueOf("FA_RegistroEmissione"), String.class, "RegistroEmissione");
			JAXBElement<String> FA_DataEmissione = new JAXBElement(QName.valueOf("FA_DataEmissione"), String.class, new SimpleDateFormat("yyyy-MM-dd").format(new Date()));
			JAXBElement<String> FA_DenominazioneDestinatario = new JAXBElement(QName.valueOf("FA_DenominazioneDestinatario"), String.class, "Regione Sociale Destinatario");
			JAXBElement<String> FA_TipoDenominazioneDestinatario = new JAXBElement(QName.valueOf("FA_TipoDenominazioneDestinatario"), String.class, "RagioneSociale");
			JAXBElement<String> FA_IdentificativoDestinatario = new JAXBElement(QName.valueOf("FA_IdentificativoDestinatario"), String.class, "123123123");
			JAXBElement<String> FA_TipoIdentificativoDestinatario = new JAXBElement(QName.valueOf("FA_TipoIdentificativoDestinatario"), String.class, "PIVA");

			datiSpecificiType.getAny().add(VersioneDatiSpecifici);
			datiSpecificiType.getAny().add(FA_NumeroEmissione);
			datiSpecificiType.getAny().add(FA_RegistroEmissione);
			datiSpecificiType.getAny().add(FA_DataEmissione);
			datiSpecificiType.getAny().add(FA_DenominazioneDestinatario);
			datiSpecificiType.getAny().add(FA_TipoDenominazioneDestinatario);
			datiSpecificiType.getAny().add(FA_IdentificativoDestinatario);
			datiSpecificiType.getAny().add(FA_TipoIdentificativoDestinatario);
		}
		else{
			JAXBElement<String> VersioneDatiSpecifici = new JAXBElement(QName.valueOf("VersioneDatiSpecifici"), Integer.class, versioneDatiSpecifici);
			JAXBElement<String> FP_NumeroRegistrazione = new JAXBElement(QName.valueOf("FP_NumeroRegistrazione"), String.class, numeroFatturaVersamento);
			JAXBElement<String> FP_DataRegistrazione = new JAXBElement(QName.valueOf("FP_DataRegistrazione"), String.class, new SimpleDateFormat("yyyy-MM-dd").format(dataFattura));
			JAXBElement<String> FP_Registro = new JAXBElement(QName.valueOf("FP_Registro"), String.class, registroVersamento);
			JAXBElement<String> FP_NumeroEmissione = new JAXBElement(QName.valueOf("FP_NumeroEmissione"), String.class, invoice.getInvoiceNumber());
			JAXBElement<String> FP_DataEmissione = new JAXBElement(QName.valueOf("FP_DataEmissione"), String.class, new SimpleDateFormat("yyyy-MM-dd").format(invoice.getIssueDate()));
			JAXBElement<String> FP_DenominazioneMittente = new JAXBElement(QName.valueOf("FP_DenominazioneMittente"), String.class, mittenteDestinatario);
			JAXBElement<String> FP_TipoDenominazioneMittente = new JAXBElement(QName.valueOf("FP_TipoDenominazioneMittente"), String.class, tipoMittenteDestinatario);
			JAXBElement<String> FP_IdentificativoMittente = new JAXBElement(QName.valueOf("FP_IdentificativoMittente"), String.class, supplierId);
			JAXBElement<String> FP_TipoIdentificativoMittente = new JAXBElement(QName.valueOf("FP_TipoIdentificativoMittente"), String.class, tipoSupplierId);

			datiSpecificiType.getAny().add(VersioneDatiSpecifici);
			datiSpecificiType.getAny().add(FP_NumeroRegistrazione);
			datiSpecificiType.getAny().add(FP_DataRegistrazione);
			datiSpecificiType.getAny().add(FP_Registro);
			datiSpecificiType.getAny().add(FP_NumeroEmissione);
			datiSpecificiType.getAny().add(FP_DataEmissione);
			datiSpecificiType.getAny().add(FP_DenominazioneMittente);
			datiSpecificiType.getAny().add(FP_TipoDenominazioneMittente);
			datiSpecificiType.getAny().add(FP_IdentificativoMittente);
			datiSpecificiType.getAny().add(FP_TipoIdentificativoMittente);
		}

		JAXBElement<DatiSpecificiType> datiSpecificiTypeE = new JAXBElement(QName.valueOf("DatiSpecifici"), DatiSpecificiType.class, datiSpecificiType);
		datiFattura.setDatiSpecifici(datiSpecificiTypeE);

		return datiFattura;
	}
}