/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.parer;

import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.parer.TentativoVersamento;
import it.fe.core.par.parer.TentativoVersamento.Esito;

import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

@Stateless
@Remote(TentativoVersamentoManager.class)
public class TentativoVersamentoManagerBean implements TentativoVersamentoManager {
	
	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	private EntityManager entityManager;
	
	@Override
	public Long insert(TentativoVersamento tentativoVersamento) {
		try {
			this.entityManager.persist(tentativoVersamento);
			return tentativoVersamento.getId();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return -1L;
	}

	@Override
	public TentativoVersamento findById(Long id) {
		try {
			return this.entityManager.find(TentativoVersamento.class, id);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public TentativoVersamento update(TentativoVersamento tentativoVersamento) {
		try {
			return this.entityManager.merge(tentativoVersamento);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public int remove(TentativoVersamento tentativoVersamento) {
		try {
			this.entityManager.remove(tentativoVersamento);
			return 1;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return -1;
	}

	@Override
	public List<TentativoVersamento> findAll() {
		try {
			return this.entityManager.createQuery("FROM TentativoVersamento").getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public List<TentativoVersamento> findByInvoiceId(Invoice invoice) {
		try {
			return this.entityManager.createQuery("FROM TentativoVersamento n WHERE n.invoice = :invoice").
					setParameter("invoice", invoice).getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	@Override
	public Boolean isArchived(Invoice invoice) {
		try {
			List<TentativoVersamento> tentativi = this.entityManager.createQuery("FROM TentativoVersamento n WHERE n.invoice = :invoice AND n.esito = :esito").
					setParameter("invoice", invoice).setParameter("esito", Esito.POSITIVO).getResultList();
			if ((tentativi == null) || (tentativi.size() < 1) )
				return Boolean.FALSE;
			else 
				return Boolean.TRUE;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return Boolean.FALSE;
	}
}