/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.process.generic;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.process.ProcessManager;
import it.fe.core.ejb.process.ProcessServiceFacade;
import it.fe.core.par.invoice.Invoice;

import javax.ejb.Remote;
import javax.ejb.Stateless;

@Stateless
@Remote(ProcessManager.class)
public class GenericProcessManagerBean implements ProcessManager {

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#approveTechnically(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String approveTechnically(Invoice invoice, String username) {
		
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_APPROVE, ProcessManager.STATE_TECHNICAL_VALIDATION, username);
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#discardAdditionalData(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String discardAdditionalData(Invoice selectedInvoice, String username) {
		throw new UnsupportedOperationException("This functionality is not active for generic process");
	}

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#editAdditionalData(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String editAdditionalData(Invoice selectedInvoice, String username) {
		throw new UnsupportedOperationException("This functionality is not active for generic process");
	}

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#insertIntoAccountableSystem(long, java.lang.String)
	 */
	public Object[] insertIntoAccountableSystem(long invoiceId, String username) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#isEditable(it.fe.core.par.invoice.Invoice)
	 */
	public boolean isEditable(Invoice invoice) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#rejectTechnically(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String rejectTechnically(Invoice invoice, String username) {
		
		String result = ProcessManager.FAILURE;
		
		String processDefinitionKey = FEPropertiesUtil.getActiveProcessKey();
		String businessKey = invoice.getSupplier().getSupplierName() + "-" + invoice.getInvoiceNumber();
		
		ProcessServiceFacade processService = JNDIUtils.getProcessServiceFacade();
		
		if (processService != null) {
			
			result = processService.signal(processDefinitionKey, businessKey, ProcessManager.ACTION_REJECT, ProcessManager.STATE_TECHNICAL_VALIDATION, username);
			
		}
		else {
			result = ProcessManager.FAILURE;
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see it.fe.core.ejb.process.ProcessManager#saveAdditionalData(it.fe.core.par.invoice.Invoice, java.lang.String)
	 */
	public String saveAdditionalData(Invoice selectedInvoice, String username) {
		throw new UnsupportedOperationException("This functionality is not active for generic process");
	}

	public String approveInsertionIntoAccountableSystem(Invoice invoice,
			String username) {
		// TODO Auto-generated method stub
		return null;
	}

}
