/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.process.performance;

import it.fe.core.ejb.invoice.InvoiceManager;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.process.performance.TaskTiming;

import java.util.Date;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.apache.log4j.Logger;

@Stateless
@Remote(ProcessPerformanceManager.class)
public class ProcessPerformanceManagerBean implements ProcessPerformanceManager {
	protected static Logger logger = Logger
			.getLogger(ProcessPerformanceManagerBean.class.getName());

	@PersistenceContext(name = "FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;

	@EJB
	public InvoiceManager invoiceManager;

	public TaskTiming addTaskTiming(Long invoiceId, String processDescriptionId,
			String processExecutionId, String taskName, Date startDate) {

		Invoice invoice = invoiceManager.getInvoiceById(invoiceId);
		if (invoice == null) {
			logger.error("addTaskTiming: no invoice with id:" + invoiceId);
			return null;
		}
		TaskTiming t = new TaskTiming();
		t.setInvoice(invoice);
		t.setProcessDescriptionId(processDescriptionId);
		t.setProcessExecutionId(processExecutionId);
		t.setTaskName(taskName);
		t.setInvoiceState(invoice.getInvoiceState());
		t.setStartTime(startDate);

		entityManager.persist(t);
		
		return t;

	}

	public TaskTiming updateTaskTimingEndDate(Long invoiceId,
			String processExecutionId, String taskName, String username,
			Date endDate) {
		
		Invoice invoice = invoiceManager.getInvoiceById(invoiceId);
		if (invoice == null) {
			logger.error("updateTaskTimingEndDate: no invoice with id:" + invoiceId);
			return null;
		}
		
		Query q = entityManager
				.createQuery("from TaskTiming t " +
						"where t.invoice.invoiceId=:invoiceId " +
						"and t.invoiceState.invoiceStateId=:invoiceStateId " +
						"and t.processExecutionId=:processExecutionId " +
						"and t.taskName=:taskname");
		q.setParameter("invoiceId", invoiceId);
		q.setParameter("invoiceStateId", invoice.getInvoiceState().getInvoiceStateId());
		q.setParameter("processExecutionId", processExecutionId);
		q.setParameter("taskname", taskName);
		
		List<TaskTiming> timings = q.getResultList();
		if (timings == null)  { 
			logger.error("updateTaskTimingEndDate: no timings found");
			return null;
		}
		if ( timings.size() != 1) { 
			logger.error("updateTaskTimingEndDate: wrong number of timings: "+timings.size());
			return null;
		}
		
		TaskTiming t = timings.get(0);
		t.setEndTime(endDate);
		t.setUsername(username);
		entityManager.merge(t);
		entityManager.flush();
		return t;
	}

	public TaskTiming findById(Long timingId) {
		return entityManager.find(TaskTiming.class, timingId);
	}
}
