/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.sdi;

import it.fe.core.par.sdi.ScartoEsitoCommittente;

import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

@Stateless
@Remote(ScartoEsitoCommittenteManager.class)
public class ScartoEsitoCommittenteManagerBean implements ScartoEsitoCommittenteManager {

	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	private EntityManager entityManager;
	
	@Override
	public Long insert(ScartoEsitoCommittente scarto) {
		try {
			this.entityManager.persist(scarto);
			return scarto.getId();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return -1L;
	}

	@Override
	public ScartoEsitoCommittente findById(Long id) {
		try {
			return this.entityManager.find(ScartoEsitoCommittente.class, id);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public ScartoEsitoCommittente update(ScartoEsitoCommittente scarto) {
		try {
			return this.entityManager.merge(scarto);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public int remove(ScartoEsitoCommittente scarto) {
		try {
			this.entityManager.remove(scarto);
			return 1;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return -1;
	}

	@Override
	public List<ScartoEsitoCommittente> findAll() {
		try {
			return this.entityManager.createQuery("FROM ScartoEsitoCommittente").getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public List<ScartoEsitoCommittente> findByIdentificativoSdi(String identificativoSdi) {
		try {
			return this.entityManager.createQuery("FROM ScartoEsitoCommittente n WHERE n.identificativoSdi = :identificativoSdi").
					setParameter("identificativoSdi", identificativoSdi).getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public List<ScartoEsitoCommittente> findByMessageId(String messageId) {
		try {
			return this.entityManager.createQuery("FROM ScartoEsitoCommittente n WHERE n.messageId = :messageId").
					setParameter("messageId", messageId).getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public List<ScartoEsitoCommittente> findByMessageIdCommittente(String messageIdCommittente) {
		try {
			return this.entityManager.createQuery("FROM ScartoEsitoCommittente n WHERE n.messageIdCommittente = :messageIdCommittente").
					setParameter("messageIdCommittente", messageIdCommittente).getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public List<ScartoEsitoCommittente> findByPecMessageId(String pecMessageId) {
		try {
			return this.entityManager.createQuery("FROM ScartoEsitoCommittente n WHERE n.pecMessageId = :pecMessageId").
					setParameter("pecMessageId", pecMessageId).getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public List<ScartoEsitoCommittente> findByNumeroFattura(String numeroFattura) {
		try {
			return this.entityManager.createQuery("FROM ScartoEsitoCommittente n WHERE n.numeroFattura = :numeroFattura").
					setParameter("numeroFattura", numeroFattura).getResultList();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

}
