/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.user;

import it.fe.core.par.user.Organization;

import java.util.logging.Logger;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@Stateless
@Remote(OrganizationManager.class)
public class OrganizationManagerBean implements OrganizationManager {
	
	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(OrganizationManagerBean.class.getName());
	
	public Organization getOrganizationByUoID(String uoId) {
		Organization organization;
		Query query = entityManager.createQuery("SELECT u FROM Organization u WHERE u.uoId = :uoId");
		query.setParameter("uoId", uoId);
		try {
			organization = (Organization) query.getSingleResult();
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			organization = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			organization = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			organization = null;
		}
		return organization;
	}
	
	public Organization getOrganizationByName(String name) {
		Organization organization;
		Query query = entityManager.createQuery("SELECT u FROM Organization u WHERE u.name = :name");
		query.setParameter("name", name);
		try {
			organization = (Organization) query.getSingleResult();
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			organization = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			organization = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			organization = null;
		}
		return organization;
	}

	public Organization getUserById(long id) {
		Organization organization;
		try {
			organization = entityManager.find(Organization.class, id);
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			organization = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			organization = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			organization = null;
		}
		
		return organization;
	}
}