/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.ejb.user;

import it.fe.core.par.user.Organization;
import it.fe.core.par.user.OrganizationProperty;

import java.util.List;
import java.util.logging.Logger;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@Stateless
@Remote(OrganizationPropertyManager.class)
public class OrganizationPropertyManagerBean implements OrganizationPropertyManager {
	
	@PersistenceContext(name="FatturazioneElettronicaPersistenceUnit")
	EntityManager entityManager;
	
	protected static Logger logger = Logger.getLogger(OrganizationPropertyManagerBean.class.getName());
	
	/* (non-Javadoc)
	 * @see it.fe.core.ejb.user.OrganizationPropertyManager#getOrganizationPropertyByOrganization(it.fe.core.par.user.Organization)
	 */
	public List<OrganizationProperty> getOrganizationPropertiesByOrganization(Organization organization) {
		List<OrganizationProperty> organizationProperties;
		Query query = entityManager.createQuery("SELECT u FROM OrganizationProperty u WHERE u.organization = :organization");
		query.setParameter("organization", organization);
		try {
			organizationProperties =  query.getResultList();
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			organizationProperties = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			organizationProperties = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			organizationProperties = null;
		}
		return organizationProperties;
	}
	
	/* (non-Javadoc)
	 * @see it.fe.core.ejb.user.OrganizationPropertyManager#getOrganizationPropertyByOrganization(it.fe.core.par.user.Organization, java.lang.String)
	 */
	public OrganizationProperty getOrganizationPropertyByOrganizationAndLabel(Organization organization, String label) {
		OrganizationProperty organizationProperty;
		Query query = entityManager.createQuery("SELECT u FROM OrganizationProperty u WHERE u.organization = :organization and u.label = :label");
		query.setParameter("organization", organization);
		query.setParameter("label", label);
		try {
			organizationProperty =  (OrganizationProperty) query.getSingleResult();
		} catch (NoResultException e) {
			logger.severe(e.getMessage());
			organizationProperty = null;
		} catch (NonUniqueResultException e) {
			logger.severe(e.getMessage());
			organizationProperty = null;
		} catch (EntityNotFoundException e) {
			logger.severe(e.getMessage());
			organizationProperty = null;
		}
		return organizationProperty;
	}

	
}
