/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.common;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.Table;

/**
 * La classe <b><i>File</i></b> contiene tutte le informazioni relative ai file legati alla fattura.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="Files")
public class FEFile implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * Numero progressivo assegnato al file
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long fileId;
	
	/**
	 * Blob del file. 
	 * Si è scelto di utilizzare un array di byte (piuttosto che un Blob Type) 
	 * per non cablare la struttura del database nel modello.
	 * 
	 * La dimensione massima del Blob è fissata a 100MB
	 */
	@Lob
	@Column(length=1024 * 1024 * 100)
	private byte[] fileBlob;
	
	private String fileName;
	
	private String fileMimeType;
	
	private String fileInfo;
	
	
	//--------------- Ctors --------------------------
	
	public FEFile() {
		super();
	}

	public FEFile(byte[] fileBlob, String fileName) {
		this.fileBlob = fileBlob;
		this.fileName = fileName;
	}
	
	
	
	//--------------- Getters/Setters --------------------------

	/**
	 * @return the fileId
	 */
	public Long getFileId() {
		return fileId;
	}

	/**
	 * @param fileId the fileId to set
	 */
	public void setFileId(Long fileId) {
		this.fileId = fileId;
	}

	/**
	 * @return the fileBlob
	 */
	public byte[] getFileBlob() {
		return fileBlob;
	}

	/**
	 * @param fileBlob the fileBlob to set
	 */
	public void setFileBlob(byte[] fileBlob) {
		this.fileBlob = fileBlob;
	}

	/**
	 * @return the fileName
	 */
	public String getFileName() {
		return fileName;
	}

	/**
	 * @param fileName the fileName to set
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	/**
	 * @return the fileMimeType
	 */
	public String getFileMimeType() {
		return fileMimeType;
	}

	/**
	 * @param fileMimeType the fileMimeType to set
	 */
	public void setFileMimeType(String fileMimeType) {
		this.fileMimeType = fileMimeType;
	}

	/**
	 * @return the fileInfo
	 */
	public String getFileInfo() {
		return fileInfo;
	}

	/**
	 * @param fileInfo the fileInfo to set
	 */
	public void setFileInfo(String fileInfo) {
		this.fileInfo = fileInfo;
	}
}
