/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.compliance;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

/**
 * La classe <b><i>TranslationType</i></b> mappa l'elenco delle tipologie di traduzioni (da un formato x a UBL 2) gestite da FE.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="TranslationTypes")
public class TranslationType implements Serializable {
	
	private static final long serialVersionUID = 1L;

	/**
	 * Identificativo
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long translationTypeId;
	
	/**
	 * Descrizione del traduttore (es. HERA_txt, UBL_xml...)
	 */
	@Column(nullable=false)
	private String description;

	
	//--------------- Ctors --------------------------
	
	public TranslationType() {
		super();
	}
	

	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the translationTypeId
	 */
	public long getTranslationTypeId() {
		return translationTypeId;
	}

	/**
	 * @param translationTypeId the translationTypeId to set
	 */
	public void setTranslationTypeId(long translationTypeId) {
		this.translationTypeId = translationTypeId;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}
}
