/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.invoice;

import it.fe.core.par.utility.Contact;

import java.io.Serializable;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;

/**
 * La classe <b><i>AllowanceCharge</i></b> contiene le informazioni relative a sconti, abbuoni, premi applicati sulla riga di una fattura.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="AllowanceCharges")
public class AllowanceCharge implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long allowanceChargeId;
	
	/**
	 * Ammontare del carico
	 */
	private double amount;
	
	/**
	 * Quantità utilizzata, in combinazione con il fattore di moltiplicazione numerica, 
	 * per calcolare l'importo del carico
	 */
	private double baseAmount;
	
	/**
	 * Fattore di moltiplicazione numerica
	 */
	private double multiplierFactorNumeric;
	
	/**
	 * Motivazione: sconto, premio, abbuono...
	 */
	private String allowanceChargeReason;
	
	
	/**
	 * True se non deve essere utilizzato per calcolare i totali
	 * False altrimenti
	 */
	private boolean forInformationOnly;
	
	/**
	 * Riferimento ai Tax Detail
	 */
	@OneToMany( fetch=FetchType.EAGER, cascade=CascadeType.ALL )
	@JoinColumn(name="allowanceChargeId", referencedColumnName="allowanceChargeId")
	private List<TaxDetail> taxDetailList;

	
	
	//--------------- Ctors --------------------------
	
	public AllowanceCharge() {
		super();
	}

	/**
	 * @param allowanceChargeId
	 * @param chargeIndicator
	 * @param amount
	 * @param allowanceChargeReason
	 */
	public AllowanceCharge(double amount, String allowanceChargeReason) {
		super();
		this.amount = amount;
		this.allowanceChargeReason = allowanceChargeReason;
	}


	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the allowanceChargeId
	 */
	public long getAllowanceChargeId() {
		return allowanceChargeId;
	}


	/**
	 * @param allowanceChargeId the allowanceChargeId to set
	 */
	public void setAllowanceChargeId(long allowanceChargeId) {
		this.allowanceChargeId = allowanceChargeId;
	}

	/**
	 * @return the amount
	 */
	public double getAmount() {
		return amount;
	}


	/**
	 * @param amount the amount to set
	 */
	public void setAmount(double amount) {
		this.amount = amount;
	}


	/**
	 * @return the baseAmount
	 */
	public double getBaseAmount() {
		return baseAmount;
	}


	/**
	 * @param baseAmount the baseAmount to set
	 */
	public void setBaseAmount(double baseAmount) {
		this.baseAmount = baseAmount;
	}


	/**
	 * @return the multiplierFactorNumeric
	 */
	public double getMultiplierFactorNumeric() {
		return multiplierFactorNumeric;
	}


	/**
	 * @param multiplierFactorNumeric the multiplierFactorNumeric to set
	 */
	public void setMultiplierFactorNumeric(double multiplierFactorNumeric) {
		this.multiplierFactorNumeric = multiplierFactorNumeric;
	}


	/**
	 * @return the allowanceChargeReason
	 */
	public String getAllowanceChargeReason() {
		return allowanceChargeReason;
	}


	/**
	 * @param allowanceChargeReason the allowanceChargeReason to set
	 */
	public void setAllowanceChargeReason(String allowanceChargeReason) {
		this.allowanceChargeReason = allowanceChargeReason;
	}

	/**
	 * @param forInformationOnly the forInformationOnly to set
	 */
	public void setForInformationOnly(boolean forInformationOnly) {
		this.forInformationOnly = forInformationOnly;
	}

	/**
	 * @return the forInformationOnly
	 */
	public boolean isForInformationOnly() {
		return forInformationOnly;
	}
}
