/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.invoice;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * La classe <b><i>InvoicePeriod</i></b> contiene le informazioni relative al periodo fatturato.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="InvoicePeriods")
public class InvoicePeriod implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long invoicePeriodId;
	
	/**
	 * Descizione del periodo di consumo (es. I bimestre).
	 */
	private String periodDescription;
	
	/**
	 * Codice del periodo di consumo.
	 */
	private String periodCode;
	
	/**
	 * Data inizio periodo di consumo.
	 */
	@Temporal(TemporalType.DATE)
	private Date startDate;
	
	/**
	 * Data fine periodo di consumo.
	 */
	@Temporal(TemporalType.DATE)
	private Date endDate;


	
	//--------------- Ctors --------------------------
	
	public InvoicePeriod() {
		super();
	}

	/**
	 * @param periodDescription
	 * @param periodCode
	 * @param startDate
	 * @param endDate
	 */
	public InvoicePeriod(String periodDescription,
			String periodCode, Date startDate, Date endDate) {
		super();
		this.periodDescription = periodDescription;
		this.periodCode = periodCode;
		this.startDate = startDate;
		this.endDate = endDate;
	}


	
	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the invoicePeriodId
	 */
	public long getInvoicePeriodId() {
		return invoicePeriodId;
	}

	/**
	 * @param invoicePeriodId the invoicePeriodId to set
	 */
	public void setInvoicePeriodId(long invoicePeriodId) {
		this.invoicePeriodId = invoicePeriodId;
	}

	/**
	 * @return the periodDescription
	 */
	public String getPeriodDescription() {
		return periodDescription;
	}

	/**
	 * @param periodDescription the periodDescription to set
	 */
	public void setPeriodDescription(String periodDescription) {
		this.periodDescription = periodDescription;
	}

	/**
	 * @return the periodCode
	 */
	public String getPeriodCode() {
		return periodCode;
	}

	/**
	 * @param periodCode the periodCode to set
	 */
	public void setPeriodCode(String periodCode) {
		this.periodCode = periodCode;
	}

	/**
	 * @return the startDate
	 */
	public Date getStartDate() {
		return startDate;
	}

	/**
	 * @param startDate the startDate to set
	 */
	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	/**
	 * @return the endDate
	 */
	public Date getEndDate() {
		return endDate;
	}

	/**
	 * @param endDate the endDate to set
	 */
	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}
}
