/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.par.invoice;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

@Entity
@Table(name="InvoiceStates")
public class InvoiceState implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Transient
	public static final String RECEIVED = "Ricevuta";
	@Transient
	public static final String TECHNICALLY_VALIDATED = "Validata tecnicamente";
	@Transient
	public static final String TECHNICALLY_SIGNED = "Firmata dal Responsabile Tecnico";
	@Transient
	public static final String ACCOUNTABLY_VALIDATED = "Validata contabilmente";
	@Transient
	public static final String ACCOUNTABLY_SIGNED = "Firmata dal Responsabile Contabile";
	@Transient
	public static final String ACCOUNTED = "Contabilizzata";
	@Transient
	public static final String TECHNICALLY_REJECTED = "Rifiutata al controllo tecnico";
	@Transient
	public static final String ACCOUNTABLY_REJECTED = "Rifiutata al controllo contabile";
	@Transient
	public static final String NOT_ACCOUNTED = "Non contabilizzata";
	@Transient
	public static final String ACCOUNTABLE_SYSTEM_INSERTED = "Inserita nel sistema contabile";
	
	@Transient
	public static final String REQUIRES_ARCHIVING = "DaConservare";

	
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long invoiceStateId;
	
	private String description;
	
	@Temporal(TemporalType.DATE)
	private Date startDate;
	
	@Temporal(TemporalType.DATE)
	private Date endDate;

	
	public InvoiceState() {
		super();
	}
	

	public long getInvoiceStateId() {
		return invoiceStateId;
	}

	public void setInvoiceStateId(long invoiceStateId) {
		this.invoiceStateId = invoiceStateId;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((endDate == null) ? 0 : endDate.hashCode());
		result = prime * result
				+ ((startDate == null) ? 0 : startDate.hashCode());
		result = prime * result
				+ ((description == null) ? 0 : description.hashCode());
		result = prime * result
				+ (int) (invoiceStateId ^ (invoiceStateId >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		InvoiceState other = (InvoiceState) obj;
		if (endDate == null) {
			if (other.endDate != null)
				return false;
		} else if (!endDate.equals(other.endDate))
			return false;
		if (startDate == null) {
			if (other.startDate != null)
				return false;
		} else if (!startDate.equals(other.startDate))
			return false;
		if (description == null) {
			if (other.description != null)
				return false;
		} else if (!description.equals(other.description))
			return false;
		if (invoiceStateId != other.invoiceStateId)
			return false;
		return true;
	}

}
