/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.invoice;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * La classe <b><i>PaymentDetail</i></b> contiene le informazioni relative al pagamento della fattura.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="PaymentDetails")
public class PaymentDetail implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long paymentDetailId;
	
	/**
	 * Data di scadenza della fattura.
	 */
	@Temporal(TemporalType.DATE)
	private Date paymentDueDate;
	
	/**
	 * Descrizione metodo di pagamento (RID, Bonifico, Rimessa diretta...).
	 */
	private String paymentMeansDescription;
	
	/**
	 * Codice metodo di pagamento.
	 */
	private String paymentMeansCode;
	
	/**
	 * Termini di pagamento.
	 */
	@Lob
	private String paymentTerms;
	
	/**
	 * Codice della coda.
	 */
	private String paymentCodeId;

	
	//TODO Gestione dei dati bancari di riferimento
	
	
	
	
	//--------------- Ctors --------------------------
	
	public PaymentDetail() {
		super();
	}

	/**
	 * @param paymentDueDate
	 * @param paymentMeansDescription
	 * @param paymentMeansCode
	 */
	public PaymentDetail(Date paymentDueDate,
			String paymentMeansDescription, String paymentMeansCode) {
		super();
		this.paymentDueDate = paymentDueDate;
		this.paymentMeansDescription = paymentMeansDescription;
		this.paymentMeansCode = paymentMeansCode;
	}
	
	
	
	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the paymentDetailId
	 */
	public long getPaymentDetailId() {
		return paymentDetailId;
	}

	/**
	 * @param paymentDetailId the paymentDetailId to set
	 */
	public void setPaymentDetailId(long paymentDetailId) {
		this.paymentDetailId = paymentDetailId;
	}

	/**
	 * @return the paymentDueDate
	 */
	public Date getPaymentDueDate() {
		return paymentDueDate;
	}

	/**
	 * @param paymentDueDate the paymentDueDate to set
	 */
	public void setPaymentDueDate(Date paymentDueDate) {
		this.paymentDueDate = paymentDueDate;
	}

	/**
	 * @return the paymentMeansDescription
	 */
	public String getPaymentMeansDescription() {
		return paymentMeansDescription;
	}

	/**
	 * @param paymentMeansDescription the paymentMeansDescription to set
	 */
	public void setPaymentMeansDescription(String paymentMeansDescription) {
		this.paymentMeansDescription = paymentMeansDescription;
	}

	/**
	 * @return the paymentMeansCode
	 */
	public String getPaymentMeansCode() {
		return paymentMeansCode;
	}

	/**
	 * @param paymentMeansCode the paymentMeansCode to set
	 */
	public void setPaymentMeansCode(String paymentMeansCode) {
		this.paymentMeansCode = paymentMeansCode;
	}

	/**
	 * @param paymentTerms the paymentTerms to set
	 */
	public void setPaymentTerms(String paymentTerms) {
		this.paymentTerms = paymentTerms;
	}

	/**
	 * @return the paymentTerms
	 */
	public String getPaymentTerms() {
		return paymentTerms;
	}

	/**
	 * @return the paymentCodeId
	 */
	public String getPaymentCodeId() {
		return paymentCodeId;
	}

	/**
	 * @param paymentCodeId the paymentCodeId to set
	 */
	public void setPaymentCodeId(String paymentCodeId) {
		this.paymentCodeId = paymentCodeId;
	}

}
