/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.invoice;

import it.fe.core.par.common.FEFile;

import java.io.Serializable;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.Table;

/**
 * La classe <b><i>PreviousDocument</i></b> contiene le informazioni relative ai documenti di riferimento della fattura. 
 * (es. contratto, ordine...)
 * 
 * <br /><br />
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="PreviousDocuments")
public class PreviousDocument implements Serializable {
	
	private static final long serialVersionUID = 1L;

	public enum DocumetType { CONTRATTO };
	
	/**
	 * Identificativo.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private long previousDocumentId;
	
	/**
	 * Tipologia di documento di riferimento (es. contratto, ordine...).
	 */
	private String documentType;
	
	/**
	 * Codice identificativo del documento (es. num. contratto).
	 */
	private String documentCode;
	
	/**
	 * Informazioni sul documento.
	 */
	private String documentInfo;
	
	/**
	 * URI del documento.
	 */
	private String documentURI;
	
	/**
	 * Riferimento al documento
	 */
	@OneToOne( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="fileId", referencedColumnName="fileId")
	private FEFile embeddedDocument;
	
	
	/**
	 * Riferimento al fornitore.
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="supplierId", referencedColumnName="supplierId")
	private Supplier supplier;
	
	
	/**
	 * Riferimento alle fatture.
	 * Si è scelto di mantenere un riferimento alla lista di fatture 
	 * per facilitare eventuali interrogazioni specifiche relative ad esempio
	 * alle previsioni o al controllo di gestione.
	 */
	@ManyToMany(mappedBy="previousDocuments")
	private List<Invoice> invoices;

	
	
	//--------------- Ctors --------------------------
	
	public PreviousDocument() {
		super();
	}

	/**
	 * @param documentType
	 * @param documentCode
	 */
	public PreviousDocument(String documentType, String documentCode) {
		super();
		this.documentType = documentType;
		this.documentCode = documentCode;
	}

	
	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the previousDocumentId
	 */
	public long getPreviousDocumentId() {
		return previousDocumentId;
	}

	/**
	 * @param previousDocumentId the previousDocumentId to set
	 */
	public void setPreviousDocumentId(long previousDocumentId) {
		this.previousDocumentId = previousDocumentId;
	}

	/**
	 * @return the documentType
	 */
	public String getDocumentType() {
		return documentType;
	}

	/**
	 * @param documentType the documentType to set
	 */
	public void setDocumentType(String documentType) {
		this.documentType = documentType;
	}

	/**
	 * @return the documentCode
	 */
	public String getDocumentCode() {
		return documentCode;
	}

	/**
	 * @param documentCode the documentCode to set
	 */
	public void setDocumentCode(String documentCode) {
		this.documentCode = documentCode;
	}

	/**
	 * @param documentInfo the documentInfo to set
	 */
	public void setDocumentInfo(String documentInfo) {
		this.documentInfo = documentInfo;
	}

	/**
	 * @return the documentInfo
	 */
	public String getDocumentInfo() {
		return documentInfo;
	}

	/**
	 * @param documentURI the documentURI to set
	 */
	public void setDocumentURI(String documentURI) {
		this.documentURI = documentURI;
	}

	/**
	 * @return the documentURI
	 */
	public String getDocumentURI() {
		return documentURI;
	}

	/**
	 * @param embeddedDocument the embeddedDocument to set
	 */
	public void setEmbeddedDocument(FEFile embeddedDocument) {
		this.embeddedDocument = embeddedDocument;
	}

	/**
	 * @return the embeddedDocument
	 */
	public FEFile getEmbeddedDocument() {
		return embeddedDocument;
	}


	/**
	 * @param supplier the supplier to set
	 */
	public void setSupplier(Supplier supplier) {
		this.supplier = supplier;
	}

	/**
	 * @return the supplier
	 */
	public Supplier getSupplier() {
		return supplier;
	}

	/**
	 * @return the invoices
	 */
	public List<Invoice> getInvoices() {
		return invoices;
	}

	/**
	 * @param invoices the invoices to set
	 */
	public void setInvoices(List<Invoice> invoices) {
		this.invoices = invoices;
	}

}
