/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.par.user;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

@Entity
@Table(name="OrganizationProperties", uniqueConstraints={@UniqueConstraint(columnNames={"organizationId", "label"})})
public class OrganizationProperty   implements Serializable {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long organizationPropertyId;
	
	private String value;
	
	@Column(name="label")
	private String label;
	
	@ManyToOne
	@JoinColumn(name="organizationId", referencedColumnName="organizationId", nullable=false)
	private Organization organization;

	public Long getOrganizationPropertyId() {
		return organizationPropertyId;
	}

	public void setOrganizationPropertyId(Long organizationPropertyId) {
		this.organizationPropertyId = organizationPropertyId;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public Organization getOrganization() {
		return organization;
	}

	public void setOrganization(Organization organization) {
		this.organization = organization;
	}

	public OrganizationProperty(Long organizationPropertyId, String value,
			String label, Organization organization) {
		super();
		this.organizationPropertyId = organizationPropertyId;
		this.value = value;
		this.label = label;
		this.organization = organization;
	}
	
	public OrganizationProperty() {
		super();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((label == null) ? 0 : label.hashCode());
		result = prime * result
				+ ((organization == null) ? 0 : organization.hashCode());
		result = prime
				* result
				+ ((organizationPropertyId == null) ? 0
						: organizationPropertyId.hashCode());
		result = prime * result + ((value == null) ? 0 : value.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		OrganizationProperty other = (OrganizationProperty) obj;
		if (label == null) {
			if (other.label != null)
				return false;
		} else if (!label.equals(other.label))
			return false;
		if (organization == null) {
			if (other.organization != null)
				return false;
		} else if (!organization.equals(other.organization))
			return false;
		if (organizationPropertyId == null) {
			if (other.organizationPropertyId != null)
				return false;
		} else if (!organizationPropertyId.equals(other.organizationPropertyId))
			return false;
		if (value == null) {
			if (other.value != null)
				return false;
		} else if (!value.equals(other.value))
			return false;
		return true;
	}
}