/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.utility;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * La classe <b><i>Sector</i></b> contiene le informazioni relative ai settori merceologici.
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="Sectors")
public class Sector implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Numero progressivo assegnato al settore merceologico.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long sectorId;
	
	/**
	 * Descrizione del settore merceologico (es. acqua)
	 */
	@Column(nullable=false)
	private String sectorName;
	
	/**
	 * Data inizio validità settore
	 */
	@Temporal(TemporalType.DATE)
	@Column(nullable=false)
	private Date startDate;
	
	/**
	 * Data fine validità settore
	 */
	@Temporal(TemporalType.DATE)
	private Date endDate;

	
	
	//--------------- Ctors --------------------------
	
	public Sector() {
		super();
	}

	public Sector(String sectorName, Date startDate, Date endDate) {
		super();
		this.sectorName = sectorName;
		this.startDate = startDate;
		this.endDate = endDate;
	}


	//--------------- Getters/Setters --------------------------
	
	public Long getSectorId() {
		return sectorId;
	}

	public void setSectorId(Long sectorId) {
		this.sectorId = sectorId;
	}

	public String getSectorName() {
		return sectorName;
	}

	public void setSectorName(String sectorName) {
		this.sectorName = sectorName;
	}

	public Date getStartDate() {
		return startDate;
	}

	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}

	public Date getEndDate() {
		return endDate;
	}

	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

}
