/**
 * This file is part of FatturazioneElettronica - Signature and timestamp facilities
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Signature and timestamp facilities is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Signature and timestamp facilities is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Signature and timestamp facilities.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.signature;

import it.trento.comune.j4sign.verification.RootsVerifier;
import it.trento.comune.j4sign.verification.VerifyResult;
import it.trento.comune.j4sign.verification.servlet.ProcessedItem;
import it.trento.comune.j4sign.verification.servlet.Verifier;

import java.util.Hashtable;
import java.util.Iterator;

import javax.security.auth.x500.X500Principal;

/**
 * 
 *
 * 
 */

public class SignatureVerificationUtils {


	//TODO: modificare
	public static String confPath = System.getProperty("user.home")+"/FE-SERVER/signature/conf";

	// DigitPA fingerprint string
	public static String fingerprintString = "20EA4FB83593DB5C4407A538C920EBC3C5B1559C";

	public static String printVerificationResultHTML(
			TimestampVerificationResult timestampVerificationResult) {

		java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("dd-MM-yyyy hh:mm");
		StringBuffer buffer = new StringBuffer();
		buffer.append("<div style='font-family: sans-serif; background: #d5d3d3; width: 900px; border-style: solid; border-width:2px; border-color: #bab2b2; margin: 10px; padding: 5px;  overflow: hidden; text-overflow: ellipsis;'>");
		buffer.append("<p style='text-align: justify;'>"); 
		buffer.append("<h2>Timestamp content : "
				+ ((timestampVerificationResult.getValidationResult()) ? "<font color='green'>OK</font>"
						: "<font color='red'>ERROR</font>"));
		buffer.append("</h2><br/>");
		buffer.append("<h2>Timestamp : <font color='green'>"+sdf.format(timestampVerificationResult.getTimestamp())+"</font></h2><br/>");

		if ((timestampVerificationResult.getErrorMessages() != null)
				&& (timestampVerificationResult.getErrorMessages().size() > 0)) {
			buffer.append("<h3>Errors</h3>");
			for (String error : timestampVerificationResult.getErrorMessages()) {
				buffer.append(error);
			}
		}
		buffer.append("</p></div>");
		return buffer.toString();
	}

	public static String printVerificationResultHTML(ProcessedItem pi) {

		StringBuffer buffer = new StringBuffer();
		buffer.append("<div style='font-family: sans-serif; background: #d5d3d3; width: 900px; border-style: solid; border-width:2px; border-color: #bab2b2; margin: 10px; padding: 5px;  overflow: hidden; text-overflow: ellipsis;'>");
		buffer.append("<p style='text-align: justify;'>"); 
		buffer.append("<h2>Signature verification: " + ((pi.isVerified()) ? "<font color='green'>OK</font>"
				: "<font color='red'>ERROR</font>"));
		buffer.append("</h2><br/>");

		Hashtable<X500Principal, VerifyResult> results = pi.getVerifyResults();
		if ((results != null) && (results.keys() != null)) {
			for (X500Principal principal : results.keySet()) {
				buffer.append("<b>Firmatario:</b> " + principal.getName("RFC1779") + "<br/>");
				VerifyResult result = results.get(principal);
				buffer.append("<b>Certificatore: </b>"
						+ result.getCert().getIssuerDN().toString() + "<br/><br/>");
				buffer.append("<b>Verifica firma:</b> ");
				buffer.append( result.isPassed()? "<font color='green'>OK</font>" : "<font color='red'>ERROR</font>");
				buffer.append("<br/>");
				buffer.append("<b>Verifica integrità:</b> ");
				buffer.append( result.isSignatureCorrect()? "<font color='green'>OK</font>" : "<font color='red'>ERROR</font>");
				buffer.append("<br/>");
				buffer.append("<b>Verifica percorso certificazione: </b>");
				buffer.append( result.isPathValid()? "<font color='green'>OK</font>" : "<font color='red'>ERROR</font>");
				buffer.append("<br/>");
				
			}
		}

		if ((pi.getErrors() != null) && (pi.getErrors().size() > 0)) {
			buffer.append("<h3>Errors</h3>");
			for (String error : pi.getErrors())
				buffer.append(error);
		}
		buffer.append("</p></div>");
		return buffer.toString();
	}

	public static void printVerificationResult(ProcessedItem pi) {
		if ((pi.getErrors() == null) || (pi.getErrors().size() == 0)) {
			System.out.println("Overall: OK");
			Hashtable<X500Principal, VerifyResult> results = pi
					.getVerifyResults();
			if ((results != null) && (results.keys() != null)) {
				for (X500Principal principal : results.keySet()) {
					System.out.println("Firmatario: " + principal.getName());
					VerifyResult result = results.get(principal);
					System.out.println("Verifica firma: " + result.isPassed());
					System.out.println("Verifica integrità: "
							+ result.isSignatureCorrect());
					System.out.println("Verifica percorso certificazione: "
							+ result.isPathValid());
					System.out.println("Certificatore: "
							+ result.getCert().getIssuerDN().toString());

				}
			}

		} else {
			System.out.println("Overall: ERROR");
			for (String error : pi.getErrors()) {
				System.out.println("\t" + error);
			}
		}
	}

	public static ProcessedItem processFile(byte[] file) {
		String baseFileName = null;
		ProcessedItem pi = new ProcessedItem("fileDiProva", false);
		// Inizio verifica
		RootsVerifier rv;
		Verifier v = null;
		try {
			rv = RootsVerifier.getInstance(confPath,
					decodeHex(fingerprintString.toCharArray()));

			v = new Verifier(file, false, rv);

		} catch (Exception e) {
			e.printStackTrace();
			pi.getErrors().add(
					"Errore nell'inizializzazione della verifica: "
							+ e.getMessage());
		}

		if (v != null) {
			while (!v.isDone())
				v.verify();

			if (!v.getErrors().isEmpty())
				pi.getErrors().addAll(v.getErrors());

			Hashtable<X500Principal, VerifyResult> r = v.getRisultati();

			if (r != null) {
				Iterator<X500Principal> principalIterator = r.keySet()
						.iterator();

				boolean isVerified = false;
				int i = 0;
				while (principalIterator.hasNext()) {
					X500Principal signer = principalIterator.next();
					VerifyResult sv = r.get(signer);
					isVerified = sv.isPassed() && ((i == 0) || isVerified);

				}

				pi.setVerifyResults(r);
				pi.setVerified(isVerified);

			}
		}
		return pi;

	}

	public static byte[] decodeHex(char[] data) throws Exception {

		int len = data.length;

		if ((len & 0x01) != 0) {
			throw new Exception("Odd number of characters.");
		}

		byte[] out = new byte[len >> 1];

		// two characters form the hex value.
		for (int i = 0, j = 0; j < len; i++) {
			int f = toDigit(data[j], j) << 4;
			j++;
			f = f | toDigit(data[j], j);
			j++;
			out[i] = (byte) (f & 0xFF);
		}

		return out;
	}

	protected static int toDigit(char ch, int index) throws Exception {
		int digit = Character.digit(ch, 16);
		if (digit == -1) {
			throw new Exception("Illegal hexadecimal character " + ch
					+ " at index " + index);
		}
		return digit;
	}

}
