/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.administration;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.generic.NestedCriterion;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.process.exception.JbpmProcessException;
import it.fe.web.common.DateUtils;
import it.fe.web.common.JSFUtils;
import it.fe.web.common.ProfileUtils;
import it.fe.web.controller.generic.FilterableHandler;
import it.fe.web.controller.invoice.InvoiceInfoHandler;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;

public class AdministrationHandler extends FilterableHandler {

	protected Invoice selectedInvoice;
	public Invoice shownInvoice;
	protected List<Meter> shownInvoiceUtilityInfo;	
	protected String invoiceNumber;
	protected String supplierName;
	protected Date dateBegin;
	protected Date dateEnd;
	protected Date receiptDateBegin;
	protected Date receiptDateEnd;
	protected boolean editable;
	public  List<String> responsibilities; 
	private Logger logger = Logger.getLogger(AdministrationHandler.class.getName());
	
	
	public List<String> getResponsibilities() {
		return responsibilities;
	}

	private void retrieveResponsibilities() {
		if (selectedInvoice != null){
			responsibilities= new ArrayList<String>();
			
			Map<String, List<String>> resps = AdministrationUtils.determineResponsibility(JNDIUtils.getInvoiceStateManager().getInvoiceStates(), selectedInvoice);
			if (resps!= null){
				String toBeAdded = "";
				for (String state : resps.keySet()) {
					toBeAdded+= state.toUpperCase()+ " - " ;
					for (String user : resps.get(state)) {
						toBeAdded+=user+",";
					}
				}
				responsibilities.add(toBeAdded.substring(0,toBeAdded.length()-1));
			}
		}
		
	}

	public Invoice getSelectedInvoice() {
		return selectedInvoice;
	}

	public void setSelectedInvoice(Invoice selectedInvoice) {
		this.selectedInvoice = selectedInvoice;
	}

	public Invoice getShownInvoice() {		
		return shownInvoice;
	}

	public void setShownInvoice(Invoice shownInvoice) {
		this.shownInvoice = shownInvoice;
	}

	public List<Meter> getShownInvoiceUtilityInfo() {
		return shownInvoiceUtilityInfo;
	}

	public void setShownInvoiceUtilityInfo(List<Meter> shownInvoiceUtilityInfo) {
		this.shownInvoiceUtilityInfo = shownInvoiceUtilityInfo;
	}
	
	public String getInvoiceNumber() {
		return invoiceNumber;
	}

	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}

	public String getSupplierName() {
		return supplierName;
	}

	public void setSupplierName(String supplierName) {
		this.supplierName = supplierName;
	}

	public Date getDateBegin() {
		return dateBegin;
	}

	public void setDateBegin(Date dateBegin) {
		this.dateBegin = dateBegin;
	}

	public Date getDateEnd() {
		return dateEnd;
	}

	public void setDateEnd(Date dateEnd) {
		this.dateEnd = dateEnd;
	}

	public boolean isEditable() {
		return editable;
	}

	public void setEditable(boolean editable) {
		this.editable = editable;
	}

	public Date getReceiptDateBegin() {
		return receiptDateBegin;
	}

	public void setReceiptDateBegin(Date receiptDateBegin) {
		this.receiptDateBegin = receiptDateBegin;
	}

	public Date getReceiptDateEnd() {
		return receiptDateEnd;
	}

	public void setReceiptDateEnd(Date receiptDateEnd) {
		this.receiptDateEnd = receiptDateEnd;
	}

	/**
	 * Classe del modello di dominio su cui opera la portlet
	 */
	@Override
	protected Class determineClass() {
		return Invoice.class;
	}

	/**
	 * Questo metodo determina gli effettivi criteri di filtraggio a partire dai criteri di filtro impostati dall'utente lato Web
	 */
	@Override
	protected DetachedCriteria determineDetachedCriteria() {
		return ProfileUtils.computeDetachedCriteria(determineOrder(), super.order, determineClass(), determineWebRestrictions());
//		Order orderCriteria = ;
//		boolean orderAdded = (orderCriteria == null); //se non ho ordinamento, devo ignorarlo nelle successive fasi.
//		DetachedCriteria out = DetachedCriteria.forClass(determineClass());
//
//		// Mappa proprietà valori i criteri del profilo
//		Map<String, List<Criterion>> criteriWeb = determineWebRestrictions();
//
//		// Ciclo sui criteri web e creo il criterio finale
//		if ((criteriWeb != null) && (criteriWeb.keySet() != null))
//			for (String property : criteriWeb.keySet()) {
//				List<Criterion> criteri = criteriWeb.get(property);
//				if ((criteri == null) || (criteri.size() == 0))
//					continue;
//
//				else if (criteri.get(0) instanceof NestedCriterion) {
//					DetachedCriteria propertyCriteria = out
//							.createCriteria(((NestedCriterion) criteri.get(0))
//									.getOuterPropertyName());
//
//					String outerClass = ((NestedCriterion) criteri.get(0))
//							.getOuterPropertyName();
//					/**
//					 * Devo verificare la presenza di criteri di ordinamento su
//					 * una proprietà innestata e aggiungerli al criterio
//					 * opportuno in fase di creazione. Hibernate criteria,
//					 * infatti, non consente di creare due criteri distinti per
//					 * la stessa proprietà (es. uno per le restrictions e uno
//					 * per l'order) E' quindi necessario creare gli ordinamenti
//					 * innestati in maniera contestuale alla creazione della
//					 * medesima proprietà innestata
//					 * 
//					 * il codice sotto è analogo a quello già scritto nella
//					 * sezione precedente per la gestione dei filtri di profilo:
//					 * abbiamo garanzia di non aggiungere due volte lo stesso
//					 * ordinamento perché questo secondo ciclo itera solo sulle
//					 * proprietà web rimaste fuori dalla precedente join con i
//					 * criteri di profilo
//					 */
//
//					// Se sto aggiungendo un sottocriterio sul quale è stato
//					// anche richiesto l'ordinamento...
//					if ((!orderAdded) && (super.order != null)
//							&& (orderCriteria != null)
//							&& (orderCriteria instanceof NestedOrder)
//							&& (((NestedOrder) orderCriteria)
//									.getOuterPropertyName().equals(outerClass))) {
//						String innerProperty = ((NestedOrder) orderCriteria)
//								.getInnerPropertyName();
//						boolean asc = ((NestedOrder) orderCriteria)
//								.isAscending();
//						Order newOrder = (asc) ? Order.asc(innerProperty)
//								: Order.desc(innerProperty);
//						propertyCriteria.addOrder(newOrder);
//						orderAdded=true;
//					}
//					if (criteri.size() == 1)
//						propertyCriteria.add(((NestedCriterion) criteri.get(0))
//								.getInnerCriterion());
//					else {
//						Conjunction c = Restrictions.conjunction();
//						for (Criterion criterion : criteri) {
//							c.add(((NestedCriterion) criterion)
//									.getInnerCriterion());
//						}
//						propertyCriteria.add(c);
//					}
//				} else {
//					if (criteri.size() == 1)
//						out.add(criteri.get(0));
//					else {
//						Conjunction conjunction = Restrictions.conjunction();
//						for (Criterion criterion : criteri) {
//							conjunction.add(criterion);
//						}
//						out.add(conjunction);
//					}
//				}
//			}
//		//Infine aggiungo l'ordinamento (solo se questo non è di tipo nested, perché in tal caso è stato gestito a monte: vedere sopra)
//		if ( (!orderAdded) &&  (super.order != null) && (orderCriteria != null)  ){
//			if (! (orderCriteria instanceof NestedOrder)){
//					out.addOrder(orderCriteria);
//			}
//			else{
//				String innerProperty = ((NestedOrder)orderCriteria).getInnerPropertyName();
//				String outerProperty = ((NestedOrder)orderCriteria).getOuterPropertyName();
//				boolean asc = ((NestedOrder)orderCriteria).isAscending();
//				Order newOrder = (asc)?Order.asc(innerProperty):Order.desc(innerProperty);
//				out.createCriteria(outerProperty ).addOrder(newOrder);
//			}
//		}
//		return out;
	}

	/**
	 * Questo metodo determina le restrizioni lato web (filtri impostati dall'utente)
	 */
	@Override
	protected Map<String, List<Criterion>> determineWebRestrictions() {
		Map<String, List<Criterion>> out = new HashMap<String, List<Criterion>>();

		if ((this.invoiceNumber != null) && (!this.invoiceNumber.equals("")))
			out.put("invoiceNumber", new ArrayList(Arrays
					.asList(new Criterion[] { Restrictions.ilike(
							"invoiceNumber", this.invoiceNumber,
							MatchMode.START) })));

		// TODO: attenzione: è necessario garantire consistenza tra il nome del
		// filtro lato web e lato profilo!!!
		// Altrimenti la successiva fase di join dei cirteri non funziona
		if ((this.supplierName != null) && (!this.supplierName.equals("")))
			out.put("supplier.supplierName", new ArrayList(Arrays
					.asList(new Criterion[] { new NestedCriterion("supplier",
							Restrictions.ilike("supplierName", supplierName,
									MatchMode.START)) })));

		if ((this.dateBegin != null) && (this.dateEnd != null))
			out.put("paymentDetail.paymentDueDate", new ArrayList(Arrays
					.asList(new Criterion[] { new NestedCriterion(
							"paymentDetail", Restrictions.between(
									"paymentDueDate", DateUtils.beginOfDay(dateBegin), DateUtils.endOfDay(dateEnd))) })));
		else if (this.dateBegin != null)
			out.put("paymentDetail.paymentDueDate", new ArrayList(Arrays
					.asList(new Criterion[] { new NestedCriterion(
							"paymentDetail", Restrictions.ge("paymentDueDate",
									DateUtils.beginOfDay(dateBegin))) })));
		else if (this.dateEnd != null)
			out.put("paymentDetail.paymentDueDate", new ArrayList(Arrays
					.asList(new Criterion[] { new NestedCriterion(
							"paymentDetail", Restrictions.le("paymentDueDate",
									DateUtils.endOfDay(dateEnd))) })));

		if ((this.receiptDateBegin!= null) && (this.receiptDateBegin!= null))
			out.put("receiptDate",new ArrayList(Arrays.asList(new Criterion[]{Restrictions.between("receiptDate", DateUtils.beginOfDay(receiptDateBegin),DateUtils.endOfDay(receiptDateEnd))})));
		else if (this.receiptDateBegin!= null)
			out.put("receiptDate",new ArrayList(Arrays.asList(new Criterion[]{Restrictions.ge("receiptDate",DateUtils.beginOfDay(receiptDateBegin))})));
		else if (this.receiptDateEnd!= null)	
			out.put("receiptDate",new ArrayList(Arrays.asList(new Criterion[]{Restrictions.le("receiptDate",DateUtils.endOfDay(receiptDateEnd))})));
	
		return out;
	}

	@Override
	public void resetSearchParameters() {
		this.invoiceNumber = null;
		this.supplierName= null;
		this.dateBegin= null;
		this.dateEnd= null;
		this.receiptDateBegin= null;
		this.receiptDateEnd= null;

	}
	
	public String openDetail() {
		this.selectedInvoice = (Invoice) dataModel.getRowData();
		this.fetchLazyCollections((Invoice) dataModel.getRowData());
		this.updateEditable();
		InvoiceInfoHandler invoiceInfoHandler = (InvoiceInfoHandler) JSFUtils.getManagedBean("invoiceInfoHandler");
		invoiceInfoHandler.setMeterSummariesList(selectedInvoice.getMeterSummary());
		invoiceInfoHandler.setPreviousDocumentList(selectedInvoice.getPreviousDocuments());
		return "openDetail";
	}	
	
	public void utilityInvoiceInfo(){
		this.shownInvoice = (Invoice) dataModel.getRowData();

		List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(shownInvoice);
		this.shownInvoice.setPreviousDocuments(documents);

	}
	
	private void fetchLazyCollections(Invoice selectedInvoice) {
		List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(selectedInvoice);
		List<InvoiceLine> lines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
		List<MeterSummary> summaries = JNDIUtils.getInvoiceManager().fetchMeterSummaries(selectedInvoice);
		this.selectedInvoice.setPreviousDocuments(documents);
		this.selectedInvoice.setInvoiceLines(lines);
		this.selectedInvoice.setMeterSummary(summaries);
	}
	
	public void updateEditable() {
		try {
			this.editable = JNDIUtils.getProcessManager().isEditable(selectedInvoice);
		} catch (JbpmProcessException e) {
			logger.error(e.getMessage());
			this.editable = false;
		}
	}
	
}
