/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.download;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.PeppolValidation;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.Invoice;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.apache.fop.apps.FOUserAgent;
import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;
import org.apache.fop.fo.FOElementMapping;
import org.apache.fop.fo.extensions.ExtensionElementMapping;
import org.apache.fop.fo.extensions.svg.BatikExtensionElementMapping;
import org.apache.fop.fo.extensions.svg.SVGElementMapping;
import org.apache.fop.fo.extensions.xmp.RDFElementMapping;
import org.apache.fop.fo.extensions.xmp.XMPElementMapping;
import org.apache.fop.render.ps.extensions.PSExtensionElementMapping;
import org.w3c.dom.Document;

public class DownloadServlet extends HttpServlet {
	FopFactory fopFactory = FopFactory.newInstance();
	TransformerFactory tFactory = TransformerFactory.newInstance();

	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {

		HttpSession sess = request.getSession();
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

		try {

			String fileName = "";
			if (request.getParameter("item") == null)
				return;
			else if (request.getParameter("item").equals("ReceiptDocument")) {
				if (request.getParameter("id") == null)
					return;
				List result = JNDIUtils.getCrudManager().findByQuery(
						"from FEFile where fileId="
								+ request.getParameter("id"));
				if ((result != null) && (result.size() > 0)) {

					FEFile doc = (FEFile) result.get(0);
					outputStream.write(doc.getFileBlob());
					fileName = doc.getFileName();
				} else
					return;

			}

			else if (request.getParameter("item").equals("PeppolValidation")) {
				if (request.getParameter("id") == null)
					return;

				List<PeppolValidation> validations = JNDIUtils.getCrudManager()
						.findByQuery(
								"from PeppolValidation where id = "
										+ request.getParameter("id"));
				if (validations != null) {
					byte[] bytes = new byte[] {};

					if ((request.getParameter("type") != null)
							&& (request.getParameter("type").equals("HTML"))) {
						bytes = validations.get(0).getHtmlFile();
						fileName = validations.get(0) + ".html";
					} else if ((request.getParameter("type") != null)
							&& (request.getParameter("type").equals("XML"))) {
						bytes = validations.get(0).getXmlFile();
						fileName = validations.get(0) + ".xml";
					}

					outputStream.write(bytes);

				} else
					return;
			}
			
			/*
			 * questa routine serve a visualizzare file XML-UBL di fatture, 
			 * ad esempio fatture provenienti da pacchetti m7m, processati all'atto della ricezione e 
			 * già spacchettati automaticamente in una serie di FEFile collegati ad un OriginalDocument
			 */
			else if (request.getParameter("item").equals("InvoiceFromXML")) {
				if (request.getParameter("id") == null)
					return;
				List result = JNDIUtils.getCrudManager().findByQuery(
						"from FEFile where fileId="
								+ request.getParameter("id"));
				if ((result != null) && (result.size() > 0)) {
					FEFile doc = (FEFile) result.get(0);
					fileName = doc.getFileName();
					byte[] bytes = new byte[] {};
					if ((request.getParameter("type") != null)
							&& (request.getParameter("type").equals("PDF"))){
						bytes = generatePDF(doc.getFileBlob());
						response.setHeader("Content-disposition", "attachment; filename=\""
							+ fileName+".pdf" + "\"");
					}
					else if ((request.getParameter("type") != null)
							&& (request.getParameter("type").equals("HTML"))){
						bytes = generateHTML(doc.getFileBlob());
						response.setHeader("Content-disposition", "inline; filename=\""
							+ fileName+".html" + "\"");
					}
					else{
						bytes = doc.getFileBlob();
						response.setHeader("Content-disposition", "attachment; filename=\""
								+ fileName+".xml" + "\"");
					}
					outputStream.write(bytes);
					fileName = doc.getFileName();
				} else
					return;
			}
			
			else if (request.getParameter("item").equals("Invoice")) {
				if (request.getParameter("id") == null)
					return;
				// List result =
				// JNDIUtils.getCrudManager().findByQuery("from Invoice where invoiceId="+request.getParameter("id"));
				Invoice invoice = JNDIUtils.getInvoiceManager().getInvoiceById(
						Long.parseLong(request.getParameter("id")));
				if (invoice != null) {
					invoice = JNDIUtils.getInvoiceManager().fetchXMLFile(invoice);
					fileName = invoice.getInvoiceNumber();
					byte[] bytes = new byte[] {};

					if ((request.getParameter("type") != null)
							&& (request.getParameter("type").equals("PDF"))){
						bytes = generatePDF(invoice);
						response.setHeader("Content-disposition", "attachment; filename=\""
								+ fileName+".pdf" + "\"");
					}
					else if ((request.getParameter("type") != null)
							&& (request.getParameter("type").equals("HTML"))){
						bytes = generateHTML(invoice);
					response.setHeader("Content-disposition", "inline; filename=\""
							+ fileName+".html" + "\"");
					}
					else if ((request.getParameter("type") != null)
							&& (request.getParameter("type").equals("XML"))){
						bytes = invoice.getXmlFile().getFileBlob();
						response.setHeader("Content-disposition", "attachment; filename=\""
								+ fileName+".xml" + "\"");
					}

					outputStream.write(bytes);
				} else
					return;

			}

			ServletOutputStream outputJSP = response.getOutputStream();
			DataOutputStream output = new DataOutputStream(outputJSP);


			// output contents

			byte[] bytes = outputStream.toByteArray();
			response.setContentLength(bytes.length);
			for (int i = 0; i < bytes.length; i++) {
				output.writeByte(bytes[i]);
			}
		} catch (FileNotFoundException e) {
			System.out.println("Download failed, file not found");
		} catch (UnsupportedEncodingException e) {
			System.out.println("Download failed, unsupported encoding");
		} catch (Exception e) {
			System.out.println("Download failed, I/O error:");
			e.printStackTrace();
		}
	}

	private byte[] generateHTML(Invoice invoice) {
		return generateHTML(invoice.getXmlFile().getFileBlob());
	}
	
	private byte[] generateHTML(byte[] invoice) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		TransformerFactory tFactory = TransformerFactory.newInstance();
		try {
			String path = getServletContext().getRealPath("/rendering/");
			
//			Transformer transformer = tFactory
//					.newTransformer(new javax.xml.transform.stream.StreamSource(new File(path,
//							"CraneUBL2UN380Invoice-html-EN.xsl")));

			DocumentBuilderFactory builderFactory =
			        DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = builderFactory.newDocumentBuilder();
			Document xmlDocument = builder.parse(new ByteArrayInputStream(invoice));
					
			String rootElement = xmlDocument.getDocumentElement().getNodeName();
			System.out.println("Root element: "+ rootElement);
			String xslt = "";
			if (rootElement.contains("Invoice")){
				xslt = "XSLTFattura.xsl";
			}
			else if (rootElement.contains("CreditNote")){
				xslt = "XSLTNotaCredito.xsl";
			}
			else if (rootElement.contains("DebitNote")){
				xslt = "XSLTNotaDebito.xsl";
			}
			
			Transformer transformer = tFactory
					.newTransformer(new javax.xml.transform.stream.StreamSource(new File(path,
							xslt)));
					
			transformer.transform(new javax.xml.transform.stream.StreamSource(
					new ByteArrayInputStream(invoice)),
					new javax.xml.transform.stream.StreamResult(os));
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				os.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		if (os != null)
			return os.toByteArray();
		else
			return new byte[] {};
	}

	private byte[] generatePDF(Invoice invoice) {
		return generatePDF(invoice.getXmlFile().getFileBlob());
	}
	private byte[] generatePDF(byte[] invoice) {
		// Construct fop with desired output format
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		try {
			
			String path = getServletContext().getRealPath("/rendering/");
			
			fopFactory.setStrictValidation(false);
			FOUserAgent foUserAgent = fopFactory.newFOUserAgent();
			Fop fop = fopFactory
					.newFop(MimeConstants.MIME_PDF, foUserAgent, os);

			fopFactory.addElementMapping(new FOElementMapping());
			fopFactory.addElementMapping(new SVGElementMapping());
			fopFactory.addElementMapping(new BatikExtensionElementMapping());
			fopFactory.addElementMapping(new ExtensionElementMapping());
			fopFactory.addElementMapping(new XMPElementMapping());
			fopFactory.addElementMapping(new RDFElementMapping());
			fopFactory.addElementMapping(new PSExtensionElementMapping());

			// Setup XSLT
//			Transformer transformer = tFactory.newTransformer(new StreamSource(
//					xsltfile));
//			Transformer transformer = tFactory.newTransformer(new StreamSource(
//					fileIs));

			Transformer transformer = tFactory.newTransformer(new StreamSource(new File(path, "CraneUBL2UN380Invoice-EN.xsl")));
//					fileIs));
					
			// Set the value of a <param> in the stylesheet
			transformer.setParameter("versionParam", "2.0");

			// Setup input for XSLT transformation
			Source src = new StreamSource(new ByteArrayInputStream(invoice));

			// Resulting SAX events (the generated FO) must be piped through to
			// FOP
			Result res = new SAXResult(fop.getDefaultHandler());

			// Start XSLT transformation and FOP processing
			try {
				transformer.transform(src, res);
			} catch (TransformerException t) {
				System.out.println(t.getMessage());
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				os.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		if (os != null)
			return os.toByteArray();
		else
			return new byte[] {};

	}

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		doGet(req, resp);
	}
}
