/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.invoice;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.invoice.TransmissionType;
import it.fe.core.par.user.Organization;
import it.fe.core.par.user.User;
import it.fe.web.common.ArchivingException;
import it.fe.web.common.SessionUtils;
import it.fe.core.ejb.parer.wsupload.EsitoType;
import it.fe.core.ejb.parer.wsupload.Risposta;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;

import org.apache.log4j.Logger;

public class ArchivingHandler implements Serializable {

	private static final long serialVersionUID = 1L;

	private static Logger logger = Logger.getLogger(ArchivingHandler.class);

	private Invoice currentInvoice;
	private Date invoiceRegistrationDate;
	private String invoiceRegistrationNumber;
	private String invoiceArchive;

	private String archivingUsername;
	private String archivingPassword;

	private String supplierName;
	private String supplierId;

	public SelectItem[] availableSupplierIdTypes = new SelectItem[] {
			new SelectItem("---", "---","---",false,false),
			new SelectItem("PIVA", "PartitaIVA","PartitaIVA",false,false),
			new SelectItem("CF", "Codice fiscale","Codice fiscale",false,false) };
	public SelectItem[] availableSupplierNameTypes = new SelectItem[] {
			new SelectItem("---", "---","---",false,false),
			new SelectItem("RagioneSociale", "Ragione sociale","Ragione sociale",false,false),
			new SelectItem("NomeCognome", "Nome Cognome","Nome Cognome",false,false) };

	public String supplierNameType;
	public String supplierIdType;

	public String getSupplierNameType() {
		return supplierNameType;
	}

	public void setSupplierNameType(String supplierNameType) {
		System.out.println("*** chiamo set supplierNameType ");
		if (supplierNameType.equals("---"))
			this.supplierNameType = null;
		else
			this.supplierNameType = supplierNameType;
	}

	public String getSupplierIdType() {
		return supplierIdType;
	}

	public void setSupplierIdType(String supplierIdType) {
		System.out.println("*** chiamo set supplierIdType ");
		if (supplierIdType.equals("---"))
			this.supplierIdType = null;
		else
			this.supplierIdType = supplierIdType;
	}

	public Invoice getCurrentInvoice() {
		return currentInvoice;
	}

	public void setCurrentInvoice(Invoice currentInvoice) {
		this.currentInvoice = currentInvoice;
	}

	public Date getInvoiceRegistrationDate() {
		return invoiceRegistrationDate;
	}

	public void setInvoiceRegistrationDate(Date invoiceRegistrationDate) {
		System.out.println("*** chiamo set data ");
		this.invoiceRegistrationDate = invoiceRegistrationDate;
	}

	public String getInvoiceRegistrationNumber() {
		return invoiceRegistrationNumber;
	}

	public void setInvoiceRegistrationNumber(String invoiceRegistrationNumber) {
		this.invoiceRegistrationNumber = invoiceRegistrationNumber;
	}

	public String getInvoiceArchive() {
		return invoiceArchive;
	}

	public void setInvoiceArchive(String invoiceArchive) {
		this.invoiceArchive = invoiceArchive;
	}

	public String getArchivingUsername() {
		return archivingUsername;
	}

	public void setArchivingUsername(String archivingUsername) {
		this.archivingUsername = archivingUsername;
	}

	public String getArchivingPassword() {
		return archivingPassword;
	}

	public void setArchivingPassword(String archivingPassword) {
		this.archivingPassword = archivingPassword;
	}

	public String getSupplierName() {
		return supplierName;
	}

	public void setSupplierName(String supplierName) {
		this.supplierName = supplierName;
	}

	public String getSupplierId() {
		return supplierId;
	}

	public void setSupplierId(String supplierId) {
		this.supplierId = supplierId;
	}

	public SelectItem[] getAvailableSupplierIdTypes() {
		return availableSupplierIdTypes;
	}

	public void setAvailableSupplierIdTypes(
			SelectItem[] availableSupplierIdTypes) {
		this.availableSupplierIdTypes = availableSupplierIdTypes;
	}

	public SelectItem[] getAvailableSupplierNameTypes() {
		return availableSupplierNameTypes;
	}

	public void setAvailableSupplierNameTypes(
			SelectItem[] availableSupplierNameTypes) {
		this.availableSupplierNameTypes = availableSupplierNameTypes;
	}

	
	public String confirmArchive() {
		FacesContext context = FacesContext.getCurrentInstance();
		if ((this.archivingUsername == null)
				|| (this.archivingUsername.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Username non valorizzato", null));
			return "openDetail";
		}
		if ((this.archivingUsername == null)
				|| (this.archivingUsername.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Username non valorizzato", null));
			return "openDetail";
		}
		
		if (this.invoiceRegistrationDate == null) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Data di emissione non valorizzata", null));
			return "openDetail";
		}
		if ((this.archivingPassword == null)
				|| (this.archivingPassword.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Password non valorizzata", null));
			return "openDetail";
		}
		if ((this.invoiceArchive == null) || (this.invoiceArchive.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Registro non valorizzato", null));
			return "openDetail";
		}
		if ((this.supplierName == null) || (this.supplierName.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Denominazione fornitore non valorizzata", null));
			return "openDetail";
		}
		if ((this.supplierNameType == null) || (this.supplierNameType.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Tipo Denominazione fornitore non valorizzata", null));
			return "openDetail";
		}
		
		if ((this.supplierIdType == null) || (this.supplierIdType.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Tipo ID fornitore non valorizzata", null));
			return "openDetail";
		}
		if ((this.supplierId == null) || (this.supplierId.equals(""))) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "ID fornitore non valorizzata", null));
			return "openDetail";
		}
		
		List <OriginalDocument> originalDocuments = JNDIUtils.getInvoiceManager().fetchOriginalDocuments(currentInvoice);
		if ((originalDocuments == null) || (originalDocuments.size() < 1)) {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Nessun documento PEPPOL collegato", null));
			return "openDetail";
		}
		
		if (currentInvoice.getIssueDate() == null)  {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Fattura priva di data di emissione: impossibile determinare automaticamente l'anno di versamento", null));
			return "openDetail";
		}
		
		return "confirmArchive";
	}
	
	public String archive() throws ArchivingException {
		FacesContext context = FacesContext.getCurrentInstance();
		Iterator<FacesMessage> msgIterator= context.getMessages();
	    while(msgIterator.hasNext())
	    {
	        msgIterator.next();
	        msgIterator.remove();
	    }

		if ((this.archivingUsername == null)
				|| (this.archivingUsername.equals(""))) {
			throw new ArchivingException("Username non valorizzato");
		}
		if (this.invoiceRegistrationDate == null) {
			throw new ArchivingException("Data di emissione non valorizzata");
		}
		if ((this.archivingPassword == null)
				|| (this.archivingPassword.equals(""))) {
			throw new ArchivingException("Password non valorizzata");
		}
		if ((this.invoiceArchive == null) || (this.invoiceArchive.equals(""))) {
			throw new ArchivingException("Registro non valorizzato");
		}
		if ((this.supplierName == null) || (this.supplierName.equals(""))) {
			throw new ArchivingException(
					"Denominazione fornitore non specificata");
		}
		if ((this.supplierId == null) || (this.supplierId.equals(""))) {
			throw new ArchivingException(
					"ID fornitore non specificato");
		}
		if ((this.supplierNameType == null) || (this.supplierNameType.equals(""))) {
			throw new ArchivingException(
					"Tipo Denominazione fornitore non specificata");
		}
		
		if ((this.supplierIdType == null) || (this.supplierIdType.equals(""))) {
			throw new ArchivingException(
					"Tipo ID fornitore non specificato");
		}
		
		List <OriginalDocument> originalDocuments = JNDIUtils.getInvoiceManager().fetchOriginalDocuments(currentInvoice);
		if ((originalDocuments == null) || (originalDocuments.size() < 1)) {
			throw new ArchivingException("Nessun documento collegato");
		}

		String messageUUID = null ;
		for (OriginalDocument originalDocument : originalDocuments) {
			if (originalDocument.getTransmissionType().getDescription().equals(TransmissionType.type.PEPPOL_INCOMING.toString()));
			messageUUID = originalDocument.getAdditionalInfo();
		}
		

		if ((messageUUID == null) || (messageUUID.equals(""))) {
			throw new ArchivingException("Nessun documento collegato");
		}
		else if (!messageUUID.toLowerCase().startsWith("uuid:"))
			messageUUID="uuid:"+messageUUID;

		GregorianCalendar calendar = new GregorianCalendar();
		calendar.setTime(currentInvoice.getIssueDate());
		int year = calendar.get(Calendar.YEAR);

		String user = SessionUtils.getCurrentUserScreenname();
		User currentUser = JNDIUtils.getUserManager().getUserByUsername(user);
		if (( currentUser != null) && (currentUser.getUoId() != null) && (!currentUser.getUoId().equals("")) ){
			
			Organization organization = JNDIUtils.getOrganizationManager().getOrganizationByUoID(currentUser.getUoId());
			if (organization != null){
				Risposta risposta = JNDIUtils.getArchivingManager().uploadDatiFiscaliFatturaPassiva(
						currentInvoice, 
						this.archivingUsername, 
						this.archivingPassword,
						messageUUID, 
						year, 
						this.invoiceRegistrationNumber, 
						this.invoiceArchive,
						this.invoiceRegistrationDate, 
						this.supplierName, 
						this.supplierNameType, 
						this.supplierId, 
						this.supplierIdType, 
						organization
						);
				
				if (risposta.getEsitoGenerale().getEsito().equals(EsitoType.NEGATIVO)){
					context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Errore in fase di versamento", risposta.getEsitoGenerale().getMessaggio()));
					return "confirmArchive";
				}
				else if (risposta.getEsitoGenerale().getEsito().equals(EsitoType.POSITIVO)){
					context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_INFO, "Versamento confermato", null));
					return "archived";
				}
				
			}
			else {
				context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Nessuna organizzazione associata all'utente corrente", null));
				return "confirmArchive";
			}
		}
		else {
			context.addMessage("error-messages", new FacesMessage(FacesMessage.SEVERITY_ERROR, "Nessuna organizzazione associata all'utente corrente", null));
			return "confirmArchive";
		}
		return "confirmArchive";
		

	}
	
	public String backToList(){
		this.archivingPassword=null;
		this.archivingUsername = null;
		this.currentInvoice = null;
		this.invoiceArchive = null;
		this.invoiceRegistrationDate = null;
		this.invoiceRegistrationNumber = null;
		this.supplierId = null;
		this.supplierIdType = null;
		this.supplierName = null;
		this.supplierNameType = null;
		
		return "invoicesView";
	}
	
	public String backToDetail(){
		return "openDetail";
	}

}
