/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.invoice;

import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.utility.Meter;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;
import it.fe.web.common.JSFUtils;
import it.fe.web.common.RenderingUtils;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class InvoiceInfoHandler implements Serializable {
	
	private static final long serialVersionUID = 1L;

	private static Logger logger = Logger.getLogger(InvoiceInfoHandler.class);
	
	private String currentDocumentNumber;
	private String currentInvoiceId;
	private String currentDocumentId;
	private PreviousDocument currentDocument;
	private Map<String,MeterSummary> meterSummariesMap;
	private Map<String, PreviousDocument> previousDocumentMap;
	private List<MeterSummary> meterSummariesList;
	private List<PreviousDocument> previousDocumentList;
	private boolean contractPanelRendered;
	private boolean podPanelRendered;
	private boolean readingListRendered;
//	private Map<String,String> contractInfo;
	
	private Map<Meter,MeterSummary> meterInfo;
	
	public void updateCurrentContract()  {
		this.currentDocumentNumber = JSFUtils.getRequestParameterByKey("currentContract");
		this.currentDocument = previousDocumentMap.get(this.currentDocumentNumber);
		this.setPodPanelRendered(false);
		this.setContractPanelRendered(true);
	}
	
	public void updateCurrentPod() {
		this.currentDocumentId = JSFUtils.getRequestParameterByKey("contractId");
		this.currentInvoiceId = JSFUtils.getRequestParameterByKey("invoiceId");
		
		this.meterInfo = RenderingUtils.meterByContract(currentInvoiceId, currentDocumentId);
		
		this.setContractPanelRendered(false);
		this.setPodPanelRendered(true);
	}
	

	/**
	 * @return the contractPanelRendered
	 */
	public boolean isContractPanelRendered() {
		return contractPanelRendered;
	}

	/**
	 * @param contractPanelRendered the contractPanelRendered to set
	 */
	public void setContractPanelRendered(boolean contractPanelRendered) {
		this.contractPanelRendered = contractPanelRendered;
	}

	/**
	 * @return the podPanelRendered
	 */
	public boolean isPodPanelRendered() {
		return podPanelRendered;
	}

	/**
	 * @param podPanelRendered the podPanelRendered to set
	 */
	public void setPodPanelRendered(boolean podPanelRendered) {
		this.podPanelRendered = podPanelRendered;
	}


//	/**
//	 * @return the contractInfo
//	 */
//	public Map<String, String> getContractInfo() {
//		return contractInfo;
//	}
//
//	/**
//	 * @param contractInfo the contractInfo to set
//	 */
//	public void setContractInfo(Map<String, String> contractInfo) {
//		this.contractInfo = contractInfo;
//	}

	/**
	 * @return the meterInfo
	 */
	public Map<Meter, MeterSummary> getMeterInfo() {
		return meterInfo;
	}

	/**
	 * @param meterInfo the meterInfo to set
	 */
	public void setMeterInfo(Map<Meter, MeterSummary> meterInfo) {
		this.meterInfo = meterInfo;
	}

	/**
	 * @return the meterSummariesMap
	 */
	public Map<String, MeterSummary> getMeterSummariesMap() {
		return meterSummariesMap;
	}

	/**
	 * @param meterSummariesMap the meterSummariesMap to set
	 */
	public void setMeterSummariesMap(Map<String, MeterSummary> meterSummariesMap) {
		this.meterSummariesMap = meterSummariesMap;
	}

	/**
	 * @return the previousDocumentMap
	 */
	public Map<String, PreviousDocument> getPreviousDocumentMap() {
		return previousDocumentMap;
	}

	/**
	 * @param previousDocumentMap the previousDocumentMap to set
	 */
	public void setPreviousDocumentMap(
			Map<String, PreviousDocument> previousDocumentMap) {
		this.previousDocumentMap = previousDocumentMap;
	}

	/**
	 * @return the meterSummariesList
	 */
	public List<MeterSummary> getMeterSummariesList() {
		return meterSummariesList;
	}

	/**
	 * @param meterSummariesList the meterSummariesList to set
	 */
	public void setMeterSummariesList(List<MeterSummary> meterSummariesList) {
		this.meterSummariesList = meterSummariesList;
		Map<String,MeterSummary> asMap = new HashMap<String, MeterSummary>();
		for (MeterSummary meterSummary : meterSummariesList) {
			asMap.put(meterSummary.getMeter().getMeterNumber(), meterSummary);
		}
		this.setMeterSummariesMap(asMap);
	}

	/**
	 * @return the previousDocumentList
	 */
	public List<PreviousDocument> getPreviousDocumentList() {
		return previousDocumentList;
	}

	/**
	 * @param previousDocumentList the previousDocumentList to set
	 */
	public void setPreviousDocumentList(List<PreviousDocument> previousDocumentList) {
		this.previousDocumentList = previousDocumentList;
		Map<String,PreviousDocument> asMap = new HashMap<String, PreviousDocument>();
		for (PreviousDocument previousDocument : previousDocumentList) {
			asMap.put(previousDocument.getDocumentCode(), previousDocument);
		}
		this.setPreviousDocumentMap(asMap);
	}

	/**
	 * @return the currentDocumentNumber
	 */
	public String getCurrentDocumentNumber() {
		return currentDocumentNumber;
	}

	/**
	 * @param currentDocumentNumber the currentDocumentNumber to set
	 */
	public void setCurrentDocumentNumber(String currentDocumentNumber) {
		this.currentDocumentNumber = currentDocumentNumber;
	}

	/**
	 * @return the currentDocument
	 */
	public PreviousDocument getCurrentDocument() {
		return currentDocument;
	}

	/**
	 * @param currentDocument the currentDocument to set
	 */
	public void setCurrentDocument(PreviousDocument currentDocument) {
		this.currentDocument = currentDocument;
	}

	public void setCurrentInvoiceId(String currentInvoiceId) {
		this.currentInvoiceId = currentInvoiceId;
	}

	public String getCurrentInvoiceId() {
		return currentInvoiceId;
	}

	/**
	 * @param currentDocumentId the currentDocumentId to set
	 */
	public void setCurrentDocumentId(String currentDocumentId) {
		this.currentDocumentId = currentDocumentId;
	}

	/**
	 * @return the currentDocumentId
	 */
	public String getCurrentDocumentId() {
		return currentDocumentId;
	}

}
