/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
 *
 */
package it.fe.web.controller.invoice;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.ejb.process.ProcessManager;
import it.fe.core.ejb.unimore.cia.UnimoreCIAGenericDAO;
import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.InvoiceState;
import it.fe.core.par.unimore.cia.Anagrafico;
import it.fe.core.par.unimore.cia.Banca;
import it.fe.core.par.unimore.cia.Causale;
import it.fe.core.par.unimore.cia.CentroCosto;
import it.fe.core.par.unimore.cia.BeneServizio;
import it.fe.core.par.unimore.cia.ContoContabile;
import it.fe.core.par.unimore.cia.ModalitaPagamento;
import it.fe.core.par.unimore.cia.UnitaOrganizzativa;
import it.fe.core.process.exception.JbpmProcessException;
import it.fe.core.utils.UnimoreConfiguration;
import it.fe.web.common.JSFUtils;
import it.fe.web.common.SessionUtils;
import it.fe.web.controller.profile.UnimoreCiaProfilesHandler;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.SelectItem;

import org.hibernate.LazyInitializationException;
import org.richfaces.event.SimpleToggleEvent;

/**
 * Handler di fatture specifico per UniMoRe per la gestione dei dati aggiuntivi;
 * estende la classe FilterableInvoicesHandler
 * 
 * 
 * @author Stefano Monti, Fulvio Di Marco
 * 
 */
public class UnimoreFilterableInvoicesHandler extends FilterableInvoicesHandler {

	private static final long serialVersionUID = 1L;

	public UnimoreFilterableInvoicesHandler() {
		ciaProfilesHandler = (UnimoreCiaProfilesHandler) JSFUtils
				.getManagedBean("unimoreCiaProfilesHandler");
	}
	
	@Override
	public String openDetail() {
		supplierCodes = null;
		retrievedSuppliersMap  = null;
		supplierCode = null;

		eserciziDisponibili = null;
		esercizio = null;

		codiciBilancioDisponibili = null;
		codiceBilancio = null;

		codiciUODisponibili = null;
		codiceUO = null;
		
		availableFinancialAccounts = null;
		retrievedFinancialAccountsMap = null;
		financialAccount = null;
		
		availablePaymentMethods = null;
		retrievedPaymentMethodsMap = null;
		paymentMethod = null;
		
		availableBankAccounts = null;
		retrievedBankAccountsMap = null;
		bankAccount = null;

		checked = new HashMap<Long, Boolean>();
		currentAllChecked = false;
		
		lineReasons = null;
		availableReasons = null;
		retrievedReasonsMap = null;

		lineCostCenters = null;
		availableCostCentersSelectItems = null;
		retrievedCostCentersMap = null;
		
		lineGoods = null;
		availableGoodsSelectItems = null;
		retrievedGoodsMap = null;
		
		return super.openDetail();
	}

	/*
	 * 
	 * Gestione dati aggiuntivi
	 */
	
	protected boolean editingState = false;

	protected List<SelectItem> supplierCodes;
	protected Map<String, Anagrafico> retrievedSuppliersMap;
	protected String supplierCode;

	protected List<SelectItem> eserciziDisponibili;
	protected String esercizio;

	protected List<SelectItem> codiciBilancioDisponibili;
	protected String codiceBilancio;

	protected List<SelectItem> codiciUODisponibili;
	protected String codiceUO;
	
	protected List<SelectItem> availablePaymentMethods;
	protected Map<String, ModalitaPagamento> retrievedPaymentMethodsMap;
	protected String paymentMethod;
	
	protected List<SelectItem> availableBankAccounts;
	protected Map<String, Banca> retrievedBankAccountsMap;
	protected String bankAccount;
	
	protected List<SelectItem> availableFinancialAccounts;
	protected Map<String, ContoContabile> retrievedFinancialAccountsMap;
	protected String financialAccount;
	
	protected boolean invoiceLinesTogglePanelOpened;

	protected Map<Long, Boolean> checked = new HashMap<Long, Boolean>();
	protected boolean currentAllChecked = false;
	
	protected Map<Long, String> lineReasons;
	protected List<SelectItem> availableReasons;
	protected Map<String, Causale> retrievedReasonsMap;

	protected Map<Long, String> lineCostCenters;
	protected List<SelectItem> availableCostCentersSelectItems;
	protected Map<String,CentroCosto> retrievedCostCentersMap;
	
	protected Map<Long, String> lineGoods;
	protected List<SelectItem> availableGoodsSelectItems;
	protected Map<String, BeneServizio> retrievedGoodsMap;

	protected String reasonToCopy = "Seleziona la causale";
	protected String costCenterToCopy = "Seleziona il centro di costo";
	protected String goodToCopy = "Seleziona il bene servizio";
	
	// modal panel attributes
	protected boolean approvalModalPanelRendered = false;
	protected boolean missingAdditionalData;
	protected boolean supplierCodeSaved;
	protected boolean financialYearSaved;
	protected boolean budgetSaved;
	protected boolean uoSaved;
	protected boolean paymentMethodSaved;
	protected boolean bankAccountSaved;
	protected boolean invoiceLinesHasAdditionalDataSaved;
	
	public String approvePreInvoiceInsertion() {
//		String ciaInsertionState;
		ProcessManager processManager = JNDIUtils.getProcessManager();
		try {
			processManager.approveInsertionIntoAccountableSystem(selectedInvoice, SessionUtils.getCurrentUserScreenname());
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		}
		
		// forcing data model reload
		this.dataModel = null;
		super.updateEditable();
		super.updateInvoiceStateAndError();
		
		boolean error = selectedInvoice.isError();
		
		// se la fattura contiene errori di inserimento in CIA lo segnalo
		if (error && !selectedInvoice.getInvoiceState().getDescription().equals(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED)) {
			String errorMessages = selectedInvoice.getErrorMessage();
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore nell'inserimento nel sistema contabile:/n"+errorMessages, "Si è verificato un errore nell'inserimento nel sistema contabile:/n"+errorMessages));  
		}
			
		else if (selectedInvoice.getInvoiceState().getDescription().equals(InvoiceState.ACCOUNTABLE_SYSTEM_INSERTED))
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO,"La fattura è stata inserita correttamente nel sistema contabile", "La fattura è stata inserita correttamente nel sistema contabile"));  
		return null;
	}
	
	public String technicallyApproveInvoice() {		
		try {
			JNDIUtils.getProcessManager().approveTechnically(selectedInvoice, SessionUtils.getCurrentUserScreenname());
		} catch (JbpmProcessException e) {
			logger.info(e.getMessage());
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR,"Si è verificato un errore: "+e.getMessage(), "Si è verificato un errore: "+e.getMessage()));  
			return null;
		}
		selectedInvoice.setInvoiceState(JNDIUtils.getInvoiceStateManager().getInvoiceStateByDescription(InvoiceState.TECHNICALLY_VALIDATED));
		// forcing data model reload
		this.dataModel = null;
		super.updateEditable();
		super.updateInvoiceStateAndError();
		return null;
	}
	
	public String technicallyRejectInvoice() {
		return super.technicallyRejectInvoice();
	}
	
	public String editAdditionalData() {
		updateAllAvailableData();
		this.setEditingState(true);
		return super.editAdditionalData();
	}

	/**
	 * Used to force reloading of all data fields (both FE and CIA)
	 */
	private void updateAllAvailableData() {
		this.updateSupplierCodes();
		this.updateEserciziDisponibili();
		this.updateCodiciBilancioDisponibili();
		this.updateCodiciUODisponibili();
		this.updateAvailableFinancialAccounts();
		this.updateAvailablePaymentMethods();
		this.updateAvailableBankAccounts();
		this.updateAvailableReasons();
		this.updateAvailableCostCenters();
		this.updateAvailableGoods();
	}
	
	public String discardAdditionalData() {
		this.setEditingState(false);
		resetAllFields();
		return super.discardAdditionalData();
	}
	
	public void saveAdditionalDataEvent(ActionEvent event) {
		this.saveAdditionalData();
	}
	
	public String saveAdditionalData() {
		
		// Anagrafico (codice fornitore)
		if (retrievedSuppliersMap != null && !retrievedSuppliersMap.isEmpty()) {
			Anagrafico anagraficoToBeSaved = retrievedSuppliersMap.get(supplierCode);
			selectedInvoice = saveOrReplaceAdditionalData(selectedInvoice, anagraficoToBeSaved, Anagrafico.class);
		}
		
		// UnitaOrganizzativa (esercizio, codiceBilancio e codice uo): me la devo ricavare dalle tre informazioni estratte dalla form.
		// prima verifico sul db locale di FE e se non esiste cerco su CIA
		String query = "SELECT * FROM CIA_UNITA_ORGANIZZATIVA WHERE ESERCIZIO='"+esercizio+"' AND IDENTIFICATIVO_BILANCIO='"+codiceBilancio+"' AND CHIAVE_COMPLETA='"+codiceUO+"'";
		List<UnitaOrganizzativa> feRetrievedUos = JNDIUtils.getCrudManager().findByNativeQuery(query, UnitaOrganizzativa.class);
		UnitaOrganizzativa uoToBeSaved = null;
		if ( (feRetrievedUos !=null) &&  (!feRetrievedUos.isEmpty()) ) {
			uoToBeSaved = feRetrievedUos.get(0);
		}
		else {
			Map<String, Object> parameters = new HashMap<String, Object>();
			parameters.put("esercizio", esercizio);
			parameters.put("identificativo_bilancio", codiceBilancio);
			parameters.put("chiave_completa", codiceUO);
			List<UnitaOrganizzativa> ciaRetrievedUOs = UnimoreCIAGenericDAO.findByParameters(UnitaOrganizzativa.class, parameters);
			if ( (ciaRetrievedUOs !=null) && (!ciaRetrievedUOs.isEmpty()) && (ciaRetrievedUOs.size() == 1) ) {
				uoToBeSaved = ciaRetrievedUOs.get(0);
			}
		}
		selectedInvoice = saveOrReplaceAdditionalData(selectedInvoice, uoToBeSaved, UnitaOrganizzativa.class);
		
		// Metodo di pagamento
		ModalitaPagamento paymentMethodToBeSaved = null;
		if (retrievedPaymentMethodsMap != null && !retrievedPaymentMethodsMap.isEmpty() && paymentMethod != null) {
			paymentMethodToBeSaved = retrievedPaymentMethodsMap.get(paymentMethod);
			selectedInvoice = saveOrReplaceAdditionalData(selectedInvoice, paymentMethodToBeSaved, ModalitaPagamento.class);
		}
		
		// Banca
		if ( paymentMethodToBeSaved != null && paymentMethodToBeSaved.getCodice_rif().equals("CC")) {
			Banca bankAccountToBeSaved = retrievedBankAccountsMap.get(bankAccount);
			selectedInvoice = saveOrReplaceAdditionalData(selectedInvoice, bankAccountToBeSaved, Banca.class);
		}
		
		// 5. per ciascuna riga devono essere trattati
		// 5.1 Causale (it.fe.core.par.unimore.cia.Causale) -> lineReasons(i)
		// 5.2 Centro di costo (it.fe.core.par.unimore.cia.CentroCosto) -> lineCostCenters(i)
		// 5.3 Codice bene servizio
		// 5.4 Conto contabile
		for (InvoiceLine invoiceLine : selectedInvoice.getInvoiceLines()) {
			// se la riga fattura non ha salvato il dato aggiuntivo -> lo salvo
			
			// causali
			if (lineReasons != null) {
				String lineReasonAsString = lineReasons.get( new Long(invoiceLine.getInvoiceLineId()) );
				if (lineReasonAsString !=null && !"".equals(lineReasonAsString)) {
					if (!"Seleziona la causale".equals(lineReasonAsString)) {
						String reasonCode = lineReasonAsString.substring(0, lineReasonAsString.indexOf('-')-1);
						invoiceLine = saveOrReplaceAdditionalData(invoiceLine, retrievedReasonsMap.get( reasonCode ), Causale.class);
					}
					else {
						// rimuovere il dato
						invoiceLine = saveOrReplaceAdditionalData(invoiceLine, null , Causale.class);
					}
					
				}
			}
			
			// centri di costo
			if (lineCostCenters != null) {
				String costCenterAsString = lineCostCenters.get( new Long(invoiceLine.getInvoiceLineId()) );
				if (costCenterAsString != null && !"".equals(costCenterAsString)) {
					if (!"Seleziona il centro di costo".equals(costCenterAsString)) {
						String costCenterCode = costCenterAsString.substring(0, costCenterAsString.indexOf('-')-1);
						invoiceLine = saveOrReplaceAdditionalData(invoiceLine, retrievedCostCentersMap.get( costCenterCode ), CentroCosto.class);
					}
					else {
						// rimuovere il dato
						invoiceLine = saveOrReplaceAdditionalData(invoiceLine, null , CentroCosto.class);
					}
						
				}
			}
			
			// beni servizio
			if (lineGoods != null) {
				String goodAsString = lineGoods.get( new Long(invoiceLine.getInvoiceLineId()) );
				if (goodAsString != null && !"".equals(goodAsString)) {
					if (!"Seleziona il bene servizio".equals(goodAsString)) {
						String goodCode = goodAsString.substring(0, goodAsString.indexOf('-')-1);
						invoiceLine = saveOrReplaceAdditionalData(invoiceLine, retrievedGoodsMap.get( goodCode ), BeneServizio.class);
					}
					else {
						// rimuovere il dato
						invoiceLine = saveOrReplaceAdditionalData(invoiceLine, null , BeneServizio.class);
					}
						
				}
			}
			
			// conto contabile
			if (financialAccount != null && !"".equals(financialAccount)) {
				if (!"Seleziona il bene servizio".equals(financialAccount)) {
					invoiceLine = saveOrReplaceAdditionalData(invoiceLine, retrievedFinancialAccountsMap.get( financialAccount ), ContoContabile.class);
				}
				else {
					// rimuovere il dato
					invoiceLine = saveOrReplaceAdditionalData(invoiceLine, null , ContoContabile.class);
				}
			}
			
		}
		
		this.setEditingState(false);
		return super.saveAdditionalData();
	}
	
	/**
	 * Utility method that helps with indication of postback
	 * 
	 * @return
	 */
	public boolean isPostback() {
		FacesContext context = FacesContext.getCurrentInstance();
        return context.getRenderKit().getResponseStateManager().isPostback(context);
	}
	
	public void showApprovalModalPanel() {
		this.setApprovalModalPanelRendered(true);
	}

	/**
	 * Listener for rich:tabPanel tab changes. May be used to ajaxilly loading
	 * tab contents on tab switching.
	 * 
	 * @param event
	 */
	public void tabChange(ValueChangeEvent event) {
		String newTab = (String) event.getNewValue();
		if (newTab.equals("invoiceAdditionalInfo")) {
			
			// Invoice.AdditionalData: lazy fetching and caching for further requests
			List<AdditionalData> invoiceAdditionalData = selectedInvoice.getAdditionalData();
			if ( !(invoiceAdditionalData instanceof ArrayList)) {
				List<AdditionalData> lazilyFetchedAdditionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(selectedInvoice);
				this.selectedInvoice.setAdditionalData(lazilyFetchedAdditionalData);
			}
			
			// Invoice.InvoiceLines & InvoiceLine.AdditionalData: lazy fetching and caching for further requests
			List<InvoiceLine> invoiceLines = selectedInvoice.getInvoiceLines();
			if ( !(invoiceLines instanceof ArrayList) ) {
				List<InvoiceLine> lazilyFetchedInvoiceLines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
				
				for (InvoiceLine il : lazilyFetchedInvoiceLines) {
					List<AdditionalData> lazilyFetchedInvoiceLineAdditionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(il);
					il.setAdditionalData(lazilyFetchedInvoiceLineAdditionalData);
				}
				this.selectedInvoice.setInvoiceLines(lazilyFetchedInvoiceLines);
			}
						
			
			supplierCode = null;
			esercizio = null;
			codiceBilancio = null;
			codiceUO = null;
			paymentMethod = null;
			bankAccount = null;
			availableCostCentersSelectItems = null;
			lineCostCenters = null;
			availableReasons = null;
			lineReasons = null;
			
			
		}
//		if (newTab.equals("invoiceSpecificInfo")) {
//			List<PreviousDocument> documents = JNDIUtils.getInvoiceManager().fetchPreviousDocuments(selectedInvoice);
//			List<MeterSummary> summaries = JNDIUtils.getInvoiceManager().fetchMeterSummaries(selectedInvoice);
//			this.selectedInvoice.setPreviousDocuments(documents);
//			this.selectedInvoice.setMeterSummary(summaries);
//		}
	}
	
	/**
	 * Listener for invoice lines panel toggling.
	 * Used to force re-rendering of components within panel.
	 * 
	 * @param actionEvent
	 */
	public void invoiceLinesPanelToggled(ActionEvent actionEvent) {
		SimpleToggleEvent toggleEvent = (SimpleToggleEvent) actionEvent;
		this.setInvoiceLinesTogglePanelOpened(toggleEvent.isIsOpen());
		if (this.isInvoiceLinesTogglePanelOpened()) {
			// force reRendering
			availableCostCentersSelectItems = null;
			availableReasons = null;
			lineCostCenters = null;
			lineReasons = null;
		}
	}
	
	public void onchangeCiaSupplierCode() {
		availableFinancialAccounts = null;
		availableBankAccounts = null;
		availablePaymentMethods = null;
	}

	public void onChangePaymentMethod() {
		availableBankAccounts = null;
		bankAccount = null;
	}

	/**
	 * Action invoked when esercizi's selectOneMenu actual value changes.
	 */
	public void onchangeEsercizio() {
		// se modifico l'esercizio selezionato devo azzerare tutto: la parte
		// successiva di rirenderizzazione si occuperà di rifare le query
		codiciBilancioDisponibili = null;
		codiceBilancio = null;
		codiciUODisponibili = null;
		codiceUO = null;
		availableFinancialAccounts = null;
		financialAccount = null;
		availableCostCentersSelectItems = null;
	}

	/**
	 * Action invoked when bilanci's selectOneMenu actual value changes.
	 */
	public void onchangeBilancio() {
		// se modifico il bilancio, posso tenere fisso solo l'esercizio e
		// riazzerare tutto il resto
		codiciUODisponibili = null;
		codiceUO = null;
	}

	/**
	 * Aggiorna gli esercizi disponibili in base al profilo CIA dell'utente connesso
	 */
	public void updateEserciziDisponibili() {

		// MOCK
		// eserciziDisponibili = new SelectItem[3];
		// GregorianCalendar gc = new GregorianCalendar();
		// gc.setTime(new Date());
		// eserciziDisponibili[0] = new SelectItem("", "Seleziona l'esercizio");
		// eserciziDisponibili[1] = new
		// SelectItem(String.valueOf(gc.get(Calendar.YEAR)),
		// String.valueOf(gc.get(Calendar.YEAR)));
		// eserciziDisponibili[2] = new
		// SelectItem(String.valueOf(gc.get(Calendar.YEAR)-1),
		// String.valueOf(gc.get(Calendar.YEAR)-1));
		// END OF MOCK

		// estraggo gli esercizi disponibili dai profili utente CIA presenti in
		// sessione HTTP
		List<String> userFinancialYears = ciaProfilesHandler.getUserFinancialYears();

		List<SelectItem> updatedList = new ArrayList<SelectItem>();

		updatedList.add(new SelectItem("", "Seleziona l'esercizio"));

		// aggiungo tutti gli altri esercizi presenti nel profilo cia dell'utente
		if ( (userFinancialYears != null) && (!userFinancialYears.isEmpty()) ) {
			for (String year : userFinancialYears) {
				updatedList.add(new SelectItem(year, year));
			}
		}

		eserciziDisponibili = updatedList;

	}

	/**
	 * Aggiorna i codici dei fornitori
	 */
	public void updateSupplierCodes() {
		if ((this.selectedInvoice == null)
				|| (this.selectedInvoice.getSupplier() == null)
				|| (this.selectedInvoice.getSupplier().getVatId() == null)) {
			supplierCodes = new ArrayList<SelectItem>();
			return;
		}

		// MOCK
		// supplierCodes = new SelectItem[4];
		// for (int i = 0 ; i<supplierCodes.length; i++)
		// supplierCodes[i] = new
		// SelectItem(this.selectedInvoice.getSupplier().getVatId(),
		// this.selectedInvoice.getSupplier().getSupplierName()+","+this.selectedInvoice.getSupplier().getVatId(),this.selectedInvoice.getSupplier().getVatId());
		// END OF MOCK
		
		List<SelectItem> updatedList = new ArrayList<SelectItem>();
		updatedList.add(new SelectItem("","Seleziona il codice CIA del fornitore"));
		
		retrievedSuppliersMap = new HashMap<String, Anagrafico>();
		
		// cerco anagrafici prima sul database di FE
		// select * from CIA_ANAGRAFICO where partita_iva='04245520376' and data_canc is null and tipo in ('B', 'C') and indicatore_entita <> 'U'
		String query = "SELECT * FROM CIA_ANAGRAFICO WHERE PARTITA_IVA='"+selectedInvoice.getSupplier().getVatId()+"' AND DATA_CANC IS NULL AND TIPO IN ('B', 'C') AND INDICATORE_ENTITA <> 'U'";
		List<Anagrafico> feRetrievedAnagrafici = JNDIUtils.getCrudManager().findByNativeQuery(query, Anagrafico.class);
		if ((feRetrievedAnagrafici != null) && !feRetrievedAnagrafici.isEmpty()) {
			for (Anagrafico anagrafico : feRetrievedAnagrafici) {
				retrievedSuppliersMap.put(cleanString(anagrafico.getCodice()), anagrafico);
			}
		}
		
		// cerco anche su CIA
		//		Map<String, Object> parameters = new HashMap<String, Object>();
		//		parameters.put("partita_iva", selectedInvoice.getSupplier().getVatId());
		//		List<String> tipo = new ArrayList<String>();
		//		tipo.add("B");
		//		tipo.add("C");
		//		parameters.put("tipo", tipo);
		//		List<Anagrafico> ciaRetrievedAnagrafici = UnimoreCIAGenericDAO.findByParameters(Anagrafico.class, parameters);
		query = "SELECT * FROM EPOCA_ANAGRAFICO WHERE PARTITA_IVA='"+selectedInvoice.getSupplier().getVatId()+"' AND DATA_CANC IS NULL AND TIPO IN ('B', 'C') AND INDICATORE_ENTITA <> 'U'";
		List<Anagrafico> ciaRetrievedAnagrafici = UnimoreCIAGenericDAO.findByNativeQuery(Anagrafico.class, query);
		if ( (ciaRetrievedAnagrafici != null) && (!ciaRetrievedAnagrafici.isEmpty())) {
			for (Anagrafico anagrafico : ciaRetrievedAnagrafici) {
				if (!retrievedSuppliersMap.containsValue(anagrafico)) {
					retrievedSuppliersMap.put(cleanString(anagrafico.getCodice()), anagrafico);
				}	
			}
		}
		
		// costruisco la lista di selectItem dalla mappa
		if(!retrievedSuppliersMap.isEmpty()) {
			Iterator<Entry<String, Anagrafico>> i = retrievedSuppliersMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, Anagrafico> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getCodice()),cleanString(entry.getValue().getCodice())));
			}
		}
		else {
			updatedList.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}
		
		supplierCodes = updatedList;
		
	}

	/**
	 * Aggiorna i codici di bilancio disponibili in base al profilo CIA dell'utente attualmente connesso
	 */
	private void updateCodiciBilancioDisponibili() {

		if ((this.esercizio == null) || (this.esercizio.equals(""))) {
			codiciBilancioDisponibili = new ArrayList<SelectItem>();
			return;
		}

		// estraggo gli identificativi di bilancio disponibili dai profili
		// utente CIA presenti in sessione HTTP
		List<String> userBudgets = ciaProfilesHandler.getUserBudgetsByFinancialYear(this.getEsercizio());

		List<SelectItem> updatedList = new ArrayList<SelectItem>();

		updatedList.add(new SelectItem("","Seleziona l'identificativo di bilancio"));
		
		// aggiungo tutti gli identificativi di bilancio alla lista di SelectItems
		// mi creo anche una struttura a mappa per accedere velocemente ai bilanci in fase di salvataggio dei dati aggiuntivi
		if ( (userBudgets != null) && (!userBudgets.isEmpty()) ) {
			for (String budget : userBudgets) {
				updatedList.add(new SelectItem(budget, budget));
			}
		}

		codiciBilancioDisponibili = updatedList;

		// MOCK
		// codiciBilancioDisponibili = new SelectItem[2];
		// for (int i = 0; i < codiciBilancioDisponibili.length; i++)
		// codiciBilancioDisponibili[i]=new
		// SelectItem("codBil_"+this.esercizio+"_"+i,"codBil_"+this.esercizio+"_"+i,"codBil_"+this.esercizio+"_"+i);
		// codiciBilancioDisponibili[0] = new SelectItem("",
		// "Seleziona bilancio");
		// codiciBilancioDisponibili[1] = new SelectItem("A.A00", "A.A00");
		// END OF MOCK

		// Map<String, Object> parameters = new HashMap<String, Object>();
		// parameters.put("esercizio", this.esercizio);
		// parameters.put("autonomia_finanziaria", "Y");
		//		
		// List<UnitaOrganizzativa> uos =
		// UnimoreCIAGenericDAO.findByParameters(UnitaOrganizzativa.class,
		// parameters);
		// if (uos != null){
		// List<SelectItem> out = new ArrayList<SelectItem>();
		// for (UnitaOrganizzativa uo : uos) {
		// out.add(new
		// SelectItem(uo.getIdentificativo_bilancio(),uo.getIdentificativo_bilancio()));
		// }
		// codiciBilancioDisponibili = out.toArray(new SelectItem[]{});
		// }
	}

	/**
	 * Updates SelectOneMenu items for uo codes
	 */
	private void updateCodiciUODisponibili() {
		if ((this.esercizio == null) || (this.esercizio.equals(""))) {
			codiciUODisponibili = new ArrayList<SelectItem>();
			return;
		}
		if ((this.codiceBilancio == null) || (this.codiceBilancio.equals(""))) {
			codiciUODisponibili = new ArrayList<SelectItem>();
			return;
		}

		// estraggo gli esercizi disponibili dai profili utente CIA presenti in
		// sessione HTTP
		List<String> userUniOrgs = ciaProfilesHandler.getUserUniOrgsByBudgetAndFinancialYear(this.getCodiceBilancio(), this.getEsercizio());

		List<SelectItem> updatedList = new ArrayList<SelectItem>();

		updatedList.add(new SelectItem("", "Seleziona il codice dell'unità organizzativa"));

		// aggiungo tutti gli altri codici UO presenti nel profilo cia
		// dell'utente (eventualmente escludendo quello prelevato
		// precedentemente da db)
		if ( (userUniOrgs != null) && (!userUniOrgs.isEmpty()) ) {
			for (String uo : userUniOrgs) {
				updatedList.add(new SelectItem(uo, uo));
			}
		}

		codiciUODisponibili = updatedList;

		// MOCK
		// codiciUODisponibili = new SelectItem[4];
		// codiciUODisponibili[0] = new SelectItem("",
		// "Seleziona l'unità organizativa");
		// codiciUODisponibili[1] = new SelectItem("A.A00.RIGE",
		// "A.A00.RIGE - Centro di Medicina Rigenerativa/Centro per la coltivazione di Cellule Staminali");
		// codiciUODisponibili[2] = new SelectItem("A.A00.DIAM.OPER.TECN",
		// "A.A00.DIAM.OPER.TECN - Direzione Tecnica");
		// codiciUODisponibili[3] = new SelectItem("A.A00.COST",
		// "A.A00.COST - Foresteria via delle COSTELLAZIONI");
		// END OF MOCK

	}

	/**
	 * Updates SelectOneMenu items for reasons
	 */
	private void updateAvailableReasons() {
		
		if ( this.isInvoiceLinesTogglePanelOpened() == false ) {
			availableReasons = new ArrayList<SelectItem>();
			return;
		}
		
		// since richfaces' comboBox renders as inputText only SelectItem.value are considered, not labels
		/*String jpqlQuery = "SELECT c FROM it.fe.core.par.unimore.cia.Causale c";
		List<Causale> dbRetrievedReasons = JNDIUtils.getCrudManager().findByQuery(jpqlQuery);*/
		
		/* CAUSALI HERA */
		/*String query = "SELECT * FROM CIA_CAUSALI";
		List<Causale> dbRetrievedReasons = JNDIUtils.getCrudManager().findByNativeQuery(query, Causale.class);
		availableReasons = new ArrayList<SelectItem>();
		retrievedReasonsMap = new HashMap<String, Causale>();
		availableReasons.add(new SelectItem("Seleziona la causale"));
		if ((dbRetrievedReasons!=null) && (!dbRetrievedReasons.isEmpty()) ) {
			for(Causale c : dbRetrievedReasons) {
				retrievedReasonsMap.put(c.getCodice(), c);
				availableReasons.add(new SelectItem(cleanString(c.getCodice())+" - "+cleanString(c.getDescrizione())));
			}
		}
		else {
			availableReasons.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}*/
		
		List<SelectItem> updatedList = new ArrayList<SelectItem>();
		updatedList.add(new SelectItem("Seleziona la causale"));
		retrievedReasonsMap = new HashMap<String, Causale>();
		
		// cerco prima sul db di fe
		//String query = "select * from epoca_causali where data_canc is null and spesa_entrata in ('spesa','spesa_entrata')";
		String query = 	"select * from CIA_CAUSALI where data_canc is null and spesa_entrata in ('spesa','spesa_entrata')";
		List<Causale> feRetrievedReasons = JNDIUtils.getCrudManager().findByNativeQuery(query, Causale.class);
		if ((feRetrievedReasons != null) && (!feRetrievedReasons.isEmpty()) ) {
			for (Causale c : feRetrievedReasons) {
				retrievedReasonsMap.put(cleanString(c.getCodice()), c);
			}
		}
		
		// cerco su CIA
		// select * from epoca_causali where data_canc is null and spesa_entrata in ('spesa','spesa_entrata')
		query = "select * from epoca_causali where data_canc is null and spesa_entrata in ('spesa','spesa_entrata')";
		List<Causale> ciaRetrievedReasons = UnimoreCIAGenericDAO.findByNativeQuery(Causale.class, query);
		if ((ciaRetrievedReasons != null) && (!ciaRetrievedReasons.isEmpty())) {
			for(Causale c : ciaRetrievedReasons) {
				if (!retrievedReasonsMap.containsValue(c)) {
					retrievedReasonsMap.put(cleanString(c.getCodice()), c);
				}
			}
			
		}
		
		// costruisco la lista di selectItem dalla mappa
		if (!retrievedReasonsMap.isEmpty()) {
			/*
			Iterator<Entry<String, CentroCosto>> i = retrievedCostCentersMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, CentroCosto> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getChiave_completa())+" - "+cleanString(entry.getValue().getNome())));
			}
			*/
			Map<String, Causale> sortedMap = new TreeMap<String, Causale>(retrievedReasonsMap);
			Iterator<Entry<String, Causale>> i = sortedMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, Causale> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getCodice())+" - "+cleanString(entry.getValue().getDescrizione())));
			}
		}	
		else {
			updatedList.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}
		
		availableReasons = updatedList;
		
	}

	/**
	 * Updates SelectOneMenu items for cost centers
	 */
	private void updateAvailableCostCenters() {
		
		if ((this.esercizio == null) || (this.esercizio.equals("")) || (this.isInvoiceLinesTogglePanelOpened() == false) ) {
			availableCostCentersSelectItems = new ArrayList<SelectItem>();
			return;
		}
		
		List<SelectItem> updatedList = new ArrayList<SelectItem>();
		updatedList.add(new SelectItem("Seleziona il centro di costo"));
		retrievedCostCentersMap = new HashMap<String, CentroCosto>();
		
		// cerco prima sul db di fe
		//String query = "SELECT * FROM CIA_CENTRO_COSTO WHERE ESERCIZIO='"+esercizio+"' AND ULTIMO_LIVELLO='Y'";
		String query = 	"select * from CIA_CENTRO_COSTO " +
						"where (CHIAVE_COMPLETA like 'C.8.MO%' or CHIAVE_COMPLETA in ('C.9.07.02','C.9.07.03')) " +
						"and ESERCIZIO='"+esercizio+"' " +
						"and DATA_CANC is null " +
						"and ULTIMO_LIVELLO='Y'";
		List<CentroCosto> feRetrievedCostCenters = JNDIUtils.getCrudManager().findByNativeQuery(query, CentroCosto.class);
		if ((feRetrievedCostCenters != null) && (!feRetrievedCostCenters.isEmpty()) ) {
			for (CentroCosto cc : feRetrievedCostCenters) {
				retrievedCostCentersMap.put(cleanString(cc.getChiave_completa()), cc);
			}
		}
		
		// cerco su CIA
		// select * from epoca_centro_costo where esercizio=<esercizioselezionato> and ultimo_livello='Y'
		// select * from epoca_centro_costo where (chiave_completa like 'C.8.MO%' or chiave_completa in ('C.9.07.02','C.9.07.03')) and esercizio='2010' and data_canc is null
		query = "select * from epoca_centro_costo " +
				"where (chiave_completa like 'C.8.MO%' or chiave_completa in ('C.9.07.02','C.9.07.03')) " +
				"and esercizio='"+esercizio+"' " +
				"and data_canc is null " +
				"and ultimo_livello='Y'";
		//Map<String, Object> parameters = new HashMap<String, Object>();
		//parameters.put("esercizio", esercizio);
		//parameters.put("ultimo_livello", "Y");
		//parameters.put("chiave_completa", "C.8.MO%");
		//List<CentroCosto> ciaRetrievedCostCenters = UnimoreCIAGenericDAO.findByParameters(CentroCosto.class, parameters);
		List<CentroCosto> ciaRetrievedCostCenters = UnimoreCIAGenericDAO.findByNativeQuery(CentroCosto.class, query);
		if ((ciaRetrievedCostCenters != null) && (!ciaRetrievedCostCenters.isEmpty())) {
			for(CentroCosto cc : ciaRetrievedCostCenters) {
				if (!retrievedCostCentersMap.containsValue(cc)) {
					retrievedCostCentersMap.put(cleanString(cc.getChiave_completa()), cc);
				}
			}
			
		}
		
		// costruisco la lista di selectItem dalla mappa
		if (!retrievedCostCentersMap.isEmpty()) {
			/*
			Iterator<Entry<String, CentroCosto>> i = retrievedCostCentersMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, CentroCosto> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getChiave_completa())+" - "+cleanString(entry.getValue().getNome())));
			}
			*/
			Map<String, CentroCosto> sortedMap = new TreeMap<String, CentroCosto>(retrievedCostCentersMap);
			Iterator<Entry<String, CentroCosto>> i = sortedMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, CentroCosto> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getChiave_completa())+" - "+cleanString(entry.getValue().getNome())));
			}
		}	
		else {
			updatedList.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}
		
		availableCostCentersSelectItems = updatedList;
		

	}

	public void reasonChangedListener(ValueChangeEvent event) {

	}

	public void cdrChangedListener(ValueChangeEvent event) {

	}

	/**
	 * Seleziona/deseleziona i checkbox delle righe fattura
	 */
	public void selectAll() {
		boolean newValue;
		if(isCurrentAllChecked())
			newValue = false;
		else
			newValue = true;
		Iterator<Map.Entry<Long, Boolean>> i = checked.entrySet().iterator();
		while (i.hasNext()) {
			Map.Entry<Long, Boolean> pair = i.next();
			pair.setValue(newValue);
		}
		setCurrentAllChecked(newValue);
	}

	/**
	 * Copia la causale in tutti i campi sottostanti
	 */
	public void copyReason() {
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (checked.get(line.getInvoiceLineId())) {
				// setto il value della selectItem della riga corrente
				lineReasons.put(new Long(line.getInvoiceLineId()), reasonToCopy);
			}
		}
	}

	/**
	 * Copia il centro di costo in tutti i campi sottostanti
	 */
	public void copyCostCenter() {
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (checked.get(line.getInvoiceLineId())) {
				// setto il value della selectItem della riga corrente
				lineCostCenters.put(new Long(line.getInvoiceLineId()), costCenterToCopy);
			}
		}
	}
	
	public void copyGood() {
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (checked.get(line.getInvoiceLineId())) {
				// setto il value della selectItem della riga corrente
				lineGoods.put(new Long(line.getInvoiceLineId()), goodToCopy);
			}
		}
	}
	
	
	
	public void resetAllFields() {
		this.setSupplierCodes(null);
		this.setRetrievedSuppliersMap(null);
		this.setSupplierCodes(null);

		this.setEserciziDisponibili(null);
		this.setEsercizio(null);

		this.setCodiciBilancioDisponibili(null);
		this.setCodiceBilancio(null);

		this.setCodiciUODisponibili(null);
		this.setCodiceUO(null);
		
		this.setAvailableFinancialAccounts(null);
		this.setFinancialAccount(null);
		
		this.setAvailablePaymentMethods(null);
		this.setPaymentMethod(null);
		this.setRetrievedPaymentMethodsMap(null);
		
		this.setAvailableBankAccounts(null);
		this.setBankAccount(null);

		checked = new HashMap<Long, Boolean>();
		currentAllChecked = false;
		
		this.setLineReasons(null);
		this.setAvailableReasons(null);
		this.setRetrievedReasonsMap(null);

		this.setLineCostCenters(null);
		this.setAvailableCostCentersSelectItems(null);
		this.setRetrievedCostCentersMap(null);
		
		this.setLineGoods(null);
		this.setAvailableGoodsSelectItems(null);
		this.setRetrievedGoodsMap(null);
		
		this.setReasonToCopy("Seleziona la causale");
		this.setCostCenterToCopy("Seleziona il centro di costo");
		
	}

	public String getCodiceBilancio() {
			// verificare se la fattura possiede dei dati aggiuntivi
			if(!this.isEditingState()) {
				if (this.invoiceHasSavedAdditionalData(selectedInvoice,UnitaOrganizzativa.class)) {
					List<UnitaOrganizzativa> storedAdditionalData = this.getInvoiceStoredAdditionalData(selectedInvoice, UnitaOrganizzativa.class);
					if ( (storedAdditionalData != null ) && (!storedAdditionalData.isEmpty()) ) {
						// trattandosi del codice del bilancio che ha creato la fattura
						// mi aspetto che sia uno solo
						codiceBilancio = storedAdditionalData.get(0).getIdentificativo_bilancio();
					}
				}
			}
		return codiceBilancio;
	}

	public void setCodiceBilancio(String codiceBilancio) {
		this.codiceBilancio = codiceBilancio;
	}

	public String getCodiceUO() {
			// verificare se la fattura possiede dei dati aggiuntivi.
		if(!this.isEditingState()) {
			if (this.invoiceHasSavedAdditionalData(selectedInvoice, UnitaOrganizzativa.class)) {
				List<UnitaOrganizzativa> storedAdditionalData = this.getInvoiceStoredAdditionalData(selectedInvoice, UnitaOrganizzativa.class);
				if (!storedAdditionalData.isEmpty()) {
					// trattandosi dell'esercizio di creazione della fattura mi
					// aspetto che sia uno solo
					codiceUO = storedAdditionalData.get(0).getChiave_completa();
				}
			}
		}
		return codiceUO;
	}

	public void setCodiceUO(String codiceUO) {
		this.codiceUO = codiceUO;
	}

	public String getEsercizio() {
		// verificare se la fattura possiede dei dati aggiuntivi.
		if (!this.isEditingState()) {
			if (this.invoiceHasSavedAdditionalData(selectedInvoice,
					UnitaOrganizzativa.class)) {
				List<UnitaOrganizzativa> storedAdditionalData = this
						.getInvoiceStoredAdditionalData(selectedInvoice,
								UnitaOrganizzativa.class);
				if ((storedAdditionalData != null)
						&& (!storedAdditionalData.isEmpty())) {
					// trattandosi dell'esercizio di creazione della fattura mi
					// aspetto che sia uno solo
					esercizio = storedAdditionalData.get(0).getEsercizio();
				}

			}
		}
		return esercizio;
	}

	public void setEsercizio(String esercizio) {
		this.esercizio = esercizio;
	}

	public String getSupplierCode() {
		if (!this.isEditingState()) {
			// verificare se la fattura possiede dei dati aggiuntivi.
			if (this.invoiceHasSavedAdditionalData(selectedInvoice, Anagrafico.class)) {
				List<Anagrafico> storedAdditionalData = this.getInvoiceStoredAdditionalData(selectedInvoice, Anagrafico.class);
				if ( (storedAdditionalData != null) && (!storedAdditionalData.isEmpty()) ) {
					// trattandosi dell'anagrafico del fornitore mi aspetto che sia uno solo
					supplierCode = storedAdditionalData.get(0).getCodice();
				}
			}
		}	
		return supplierCode;
	}

	public void setSupplierCode(String supplierCode) {
		this.supplierCode = supplierCode;
	}

	/**
	 * @return the reasonToCopy
	 */
	public String getReasonToCopy() {
		return reasonToCopy;
	}

	/**
	 * @param reasonToCopy
	 *            the reasonToCopy to set
	 */
	public void setReasonToCopy(String reasonToCopy) {
		this.reasonToCopy = reasonToCopy;
	}

	/**
	 * @return the eserciziDisponibili
	 */
	public List<SelectItem> getEserciziDisponibili() {
		if (eserciziDisponibili == null)
			updateEserciziDisponibili();
		return eserciziDisponibili;
	}

	/**
	 * @param eserciziDisponibili
	 *            the eserciziDisponibili to set
	 */
	public void setEserciziDisponibili(List<SelectItem> eserciziDisponibili) {
		this.eserciziDisponibili = eserciziDisponibili;
	}

	/**
	 * @return the supplierCodes
	 */
	public List<SelectItem> getSupplierCodes() {
		if (supplierCodes == null)
			updateSupplierCodes();
		return supplierCodes;
	}

	/**
	 * @param supplierCodes
	 *            the supplierCodes to set
	 */
	public void setSupplierCodes(List<SelectItem> supplierCodes) {
		this.supplierCodes = supplierCodes;
	}

	/**
	 * @return the codiciBilancioDisponibili
	 */
	public List<SelectItem> getCodiciBilancioDisponibili() {
		if (codiciBilancioDisponibili == null)
			updateCodiciBilancioDisponibili();
		return codiciBilancioDisponibili;
	}

	/**
	 * @param codiciBilancioDisponibili
	 *            the codiciBilancioDisponibili to set
	 */
	public void setCodiciBilancioDisponibili(
			List<SelectItem> codiciBilancioDisponibili) {
		this.codiciBilancioDisponibili = codiciBilancioDisponibili;
	}

	/**
	 * @return the codiciUODisponibili
	 */
	public List<SelectItem> getCodiciUODisponibili() {
		if (codiciUODisponibili == null)
			updateCodiciUODisponibili();
		return codiciUODisponibili;
	}

	/**
	 * @param codiciUODisponibili
	 *            the codiciUODisponibili to set
	 */
	public void setCodiciUODisponibili(List<SelectItem> codiciUODisponibili) {
		this.codiciUODisponibili = codiciUODisponibili;
	}


	/**
	 * @return the checked
	 */
	public Map<Long, Boolean> getChecked() {
		return checked;
	}

	/**
	 * @param checked
	 *            the checked to set
	 */
	public void setChecked(Map<Long, Boolean> checked) {
		this.checked = checked;
	}

	/**
	 * @return the costCenterToCopy
	 */
	public String getCostCenterToCopy() {
		return costCenterToCopy;
	}

	/**
	 * @param costCenterToCopy
	 *            the costCenterToCopy to set
	 */
	public void setCostCenterToCopy(String costCenterToCopy) {
		this.costCenterToCopy = costCenterToCopy;
	}

	private void updateLineReasons() {
		
		if ( this.isInvoiceLinesTogglePanelOpened() == false ) {
			lineReasons = new HashMap<Long, String>();
			return;
		}
		
		
		lineReasons = new HashMap<Long, String>();
		
		try {
			for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
				List<Causale> retrievedLineReasons = this.getInvoiceLineStoredAdditionalData(line, Causale.class);
				if (!retrievedLineReasons.isEmpty()) {
					// prendo la prima perchè la causale è sempre una sola
					lineReasons.put(new Long(line.getInvoiceLineId()), retrievedLineReasons.get(0).getCodice()+" - "+retrievedLineReasons.get(0).getDescrizione());
				}
				else {
					lineReasons.put(new Long(line.getInvoiceLineId()), "");
				}
				
			}
		} catch (LazyInitializationException e) {
			return;
		}
		
	}

	/**
	 * @return the lineReasons
	 */
	public Map<Long, String> getLineReasons() {
		if (lineReasons == null) {
			this.updateLineReasons();
		}
		return lineReasons;
	}

	/**
	 * @param lineReasons the lineReasons to set
	 */
	public void setLineReasons(Map<Long, String> lineReasons) {
		this.lineReasons = lineReasons;
	}

	/**
	 * @return the currentAllChecked
	 */
	public boolean isCurrentAllChecked() {
		return currentAllChecked;
	}

	/**
	 * @param currentAllChecked the currentAllChecked to set
	 */
	public void setCurrentAllChecked(boolean currentAllChecked) {
		this.currentAllChecked = currentAllChecked;
	}

	/**
	 * @return the lineCostCenters
	 */
	public Map<Long, String> getLineCostCenters() {
		if (lineCostCenters == null) {
			updateLineCostCenters();
		}
		return lineCostCenters;
	}

	/**
	 * @param lineCostCenters the lineCostCenters to set
	 */
	public void setLineCostCenters(Map<Long, String> lineCostCenters) {
		this.lineCostCenters = lineCostCenters;
	}
	
	private void updateLineCostCenters() {
		
		if ((this.esercizio == null) || (this.esercizio.equals("")) || (this.isInvoiceLinesTogglePanelOpened() == false) ) {
			lineCostCenters = new HashMap<Long, String>();
			return;
		}
		
		
		try {
			lineCostCenters = new HashMap<Long, String>();
			for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
				List<CentroCosto> retrievedCostCenters = this.getInvoiceLineStoredAdditionalData(line, CentroCosto.class);
				if (!retrievedCostCenters.isEmpty()) {
					// prendo la prima perchè la causale è sempre una sola
					lineCostCenters.put(new Long(line.getInvoiceLineId()), retrievedCostCenters.get(0).getChiave_completa()+" - "+retrievedCostCenters.get(0).getNome());
				}
				else {
					lineCostCenters.put(new Long(line.getInvoiceLineId()), "");
				}		
			}
		} catch (LazyInitializationException e) {
			return;
		}
		
	}

	/**
	 * @return the availableCostCentersSelectItems
	 */
	public List<SelectItem> getAvailableCostCentersSelectItems() {
		if (availableCostCentersSelectItems == null) {
			updateAvailableCostCenters();
		}
		return availableCostCentersSelectItems;
	}

	/**
	 * @param availableCostCentersSelectItems the availableCostCentersSelectItems to set
	 */
	public void setAvailableCostCentersSelectItems(
			List<SelectItem> availableCostCentersSelectItems) {
		this.availableCostCentersSelectItems = availableCostCentersSelectItems;
	}

	/**
	 * @return the availableReasons
	 */
	public List<SelectItem> getAvailableReasons() {
		if (availableReasons == null)
			this.updateAvailableReasons();
		return availableReasons;
	}

	/**
	 * @param availableReasons the availableReasons to set
	 */
	public void setAvailableReasons(List<SelectItem> availableReasons) {
		this.availableReasons = availableReasons;
	}
	
	private void updateAvailablePaymentMethods() {
		
		if ((this.selectedInvoice == null)
				|| (this.selectedInvoice.getSupplier() == null)
				|| (this.selectedInvoice.getSupplier().getVatId() == null)) {
			availablePaymentMethods = new ArrayList<SelectItem>();
			return;
		}
		
		if ((this.supplierCode == null) || (this.supplierCode.equals(""))) {
			availablePaymentMethods = new ArrayList<SelectItem>();
			return;
		}
		
		// 		MOCK
		//		modPagamentoDisponibili = new SelectItem[6];
		//		modPagamentoDisponibili[0] = new SelectItem("",
		//				"Seleziona la modalità di pagamento");
		//		modPagamentoDisponibili[1] = new SelectItem(
		//				"CC - Conto Corrente Bancario", "CC - Conto Corrente Bancario");
		//		modPagamentoDisponibili[2] = new SelectItem("BP - Banco Posta",
		//				"BP - Banco Posta");
		//		modPagamentoDisponibili[3] = new SelectItem(
		//				"CP - Conto Corrente Postale", "CP - Conto Corrente Postale");
		//		modPagamentoDisponibili[4] = new SelectItem("AC - Assegno Circolare",
		//				"AC - Assegno Circolare");
		//		modPagamentoDisponibili[5] = new SelectItem("CT - Cassa Tesoriere",
		//				"CT - Cassa Tesoriere");
		
		List<SelectItem> updatedList = new ArrayList<SelectItem>();
		updatedList.add(new SelectItem("", "Seleziona la modalità di pagamento"));
		retrievedPaymentMethodsMap = new HashMap<String, ModalitaPagamento>();
		
		// cerco modalità di pagamento prima sul database di FE
		String query = "SELECT * FROM CIA_MODALITA_PAGAMENTO WHERE CODICE='"+supplierCode+"'";
		List<ModalitaPagamento> feRetrievedModalitaPagamento = JNDIUtils.getCrudManager().findByNativeQuery(query, ModalitaPagamento.class);
		if ((feRetrievedModalitaPagamento != null) && (!feRetrievedModalitaPagamento.isEmpty()) ) {
			for (ModalitaPagamento modalitaPagamento : feRetrievedModalitaPagamento) {
				retrievedPaymentMethodsMap.put(cleanString(modalitaPagamento.getCodice_rif()), modalitaPagamento);
			}
		}
		
		// cerco su CIA
		Map<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("codice", supplierCode);
		List<ModalitaPagamento> ciaRetrievedModalitaPagamento = UnimoreCIAGenericDAO.findByParameters(ModalitaPagamento.class, parameters);
		if ( (ciaRetrievedModalitaPagamento != null) && (!ciaRetrievedModalitaPagamento.isEmpty())) {
			for (ModalitaPagamento modalitaPagamento : ciaRetrievedModalitaPagamento) {
				if (!retrievedPaymentMethodsMap.containsValue(modalitaPagamento)) {
					retrievedPaymentMethodsMap.put(cleanString(modalitaPagamento.getCodice_rif()), modalitaPagamento);
				}	
			}
		}
		
		// costruisco la lista di selectItem dalla mappa
		if(!retrievedPaymentMethodsMap.isEmpty()) {
			Iterator<Entry<String, ModalitaPagamento>> i = retrievedPaymentMethodsMap.entrySet().iterator();
			List<String> codiciFornitoriUtenze = Arrays.asList(UnimoreConfiguration.ciaAnagraficoUtenze);
			while (i.hasNext()) {
								
				Entry<String, ModalitaPagamento> entry = i.next();
				String codice_rif = cleanString(entry.getValue().getCodice_rif());
				
				// NB. per i fornitori delle utenze l'unica modalità di pagamento impiegata è CC
				if ("CT".equals(codice_rif) && codiciFornitoriUtenze.contains(supplierCode)) {
					continue;
				}
				else {
					updatedList.add(
							new SelectItem(codice_rif, codice_rif));
				}
				
			}
		}
		else {
			updatedList.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}
		
		availablePaymentMethods = updatedList;
		
	}

	/**
	 * @return the availablePaymentMethods
	 */
	public List<SelectItem> getAvailablePaymentMethods() {
		if (availablePaymentMethods == null) {
			updateAvailablePaymentMethods();
		}
		return availablePaymentMethods;
	}

	/**
	 * @param availablePaymentMethods the availablePaymentMethods to set
	 */
	public void setAvailablePaymentMethods(List<SelectItem> availablePaymentMethods) {
		this.availablePaymentMethods = availablePaymentMethods;
	}

	/**
	 * @return the paymentMethod
	 */
	public String getPaymentMethod() {
		try {
			//if (FacesContext.getCurrentInstance().getRenderResponse()) {
				// verificare se la fattura possiede dei dati aggiuntivi.
				if (this.invoiceHasSavedAdditionalData(selectedInvoice, ModalitaPagamento.class)) {
					List<ModalitaPagamento> storedAdditionalData = this.getInvoiceStoredAdditionalData(selectedInvoice, ModalitaPagamento.class);
					if ( (storedAdditionalData!=null) && (!storedAdditionalData.isEmpty()) ) {
						// trattandosi della modalità di pagamento della fattura mi aspetto che sia una sola
						paymentMethod = storedAdditionalData.get(0).getCodice_rif();
					}

				}
			//}
		} catch (LazyInitializationException e) {
			paymentMethod = "";
		}
		
		return paymentMethod;
	}

	/**
	 * @param paymentMethod the paymentMethod to set
	 */
	public void setPaymentMethod(String paymentMethod) {
		this.paymentMethod = paymentMethod;
	}

	/**
	 * @return the retrievedPaymentMethodsMap
	 */
	public Map<String, ModalitaPagamento> getRetrievedPaymentMethodsMap() {
		return retrievedPaymentMethodsMap;
	}

	/**
	 * @param retrievedPaymentMethodsMap the retrievedPaymentMethodsMap to set
	 */
	public void setRetrievedPaymentMethodsMap(
			Map<String, ModalitaPagamento> retrievedPaymentMethodsMap) {
		this.retrievedPaymentMethodsMap = retrievedPaymentMethodsMap;
	}

	/**
	 * @return the availableBankAccounts
	 */
	public List<SelectItem> getAvailableBankAccounts() {
		if (availableBankAccounts == null) {
			updateAvailableBankAccounts();
		}
		return availableBankAccounts;
	}

	private void updateAvailableBankAccounts() {
		if ((this.selectedInvoice == null)
				|| (this.selectedInvoice.getSupplier() == null)
				|| (this.selectedInvoice.getSupplier().getVatId() == null)) {
			availableBankAccounts = new ArrayList<SelectItem>();
			return;
		}
		
		if ((this.supplierCode == null) || (this.supplierCode.equals(""))) {
			availableBankAccounts = new ArrayList<SelectItem>();
			return;
		}
		
		if ( (this.paymentMethod == null) || (this.paymentMethod.equals("")) || (!this.paymentMethod.equals("CC")) )  {
			availableBankAccounts = new ArrayList<SelectItem>();
			return;
		}
		
		// 		MOCK
		//		ccDisponibili = new SelectItem[3];
		//		ccDisponibili[0] = new SelectItem("", "Seleziona il conto corrente");
		//		ccDisponibili[1] = new SelectItem(
		//				"BANCA MONTE DEI PASCHI DI SIENA SPA - SUCC. DI BOLOGNA; IBAN: IT71H0103002400000004746870",
		//				"BANCA MONTE DEI PASCHI DI SIENA SPA - SUCC. DI BOLOGNA; IBAN: IT71H0103002400000004746870");
		//		ccDisponibili[2] = new SelectItem(
		//				"BANCO DI SICILIA - SEDE; IBAN: IT55E0102002400000000307050",
		//				"BANCO DI SICILIA - SEDE; IBAN: IT55E0102002400000000307050");
		
		List<SelectItem> updatedList = new ArrayList<SelectItem>();
		retrievedBankAccountsMap = new HashMap<String, Banca>();
		
		// cerco Conti Bancari prima sul database di FE
		String query = "SELECT * FROM CIA_BANCA WHERE CODICE='"+supplierCode+"'";
		List<Banca> feRetrievedBankAccounts = JNDIUtils.getCrudManager().findByNativeQuery(query, Banca.class);
		if ((feRetrievedBankAccounts != null) && (!feRetrievedBankAccounts.isEmpty()) ) {
			for (Banca ba : feRetrievedBankAccounts) {
				retrievedBankAccountsMap.put(cleanString(ba.getCodice())+"-"+cleanString(ba.getDescrizione()), ba);
			}
		}
		
		// cerco su CIA
		
		Map<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("codice", supplierCode);
		List<Banca> ciaRetrievedBankAccounts = UnimoreCIAGenericDAO.findByParameters(Banca.class, parameters);
		if ( (ciaRetrievedBankAccounts != null) && (!ciaRetrievedBankAccounts.isEmpty())) {
			for (Banca ba : ciaRetrievedBankAccounts) {
				if(!retrievedBankAccountsMap.containsValue(ba)) {
					retrievedBankAccountsMap.put(cleanString(ba.getCodice())+"-"+cleanString(ba.getDescrizione()), ba);
				}
			}
		}
		
		// costruisco la lista di selectItem dalla mappa
		if(!retrievedBankAccountsMap.isEmpty()) {
			Iterator<Entry<String, Banca>> i = retrievedBankAccountsMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, Banca> entry = i.next();
				updatedList.add(
						new SelectItem(
								cleanString(entry.getValue().getCodice())+"-"+cleanString(entry.getValue().getDescrizione()), 
								cleanString(entry.getValue().getDescrizione()) +" - "+cleanString(entry.getValue().getAbi())+" - "+cleanString(entry.getValue().getCab())+" - "+cleanString(entry.getValue().getNumeroconto()) ));
			}
		}
		else {
			updatedList.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}
		
		availableBankAccounts = updatedList;
	}
	
	/**
	 * @return the availableBankAccounts
	 */
	public List<SelectItem> getAvailableFinancialAccounts() {
		if (availableFinancialAccounts == null) {
			updateAvailableFinancialAccounts();
		}
		return availableFinancialAccounts;
	}
	
	private void updateAvailableFinancialAccounts() {
		// NOTA BENE: in Amministrazione Centrale a UniMoRe i conti contabili sono filtrabili in base al tipo di fornitore, per questo vedi UnimoreConfiguration
		// la select su db avviene comunque secondo i criteri forniti dalla documentazione (anche per esercizio, mastrino e sezione)
		
		if ((this.selectedInvoice == null)
				|| (this.selectedInvoice.getSupplier() == null)
				|| (this.selectedInvoice.getSupplier().getVatId() == null)) {
			availableFinancialAccounts = new ArrayList<SelectItem>();
			return;
		}
		
		if ((this.supplierCode == null) || (this.supplierCode.equals(""))) {
			availableFinancialAccounts = new ArrayList<SelectItem>();
			return;
		}
		
		if ((this.esercizio == null) || (this.esercizio.equals(""))) {
			availableFinancialAccounts = new ArrayList<SelectItem>();
			return;
		}
		
		// ottengo da UnimoreConfiguration i conti contabili compatibili con il fornitore selezionato
		String accountKeysList = "";
		// HERA SpA oppure HERA COMM S.R.L.
		if ( supplierCode.equals("31285") || supplierCode.equals("36640") )  {
			for (int i=0; i<UnimoreConfiguration.ciaContiContabiliHERA.length; i++) {
				accountKeysList += "'"+UnimoreConfiguration.ciaContiContabiliHERA[i]+"'";
				if (i != UnimoreConfiguration.ciaContiContabiliHERA.length - 1 ) {
					accountKeysList += ", ";
				}
			}
		}
		
		List<SelectItem> updatedList = new ArrayList<SelectItem>();
		updatedList.add(new SelectItem("", "Seleziona il conto contabile"));
		retrievedFinancialAccountsMap = new HashMap<String, ContoContabile>();
		
		// cerco conto_contabile prima sul database di FE
		// select * from CIA_CONTO_CONTABILE 
		// where esercizio='2010' and mastrino='Y' and sezione='A'
		// and chiave_completa is in (LISTA_CHIAVI)
		// order by chiave_completa asc
		String query = 
			"select * from CIA_CONTO_CONTABILE " +
			"where esercizio='"+esercizio+"' and mastrino='Y' and sezione='A' ";
		if (!accountKeysList.equals("")) {
			query = query.concat("and chiave_completa in ("+accountKeysList+") ");
		}
		query = query.concat("order by chiave_completa asc");
		
		List<ContoContabile> feRetrievedContoContabile = JNDIUtils.getCrudManager().findByNativeQuery(query, ContoContabile.class);
		if ((feRetrievedContoContabile != null) && (!feRetrievedContoContabile.isEmpty()) ) {
			for (ContoContabile contoContabile : feRetrievedContoContabile) {
				retrievedFinancialAccountsMap.put(cleanString(contoContabile.getChiave_completa()), contoContabile);
			}
		}
		
		// cerco su CIA
		query = 
			"select * from epoca_conto_contabile " +
			"where esercizio='"+esercizio+"' and mastrino='Y' and sezione='A' ";
		if (!accountKeysList.equals("")) {
			query = query.concat("and chiave_completa in ("+accountKeysList+") ");
		}
		query = query.concat("order by chiave_completa asc");
		
		List<ContoContabile> ciaRetrievedFinancialAccounts = UnimoreCIAGenericDAO.findByNativeQuery(ContoContabile.class, query);
		if ((ciaRetrievedFinancialAccounts != null) && (!ciaRetrievedFinancialAccounts.isEmpty())) {
			for(ContoContabile cc : ciaRetrievedFinancialAccounts) {
				if (!retrievedFinancialAccountsMap.containsValue(cc)) {
					retrievedFinancialAccountsMap.put(cleanString(cc.getChiave_completa()), cc);
				}
			}
		}
		
		// costruisco la lista di selectItem dalla mappa
		if(!retrievedFinancialAccountsMap.isEmpty()) {
			Iterator<Entry<String, ContoContabile>> i = retrievedFinancialAccountsMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, ContoContabile> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getChiave_completa()), cleanString(entry.getValue().getChiave_completa())+" - "+cleanString(entry.getValue().getNome_conto())));
			}
		}
		else {
			updatedList.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}
		
		availableFinancialAccounts = updatedList;
		
	}

	private String cleanSpecialCharacters(String input) {
		String output = input;
		output = output.replace(".", "");
		output = output.replace("'", " ");
		return output;
	}

	/**
	 * @param availableBankAccounts the availableBankAccounts to set
	 */
	public void setAvailableBankAccounts(List<SelectItem> availableBankAccounts) {
		this.availableBankAccounts = availableBankAccounts;
	}

	/**
	 * @return the bankAccount
	 */
	public String getBankAccount() {
		try {
			//if (FacesContext.getCurrentInstance().getRenderResponse()) {
			// verificare se la fattura possiede dei dati aggiuntivi.
			if (this.invoiceHasSavedAdditionalData(selectedInvoice, Banca.class)) {
				List<Banca> storedAdditionalData = this.getInvoiceStoredAdditionalData(selectedInvoice, Banca.class);
				if ( (storedAdditionalData!=null) && (!storedAdditionalData.isEmpty()) ) {
					// trattandosi della modalità di pagamento della fattura mi aspetto che sia una sola
					bankAccount = storedAdditionalData.get(0).getCodice()+"-"+storedAdditionalData.get(0).getDescrizione();
				}
			}
		//}
			
		} catch (LazyInitializationException e) {
			bankAccount = "";
		}
		return bankAccount;
	}

	/**
	 * @param bankAccount the bankAccount to set
	 */
	public void setBankAccount(String bankAccount) {
		this.bankAccount = bankAccount;
	}

	/**
	 * @return the retrievedBankAccountsMap
	 */
	public Map<String, Banca> getRetrievedBankAccountsMap() {
		return retrievedBankAccountsMap;
	}

	/**
	 * @param retrievedBankAccountsMap the retrievedBankAccountsMap to set
	 */
	public void setRetrievedBankAccountsMap(
			Map<String, Banca> retrievedBankAccountsMap) {
		this.retrievedBankAccountsMap = retrievedBankAccountsMap;
	}

	/**
	 * @return the supplierCodeSaved
	 */
	public boolean isSupplierCodeSaved() {
		return this.invoiceHasSavedAdditionalData(selectedInvoice, Anagrafico.class);
	}

	/**
	 * @param supplierCodeSaved the supplierCodeSaved to set
	 */
	public void setSupplierCodeSaved(boolean supplierCodeSaved) {
		this.supplierCodeSaved = supplierCodeSaved;
	}

	/**
	 * @return the financialYearSaved
	 */
	public boolean isFinancialYearSaved() {
		return this.invoiceHasSavedAdditionalData(selectedInvoice, UnitaOrganizzativa.class);
	}

	/**
	 * @param financialYearSaved the financialYearSaved to set
	 */
	public void setFinancialYearSaved(boolean financialYearSaved) {
		this.financialYearSaved = financialYearSaved;
	}

	/**
	 * @return the budgetSaved
	 */
	public boolean isBudgetSaved() {
		return this.invoiceHasSavedAdditionalData(selectedInvoice, UnitaOrganizzativa.class);
	}

	/**
	 * @param budgetSaved the budgetSaved to set
	 */
	public void setBudgetSaved(boolean budgetSaved) {
		this.budgetSaved = budgetSaved;
	}

	/**
	 * @return the uoSavede
	 */
	public boolean isUoSaved() {
		return this.invoiceHasSavedAdditionalData(selectedInvoice, UnitaOrganizzativa.class);
	}

	/**
	 * @param uoSaved the uoSaved to set
	 */
	public void setUoSaved(boolean uoSaved) {
		this.uoSaved = uoSaved;
	}

	/**
	 * @return the paymentMethodSaved
	 */
	public boolean isPaymentMethodSaved() {
		return this.invoiceHasSavedAdditionalData(selectedInvoice, ModalitaPagamento.class);
	}

	/**
	 * @param paymentMethodSaved the paymentMethodSaved to set
	 */
	public void setPaymentMethodSaved(boolean paymentMethodSaved) {
		this.paymentMethodSaved = paymentMethodSaved;
	}

	/**
	 * @return the bankAccountSaved
	 */
	public boolean isBankAccountSaved() {
		return this.invoiceHasSavedAdditionalData(selectedInvoice, Banca.class);
	}

	/**
	 * @param bankAccountSaved the bankAccountSaved to set
	 */
	public void setBankAccountSaved(boolean bankAccountSaved) {
		this.bankAccountSaved = bankAccountSaved;
	}

	/**
	 * @return the invoiceLinesHasAdditionalDataSaved
	 */
	public boolean isInvoiceLinesHasAdditionalDataSaved() {
		
		List<InvoiceLine> invoiceLines = selectedInvoice.getInvoiceLines();
		
		// lazy fetching and caching for furter requests
		if (! (invoiceLines instanceof ArrayList) ) {
			List<InvoiceLine> lazilyFetchedInvoiceLines = JNDIUtils.getInvoiceManager().fetchInvoiceLines(selectedInvoice);
			for (InvoiceLine il : lazilyFetchedInvoiceLines) {
				List<AdditionalData> lazilyFetchedInvoiceLineAdditionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(il);
				il.setAdditionalData(lazilyFetchedInvoiceLineAdditionalData);
			}
			this.selectedInvoice.setInvoiceLines(lazilyFetchedInvoiceLines);
		}
		
		// itero su tutte le righe e verifico se hanno sia Causale, CentroCosto, BeneServizio, ContoContabile salvati. Alla prima falsa esco per ottimizzare.
		boolean out = false;
		for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
			if (
					this.invoiceLineHasSavedAdditionalData(line, Causale.class) 
						&& 
					this.invoiceLineHasSavedAdditionalData(line, CentroCosto.class)
						&&
					this.invoiceLineHasSavedAdditionalData(line, BeneServizio.class)
						&&
					this.invoiceLineHasSavedAdditionalData(line, ContoContabile.class)) 
			{
				out = true;
			}
			else {
				out = false;
				break;
			}
		}
		return out;
	}

	/**
	 * @param invoiceLinesHasAdditionalDataSaved the invoiceLinesHasAdditionalDataSaved to set
	 */
	public void setInvoiceLinesHasAdditionalDataSaved(boolean invoiceLinesHasAdditionalDataSaved) {
		this.invoiceLinesHasAdditionalDataSaved = invoiceLinesHasAdditionalDataSaved;
	}

	/**
	 * @return the approvalModalPanelRendered
	 */
	public boolean isApprovalModalPanelRendered() {
		return approvalModalPanelRendered;
	}

	/**
	 * @param approvalModalPanelRendered the approvalModalPanelRendered to set
	 */
	public void setApprovalModalPanelRendered(boolean approvalModalPanelRendered) {
		this.approvalModalPanelRendered = approvalModalPanelRendered;
	}
	
	/**
	 * Method used to retrieve and return additional data by class type
	 * 
	 * @param invoice
	 * @param clazz
	 * @return
	 */
	private boolean invoiceHasSavedAdditionalData(Invoice invoice, Class clazz) {
		boolean result = false;
		
		List<AdditionalData> additionalData = invoice.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(invoice);
			selectedInvoice.setAdditionalData(additionalData);
		} 
		if ( (additionalData!=null) && (!additionalData.isEmpty())) {
			for (AdditionalData ad : additionalData) {
				if (ad != null) {
					if (ad.getClass().equals(clazz)) {
						result = true;
						break;
					}
				}
			}
		}
		return result;
	}
	
	/**
	 * @param invoiceLine
	 * @param clazz
	 * @return
	 */
	private boolean invoiceLineHasSavedAdditionalData(InvoiceLine invoiceLine, Class clazz) {
		boolean result = false;
		List<AdditionalData> additionalData = invoiceLine.getAdditionalData();
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(invoiceLine);
			invoiceLine.setAdditionalData(additionalData);
		} 
		if ( (additionalData!=null) && (!additionalData.isEmpty()) ) {
			for (AdditionalData ad : additionalData) {
				if (ad != null) {
					if (ad.getClass().equals(clazz)) {
						result = true;
						break;
					}
				}
			}
		}
		return result;
	}
	
	/**
	 * Method used to retrieve and return additional data by class type
	 * 
	 * @param <T>
	 * @param invoiceLine
	 * @param clazz
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private <T> List<T> getInvoiceLineStoredAdditionalData(InvoiceLine invoiceLine, Class<T> clazz) {
		
		List<T> result = new ArrayList<T>();

		List<AdditionalData> additionalData = invoiceLine.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(invoiceLine);
			invoiceLine.setAdditionalData(additionalData);
		} 
		
		if (additionalData != null && !additionalData.isEmpty()) {
			for (AdditionalData ad : additionalData) {
				if (ad!=null) {
					if (ad.getClass().equals(clazz)) {
						result.add((T) ad);
					}
				}	
			}
		}

		return result;
		
	}

	@SuppressWarnings("unchecked")
	private <T> List<T> getInvoiceStoredAdditionalData(Invoice invoice,
			Class<T> clazz) {
		List<T> result = new ArrayList<T>();

		List<AdditionalData> additionalData = invoice.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(invoice);
			// saving them in order to improve performance for further requests
			selectedInvoice.setAdditionalData(additionalData);
		}
		
		if (additionalData != null && !additionalData.isEmpty()) {
			for (AdditionalData ad : additionalData) {
				
				if (ad!=null) {
					if (ad.getClass().equals(clazz)) {
						result.add((T) ad);
					}
				}

			}
		}

		
		return result;
	}
	
	/**
	 * Helper method to save or replace AdditionalData inside Invoice's AdditionalData list
	 * 
	 * @param invoice
	 * @param newAdditionalData
	 * @param clazz
	 */
	public Invoice saveOrReplaceAdditionalData(Invoice invoice,
			AdditionalData newAdditionalData, Class clazz) {

		List<AdditionalData> additionalDataList = invoice.getAdditionalData();
		
		if (additionalDataList == null)
			return invoice;

		// this is to solve java.util.ConcurrentModificationException
		List<AdditionalData> out = new ArrayList<AdditionalData>(
				additionalDataList);
		
		if (additionalDataList!=null) {
		
			boolean sameTypeAdFound = false;
			
			for (AdditionalData ad : additionalDataList) {
				if (ad !=null) {
					if (ad.getClass().equals(clazz)) {
						sameTypeAdFound = true;
						if (newAdditionalData == null) {
							out.remove(additionalDataList.indexOf(ad));
						} else {
							out.set(additionalDataList.indexOf(ad), newAdditionalData);
						}
					}
				}
			}
			if (!sameTypeAdFound && newAdditionalData != null)
				out.add(newAdditionalData);
			
		}
		
		

		invoice.setAdditionalData(out);
		return invoice;
	}
	
	/**
	 * Helper method to save or replace AdditionalData inside InvoiceLine's AdditionalData list
	 * 
	 * @param invoiceLine
	 * @param newAdditionalData
	 * @param clazz
	 */
	public InvoiceLine saveOrReplaceAdditionalData(InvoiceLine invoiceLine, AdditionalData newAdditionalData, Class clazz) {

		List<AdditionalData> additionalDataList = invoiceLine
				.getAdditionalData();

		// this is to solve java.util.ConcurrentModificationException
		List<AdditionalData> out = new ArrayList<AdditionalData>(
				additionalDataList);
		
		if (additionalDataList != null) {
			boolean sameTypeAdFound = false;
			for (AdditionalData ad : additionalDataList) {
				if (ad != null) {
					if (ad.getClass().equals(clazz)) {
						sameTypeAdFound = true;
						
						if (newAdditionalData == null) 
							out.remove(additionalDataList.indexOf(ad));
						else 
							out.set(additionalDataList.indexOf(ad), newAdditionalData);
					}
				}
				
			}
			if (!sameTypeAdFound && newAdditionalData != null)
				out.add(newAdditionalData);
		}

	
		invoiceLine.setAdditionalData(out);
		return invoiceLine;
	}

	/**
	 * @return the missingAdditionalData
	 */
	public boolean isMissingAdditionalData() {
		return 
			!isSupplierCodeSaved() 
				|| 
			!isBudgetSaved() 
				||
			!isFinancialYearSaved()
				||
			!isPaymentMethodSaved()
				||
			!isBankAccountSaved()
				||
			!isInvoiceLinesHasAdditionalDataSaved();
	}

	/**
	 * @param missingAdditionalData the missingAdditionalData to set
	 */
	public void setMissingAdditionalData(boolean missingAdditionalData) {
		this.missingAdditionalData = missingAdditionalData;
	}

	/**
	 * @return the invoiceLinesTogglePanelOpened
	 */
	public boolean isInvoiceLinesTogglePanelOpened() {
		return invoiceLinesTogglePanelOpened;
	}

	/**
	 * @param invoiceLinesTogglePanelOpened the invoiceLinesTogglePanelOpened to set
	 */
	public void setInvoiceLinesTogglePanelOpened(
			boolean invoiceLinesTogglePanelOpened) {
		this.invoiceLinesTogglePanelOpened = invoiceLinesTogglePanelOpened;
	}

	/**
	 * @return the retrievedSuppliersMap
	 */
	public Map<String, Anagrafico> getRetrievedSuppliersMap() {
		return retrievedSuppliersMap;
	}

	/**
	 * @param retrievedSuppliersMap the retrievedSuppliersMap to set
	 */
	public void setRetrievedSuppliersMap(
			Map<String, Anagrafico> retrievedSuppliersMap) {
		this.retrievedSuppliersMap = retrievedSuppliersMap;
	}

	/**
	 * @return the retrievedReasonsMap
	 */
	public Map<String, Causale> getRetrievedReasonsMap() {
		return retrievedReasonsMap;
	}

	/**
	 * @param retrievedReasonsMap the retrievedReasonsMap to set
	 */
	public void setRetrievedReasonsMap(Map<String, Causale> retrievedReasonsMap) {
		this.retrievedReasonsMap = retrievedReasonsMap;
	}

	/**
	 * @return the retrievedCostCentersMap
	 */
	public Map<String, CentroCosto> getRetrievedCostCentersMap() {
		return retrievedCostCentersMap;
	}

	/**
	 * @param retrievedCostCentersMap the retrievedCostCentersMap to set
	 */
	public void setRetrievedCostCentersMap(
			Map<String, CentroCosto> retrievedCostCentersMap) {
		this.retrievedCostCentersMap = retrievedCostCentersMap;
	}

	/**
	 * @return the editingState
	 */
	public boolean isEditingState() {
		return editingState;
	}

	/**
	 * @param editingState the editingState to set
	 */
	public void setEditingState(boolean editingState) {
		this.editingState = editingState;
	}

	/**
	 * @return the goodToCopy
	 */
	public String getGoodToCopy() {
		return goodToCopy;
	}

	/**
	 * @param goodToCopy the goodToCopy to set
	 */
	public void setGoodToCopy(String goodToCopy) {
		this.goodToCopy = goodToCopy;
	}

	/**
	 * @return the lineGoods
	 */
	public Map<Long, String> getLineGoods() {
		if (lineGoods == null) {
			this.updateLineGoods();
		}
		return lineGoods;
	}

	private void updateLineGoods() {
		
		if ( this.isInvoiceLinesTogglePanelOpened() == false ) {
			lineGoods = new HashMap<Long, String>();
			return;
		}
		
		
		lineGoods = new HashMap<Long, String>();
		
		try {
			for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
				List<BeneServizio> retrievedGoods = this.getInvoiceLineStoredAdditionalData(line, BeneServizio.class);
				if (!retrievedGoods.isEmpty()) {
					// prendo la prima perchè il codice bene servizio è sempre uno solo
					lineGoods.put(new Long(line.getInvoiceLineId()), retrievedGoods.get(0).getCodice_bene()+" - "+retrievedGoods.get(0).getDescrizione());
				}
				else {
					lineGoods.put(new Long(line.getInvoiceLineId()), "");
				}
				
			}
		} catch (LazyInitializationException e) {
			return;
		}
	}

	/**
	 * @param lineGoods the lineGoods to set
	 */
	public void setLineGoods(Map<Long, String> lineGoods) {
		this.lineGoods = lineGoods;
	}

	/**
	 * @return the availableGoodsSelectItems
	 */
	public List<SelectItem> getAvailableGoodsSelectItems() {
		
		if (availableGoodsSelectItems == null) {
			updateAvailableGoods();
		}
		return availableGoodsSelectItems;
	}

	/**
	 * Caricamento dei Beni Servizio disponibili
	 */
	private void updateAvailableGoods() {
		
		List<SelectItem> updatedList = new ArrayList<SelectItem>();
		updatedList.add(new SelectItem("Seleziona il bene servizio"));
		retrievedGoodsMap = new HashMap<String, BeneServizio>();
		
		// cerco prima sul db di fe
		String query = "SELECT * FROM CIA_BENE_SERVIZIO WHERE DATA_CANC IS null";
		List<BeneServizio> feRetrievedGoods = JNDIUtils.getCrudManager().findByNativeQuery(query, BeneServizio.class);
		if ((feRetrievedGoods != null) && (!feRetrievedGoods.isEmpty()) ) {
			for (BeneServizio feBeneServizio : feRetrievedGoods) {
				retrievedGoodsMap.put(cleanString(feBeneServizio.getCodice_bene()), feBeneServizio);
			}
		}
		
		// cerco su CIA
		// select * from epoca_bene_servizio where data_canc is null
		Map<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("data_canc", "null");
		List<BeneServizio> ciaRetrievedGoods = UnimoreCIAGenericDAO.findByParameters(BeneServizio.class, parameters);
		if ((ciaRetrievedGoods != null) && (!ciaRetrievedGoods.isEmpty())) {
			for(BeneServizio ciaBeneServizio : ciaRetrievedGoods) {
				if (!retrievedGoodsMap.containsValue(ciaBeneServizio)) {
					retrievedGoodsMap.put(cleanString(ciaBeneServizio.getCodice_bene()), ciaBeneServizio);
				}
			}
			
		}
		
		// costruisco la lista di selectItem dalla mappa
		if (!retrievedGoodsMap.isEmpty()) {
			
			Map<String, BeneServizio> sortedMap = new TreeMap<String, BeneServizio>(retrievedGoodsMap);
			Iterator<Entry<String, BeneServizio>> i = sortedMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, BeneServizio> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getCodice_bene())+" - "+cleanString(entry.getValue().getDescrizione())));
			}
			
		}
	
		// costruisco la lista di selectItem dalla mappa
		/*
		if (!retrievedGoodsMap.isEmpty()) {
			Iterator<Entry<String, BeneServizio>> i = retrievedGoodsMap.entrySet().iterator();
			while (i.hasNext()) {
				Entry<String, BeneServizio> entry = i.next();
				updatedList.add(
						new SelectItem(cleanString(entry.getValue().getCodice_bene())+" - "+cleanString(entry.getValue().getDescrizione())));
			}
		}
		*/
		
		else {
			updatedList.add(new SelectItem("0 - DB vuoto o errore di interrogazione"));
		}
		
		availableGoodsSelectItems = updatedList;
		
	}

	/**
	 * @param availableGoodsSelectItems the availableGoodsSelectItems to set
	 */
	public void setAvailableGoodsSelectItems(
			List<SelectItem> availableGoodsSelectItems) {
		this.availableGoodsSelectItems = availableGoodsSelectItems;
	}

	/**
	 * @return the retrievedGoodsMap
	 */
	public Map<String, BeneServizio> getRetrievedGoodsMap() {
		return retrievedGoodsMap;
	}

	/**
	 * @param retrievedGoodsMap the retrievedGoodsMap to set
	 */
	public void setRetrievedGoodsMap(
			Map<String, BeneServizio> retrievedGoodsMap) {
		this.retrievedGoodsMap = retrievedGoodsMap;
	}
	
	@Override
	public boolean isEditable() {
		return this.editable && "Validata tecnicamente".equals(selectedInvoice.getInvoiceState().getDescription());
	}
	
	@Override
	public boolean isFormDisabled() {
		return ! ("Validata tecnicamente".equals(selectedInvoice.getInvoiceState().getDescription()) && !(isEditable()) );
	}

	/**
	 * @return the retrievedFinancialAccountsMap
	 */
	public Map<String, ContoContabile> getRetrievedFinancialAccountsMap() {
		return retrievedFinancialAccountsMap;
	}

	/**
	 * @param retrievedFinancialAccountsMap the retrievedFinancialAccountsMap to set
	 */
	public void setRetrievedFinancialAccountsMap(
			Map<String, ContoContabile> retrievedFinancialAccountsMap) {
		this.retrievedFinancialAccountsMap = retrievedFinancialAccountsMap;
	}

	/**
	 * @return the financialAccount
	 */
	public String getFinancialAccount() {
		// verifico se le righe della fattura hanno già salvato il conto contabile (medesimo per tutte le righe)
		try {
			for (InvoiceLine line : selectedInvoice.getInvoiceLines()) {
				List<ContoContabile> retrievedFinancialAccounts = this.getInvoiceLineStoredAdditionalData(line, ContoContabile.class);
				if ( retrievedFinancialAccounts!=null && !retrievedFinancialAccounts.isEmpty()) {
					// prendo la prima perchè il conto contabile è sempre uno solo
					financialAccount = retrievedFinancialAccounts.get(0).getChiave_completa();	
				}
				if (financialAccount !=null && !"".equals(financialAccount) )
					break;
				
			}
		} catch (LazyInitializationException e) {
			return financialAccount = "";
		}
		return financialAccount;
	}

	/**
	 * @param financialAccount the financialAccount to set
	 */
	public void setFinancialAccount(String financialAccount) {
		this.financialAccount = financialAccount;
	}

	/**
	 * @param availableFinancialAccounts the availableFinancialAccounts to set
	 */
	public void setAvailableFinancialAccounts(
			List<SelectItem> availableFinancialAccounts) {
		this.availableFinancialAccounts = availableFinancialAccounts;
	}
	
	private String cleanString(String s) {
		while (s.contains("  ")) {
			s = s.replace("  ", " ");
		}
		return s.trim();
	}

	
	
	
}
