/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.invoiceUpload;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.common.utils.XmlFactory;
import it.fe.core.par.invoice.Client;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.PaymentDetail;
import it.fe.core.par.invoice.Supplier;
import it.fe.core.par.user.ProfileDetail;
import it.fe.core.par.user.Role;
import it.fe.web.common.JSFUtils;
import it.fe.web.common.SessionUtils;
import it.fe.web.controller.peppol.PeppolHandler;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.portlet.PortletContext;
import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.jdom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Managed bean that handles forms for UBL2-compliant invoices. Serves as
 * skeleton also for Peppol portlets
 * 
 * @author stefano
 * 
 */
public class UBLUploadHandler extends PeppolHandler {

	protected static Logger logger = Logger.getLogger(UBLUploadHandler.class
			.getName());

	public static final String ADMIN_ROLE = "AMMINISTRATORE";
	private static final String EUR = "EUR";
	private static final String NationsCodeFile = "/validation/identifiers/iso_3166-1_list_en.xml";
	private static final String UnitCodeFile = "/validation/xsd/common/CodeList_UnitCode_UNECE_7_04.xsd";
	private static final String CurrencyCodeFile = "/validation/xsd/common/CodeList_CurrencyCode_ISO_7_04.xsd";
	private static final String InvoiceUBLSchemaFile = "/validation/xsd/UBL-2.0/UBL-Invoice-2.0.xsd";

	private static final String ZZ = "ZZ";

	private static final String dropFolder = "/tmp/FE";


	public InvoiceLine currentInvoiceLine;
	public int lineToBeRemovedID;

	/**
	 * Fattura da inviare
	 */
	public Invoice invoice;

	/**
	 * Variabile helper per tenere traccia del ruolo amministrativo di un utente
	 */
	protected Boolean isAdmin = null;

	protected PostalAddress supplierPostalAddress;

	protected PostalAddress clientPostalAddress;

	protected SelectItem[] availableNationCodes;

	protected SelectItem[] availableCurrencies;
	protected SelectItem[] availableUnitCodes;

	private String supplierId;
	private String clientId;
	
	/**
	 * Utilizzata per mostrare o meno i campi cliente come editabili
	 */
	protected boolean clientFormDisabled = true;

	public UBLUploadHandler() {
		initInvoice();
		initInvoiceLine();
		initAvailableNationCodes();
		initAvailableCurrencies();
		initAvailableUnitCodes();
		initSupplierAddress();
		initClientAddress();
		
	}

	private void initSupplierAddress() {
		supplierPostalAddress = new PostalAddress();
		supplierPostalAddress.setCountry("IT");	
	}
	private void initClientAddress() {
		clientPostalAddress = new PostalAddress();
		clientPostalAddress.setBuildingNumber("16");
		clientPostalAddress.setCityName("Modena");
		clientPostalAddress.setPostalZone("41121");
		clientPostalAddress.setStreetName("Piazza Grande");
		clientPostalAddress.setCountrySubentityCode("MO");
		clientPostalAddress.setCountry("IT");
		if ( (invoice != null) && (invoice.getClient()!= null) ){
			invoice.getClient().setClientName("Comune Modena");
			invoice.getClient().setVatId("00221940364");
		}
	}

	protected void initAvailableNationCodes() {
		PortletContext context = (PortletContext) FacesContext
				.getCurrentInstance().getExternalContext().getContext();

		String filename = context.getRealPath(NationsCodeFile);
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		domFactory.setNamespaceAware(true); // never forget this!
		DocumentBuilder builder;
		try {
			builder = domFactory.newDocumentBuilder();
			Document doc = builder.parse(filename);

			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			xpath.setNamespaceContext(new NamespaceContext() {
				public String getNamespaceURI(String prefix) {
					if (prefix.equals("xsd")) {
						return "http://www.w3.org/2001/XMLSchema";
					}
					return XMLConstants.NULL_NS_URI;
				}

				public String getPrefix(String namespaceURI) {
					if (namespaceURI.equals("http://www.w3.org/2001/XMLSchema")) {
						return "xsd";
					}

					return null;
				}

				public Iterator getPrefixes(String namespaceURI) {
					ArrayList list = new ArrayList();

					if (namespaceURI.equals("http://www.w3.org/2001/XMLSchema")) {
						list.add("xsd");
					}
					return list.iterator();
				}
			});

			XPathExpression names = xpath.compile("//ISO_3166-1_Country_name/text()");
			XPathExpression codeNames = xpath
					.compile("//ISO_3166-1_Alpha-2_Code_element/text()");

			NodeList nodesResult =(NodeList) names.evaluate(doc, XPathConstants.NODESET);
			NodeList codesResult =(NodeList)  codeNames.evaluate(doc, XPathConstants.NODESET);
	

			if ((nodesResult != null) && (nodesResult.getLength() > 0)) {
				availableNationCodes = new SelectItem[nodesResult.getLength()];
				for (int i = 0; i < nodesResult.getLength(); i++) {
					availableNationCodes[i] = new SelectItem(codesResult.item(i)
							.getNodeValue(), nodesResult.item(i).getNodeValue()
							+ " - " + codesResult.item(i).getNodeValue());
				}
			} 
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected void initAvailableCurrencies() {
		PortletContext context = (PortletContext) FacesContext
				.getCurrentInstance().getExternalContext().getContext();

		String filename = context.getRealPath(CurrencyCodeFile);
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		domFactory.setNamespaceAware(true); // never forget this!
		DocumentBuilder builder;
		try {
			builder = domFactory.newDocumentBuilder();
			Document doc = builder.parse(filename);

			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			xpath.setNamespaceContext(new NamespaceContext() {
				public String getNamespaceURI(String prefix) {
					if (prefix.equals("xsd")) {
						return "http://www.w3.org/2001/XMLSchema";
					}
					return XMLConstants.NULL_NS_URI;
				}

				public String getPrefix(String namespaceURI) {
					if (namespaceURI.equals("http://www.w3.org/2001/XMLSchema")) {
						return "xsd";
					}

					return null;
				}

				public Iterator getPrefixes(String namespaceURI) {
					ArrayList list = new ArrayList();

					if (namespaceURI.equals("http://www.w3.org/2001/XMLSchema")) {
						list.add("xsd");
					}
					return list.iterator();
				}
			});

			XPathExpression expr = xpath.compile("//xsd:enumeration/@value");

			Object result = expr.evaluate(doc, XPathConstants.NODESET);
			NodeList nodes = (NodeList) result;

			if ((nodes != null) && (nodes.getLength() > 0)) {
				availableCurrencies = new SelectItem[nodes.getLength()];
				for (int i = 0; i < nodes.getLength(); i++) {
					availableCurrencies[i] = new SelectItem(nodes.item(i)
							.getNodeValue());
				}
			} else
				availableCurrencies = new SelectItem[] { new SelectItem(EUR,
						EUR) };

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	protected void initAvailableUnitCodes() {
		PortletContext context = (PortletContext) FacesContext
				.getCurrentInstance().getExternalContext().getContext();

		String filename = context.getRealPath(UnitCodeFile);
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		domFactory.setNamespaceAware(true); // never forget this!
		DocumentBuilder builder;
		try {
			builder = domFactory.newDocumentBuilder();
			Document doc = builder.parse(filename);

			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			xpath.setNamespaceContext(new NamespaceContext() {
				public String getNamespaceURI(String prefix) {
					if (prefix.equals("xsd")) {
						return "http://www.w3.org/2001/XMLSchema";
					} else if (prefix.equals("ccts")) {
						return "urn:un:unece:uncefact:documentation:2";
					}
					return XMLConstants.NULL_NS_URI;
				}

				public String getPrefix(String namespaceURI) {
					if (namespaceURI.equals("http://www.w3.org/2001/XMLSchema")) {
						return "xsd";
					} else if (namespaceURI
							.equals("urn:un:unece:uncefact:documentation:2")) {
						return "ccts";
					}

					return null;
				}

				public Iterator getPrefixes(String namespaceURI) {
					ArrayList list = new ArrayList();

					if (namespaceURI.equals("http://www.w3.org/2001/XMLSchema")) {
						list.add("xsd");
					} else if (namespaceURI
							.equals("urn:un:unece:uncefact:documentation:2")) {
						list.add("ccts");
					}

					return list.iterator();
				}
			});

			XPathExpression ids = xpath.compile("//xsd:enumeration/@value");
			XPathExpression codeNames = xpath
					.compile("//xsd:enumeration/xsd:annotation/xsd:documentation/ccts:CodeName/text()");

			NodeList resultIDs = (NodeList) ids.evaluate(doc,
					XPathConstants.NODESET);
			NodeList resultCodenames = (NodeList) codeNames.evaluate(doc,
					XPathConstants.NODESET);

			if ((resultIDs != null) && (resultIDs.getLength() > 0)) {
				availableUnitCodes = new SelectItem[resultIDs.getLength()];
				for (int i = 0; i < resultIDs.getLength(); i++) {
					availableUnitCodes[i] = new SelectItem(resultIDs.item(i)
							.getNodeValue(), resultIDs.item(i).getNodeValue()
							+ " - " + resultCodenames.item(i).getNodeValue());
				}
			} else
				availableUnitCodes = new SelectItem[] { new SelectItem(ZZ,
						"ZZ - mutually defined") };

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public Boolean getIsAdmin() {
		isAdmin = false;
		if (isAdmin == null) {
			Role r = JNDIUtils.getRoleManager()
					.getRoleByDescription(ADMIN_ROLE);
			for (ProfileDetail pd : SessionUtils.getCurrentUserProfileDetail()) {
				if (pd.getRole().getRoleId() == r.getRoleId()) {
					isAdmin = true;
				}
			}
		}
		return isAdmin;
	}

	public String getSupplierId() {
		return supplierId;
	}

	public SelectItem[] getAvailableNationCodes() {
		return availableNationCodes;
	}

	public void setAvailableNationCodes(SelectItem[] availableNationCodes) {
		this.availableNationCodes = availableNationCodes;
	}

	public String getClientId() {
		return clientId;
	}

	public void setClientId(String clientId) {
		this.clientId = clientId;
	}

	public void setSupplierId(String supplierId) {
		this.supplierId = supplierId;
	}

	public Invoice getInvoice() {
		return invoice;
	}

	public SelectItem[] getAvailableCurrencies() {
		return availableCurrencies;
	}

	public SelectItem[] getAvailableUnitCodes() {
		return availableUnitCodes;
	}

	public InvoiceLine getCurrentInvoiceLine() {
		return currentInvoiceLine;
	}

	public void setCurrentInvoiceLine(InvoiceLine currentInvoiceLine) {
		this.currentInvoiceLine = currentInvoiceLine;
	}

	public int getLineToBeRemovedID() {
		return lineToBeRemovedID;
	}

	public void setLineToBeRemovedID(int lineToBeRemovedID) {
		this.lineToBeRemovedID = lineToBeRemovedID;
	}

	public PostalAddress getClientPostalAddress() {
		return clientPostalAddress;
	}

	public void setClientPostalAddress(PostalAddress clientPostalAddress) {
		this.clientPostalAddress = clientPostalAddress;
	}

	public PostalAddress getSupplierPostalAddress() {
		return supplierPostalAddress;
	}

	public void setSupplierPostalAddress(PostalAddress supplierPostalAddress) {
		this.supplierPostalAddress = supplierPostalAddress;
	}

	/*
	 * ACTION METHODS
	 */

	public boolean isClientFormDisabled() {
		return clientFormDisabled;
	}

	public void setClientFormDisabled(boolean clientFormDisabled) {
		this.clientFormDisabled = clientFormDisabled;
	}

	public String addLine() {
		if (this.currentInvoiceLine != null) {
			invoice.getInvoiceLines().add(this.currentInvoiceLine);
			updateCurrentInvoiceTotal(currentInvoiceLine.getPriceAmount());
			initInvoiceLine();
		}
		return "";
	}

	private void updateCurrentInvoiceTotal(Double amountToAdd) {
		invoice.setAllInclusiveTotal(invoice.getAllInclusiveTotal()
				+ amountToAdd);
	}

	public String removeLine() {

		int lineNumber = lineToBeRemovedID;

		InvoiceLine toBeRemoved = null;
		for (InvoiceLine il : invoice.getInvoiceLines())
			if (il.getLineNumber() == lineNumber) {
				updateCurrentInvoiceTotal(-il.getPriceAmount());
				toBeRemoved = il;
			}
		if (toBeRemoved != null)
			invoice.getInvoiceLines().remove(toBeRemoved);

		// aggiorna il numero di linea della riga corrente
		updateCurrentInvoiceLineNumber();

		// aggiorna il numero di linea delle righe rimanenti
		updateInvoiceLineNumbers();

		return "";
	}

	private void updateInvoiceLineNumbers() {
		int i = 1;
		for (InvoiceLine il : this.invoice.getInvoiceLines()) {
			il.setLineNumber(i);
			i++;
		}

	}

	private void updateCurrentInvoiceLineNumber() {
		this.currentInvoiceLine.setLineNumber(this.invoice.getInvoiceLines()
				.size() + 1);
	}

	public void resetInvoice() {
		initInvoice();
	}

	private void initInvoiceLine() {
		this.currentInvoiceLine = new InvoiceLine();
		this.currentInvoiceLine.setCurrencyID(EUR);
		this.currentInvoiceLine.setUnitCode(ZZ);
		updateCurrentInvoiceLineNumber();
	}

	public void initInvoice() {
		invoice = new Invoice();
		invoice.setSupplier(new Supplier());
		invoice.setClient(new Client());
		invoice.setInvoiceLines(new ArrayList<InvoiceLine>());
		invoice.setPaymentDetail(new PaymentDetail());
	}


	public byte[] createUBL() {
		Element root = null;
		root = XmlFactory.createDocument();

		SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
		XmlFactory.setNumeroFattura(root, invoice.getInvoiceNumber());
		XmlFactory.setDataEmissione(root, sdf.format(invoice.getIssueDate()));

		XmlFactory.setFornitore(root, "", invoice.getSupplier()
				.getSupplierName(), this.supplierId, supplierPostalAddress
				.getStreetName(), supplierPostalAddress.getBuildingNumber(),
				supplierPostalAddress.getCityName(), supplierPostalAddress
						.getPostalZone(), supplierPostalAddress.getDepartment(), supplierPostalAddress.getCountry(),
				supplierPostalAddress.getCountry(), "", supplierPostalAddress
						.getStreetName(), supplierPostalAddress
						.getBuildingNumber(), supplierPostalAddress
						.getCityName(), supplierPostalAddress.getPostalZone(), supplierPostalAddress.getDepartment(),
				supplierPostalAddress.getCountry(), supplierPostalAddress
						.getCountry(), invoice.getSupplier().getVatId());

		XmlFactory.setCliente(root, "", invoice.getClient().getClientName(),
				clientPostalAddress.getStreetName(), clientPostalAddress
						.getBuildingNumber(),
				clientPostalAddress.getCityName(), clientPostalAddress
						.getPostalZone(), clientPostalAddress.getDepartment(), clientPostalAddress.getCountry(),
				clientPostalAddress.getCountry(), "", "", "", "", "", "", "", "",
				invoice.getClient().getVatId());

		if ((invoice.getInvoiceLines() == null)
				|| (invoice.getInvoiceLines().size() == 0)) {
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"lineRequired");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));

			return new byte[] {};
		}
		XmlFactory.setDataScadenza(root, sdf.format(invoice.getPaymentDetail()
				.getPaymentDueDate()), null);
		XmlFactory.setTotaleFattura(root, String.valueOf(invoice
				.getAllInclusiveTotal()), determineCurrencyID(invoice
				.getInvoiceLines()));

		for (InvoiceLine il : invoice.getInvoiceLines()) {
			XmlFactory.setRigaFattura(root, null, il.getItemName(), il
					.getLineNumber(), il.getLineAccountingCost(), il
					.getOrderLineReference(),
					il.getSellersItemIdentification(),

					il.getInvoicedQuantity(), il.getPriceAmount(),
					new ArrayList(), il.getLineExtensionAmount(), il
							.getTaxDetail(), "", il.getItemDescription(), il
							.getUnitCode(), il.getCurrencyID(), null,
					new ArrayList(), null);
		}

		byte[] result = XmlFactory.generateXml(root);
		return result;
	}

	private String determineCurrencyID(List<InvoiceLine> lines) {
		if (lines != null) {
			String currency = null;
			for (InvoiceLine invoiceLine : lines) {
				String currentCurrency = invoiceLine.getCurrencyID();
				if ((currentCurrency == null) || (currentCurrency.equals("")))
					continue;
				else if (currency == null)
					currency = new String(currentCurrency);
				else if (!currency.equalsIgnoreCase(currentCurrency)) {
					String msg = JSFUtils.getBundledMessage(
							"it.fe.bundle.Messages", "inconsistentCurrencies");

					FacesContext.getCurrentInstance().addMessage(
							null,
							new FacesMessage(FacesMessage.SEVERITY_ERROR, msg,
									msg));

					return "";
				}

			}
			return currency;

		}
		return "";
	}

	public String clear() {

		initInvoice();
		initInvoiceLine();
		initAvailableCurrencies();
		initAvailableUnitCodes();
		initSupplierAddress();
		initClientAddress();
		return "";

	}

	@Override
	public void updatePeppolDocuments() {
		// TODO da rimuovere e rifattorizzare la dipendenza

	}

}
