/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol;


import java.io.IOException;
import java.security.CodeSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class IdentifiersHolder {

	public enum Documents {
		Order, OrderResponseSimple, Invoice, CreditNote, Catalogue, Quotation, ApplicationResponse,
	}
	

	public String version;

	public String businessIdScheme; // = "busdox-actorid-upis";
	public String documentIdScheme; // = "busdox-docid-qns";
	public String processIdScheme; // = "cenbii-procid-ubl";

	public Map<String, String> documentIdValues;// =
	// "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2::SubmitInvoice##UBL-2.0";
	public Map<String, String> processIdValues; // = "BII04"

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public String getBusinessIdScheme() {
		return businessIdScheme;
	}

	public void setBusinessIdScheme(String businessIdScheme) {
		this.businessIdScheme = businessIdScheme;
	}

	public String getDocumentIdScheme() {
		return documentIdScheme;
	}

	public void setDocumentIdScheme(String documentIdScheme) {
		this.documentIdScheme = documentIdScheme;
	}

	public String getProcessIdScheme() {
		return processIdScheme;
	}

	public void setProcessIdScheme(String processIdScheme) {
		this.processIdScheme = processIdScheme;
	}

	public Map<String, String> getDocumentIdValues() {
		return documentIdValues;
	}

	public void setDocumentIdValues(Map<String, String> documentIdValues) {
		this.documentIdValues = documentIdValues;
	}

	public Map<String, String> getProcessIdValues() {
		return processIdValues;
	}

	public void setProcessIdValues(Map<String, String> processIdValues) {
		this.processIdValues = processIdValues;
	}

	public static List<String> getVersions(Document xml) {
		List<String> out = new ArrayList<String>();

		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();

		XPathExpression versions;
		
		try {
			versions = xpath.compile("//IdentifiersHolder/@version");
			
			NodeList nodes = (NodeList) versions.evaluate(xml,
					XPathConstants.NODESET);

			if (nodes != null) {
				for (int i = 0; i < nodes.getLength(); i++) {
					out.add(nodes.item(i).getNodeValue());
				}
			}

		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return out;
	}

	public static String getConfigElement(Document xml, String version,
			String element) {

		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();

		XPathExpression versions;
		try {
			versions = xpath.compile("//IdentifiersHolder[@version = '"
					+ version + "']/" + element + "/text()");

			NodeList nodes = (NodeList) versions.evaluate(xml,
					XPathConstants.NODESET);

			if ((nodes != null) && (nodes.item(0) != null) ){
				return nodes.item(0).getNodeValue();
			}

		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return "";
	}

	public static String getProcessScheme(Document xml, String version) {
		return getConfigElement(xml, version, "processIdScheme");
	}

	public static String getDocumentScheme(Document xml, String version) {
		return getConfigElement(xml, version, "documentIdScheme");
	}

	public static String getBusinessScheme(Document xml, String version) {
		return getConfigElement(xml, version, "businessIdScheme");
	}

	public static Map<String, String> getDocumentIDs(Document xml,
			String version) {
		Map<String, String> out = new HashMap<String, String>();

		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();

		XPathExpression idExpr;
		XPathExpression docTypeExpr;
		try {
			idExpr = xpath.compile("//IdentifiersHolder[@version = '" + version
					+ "']/AvailableDocumentIDs/documentId/text()");

			docTypeExpr = xpath.compile("//IdentifiersHolder[@version = '"
					+ version + "']/AvailableDocumentIDs/documentId/@docType");

			NodeList idNodes = (NodeList) idExpr.evaluate(xml,
					XPathConstants.NODESET);

			NodeList docTypeNodes = (NodeList) docTypeExpr.evaluate(xml,
					XPathConstants.NODESET);

			if (docTypeNodes != null) {
				for (int i = 0; i < docTypeNodes.getLength(); i++) {
					out.put(docTypeNodes.item(i).getNodeValue(), idNodes
							.item(i).getNodeValue());
				}
			}

		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return out;
	}

	public static Map<String, String> getProcessIDs(Document xml, String version) {
		Map<String, String> out = new HashMap<String, String>();

		XPathFactory factory = XPathFactory.newInstance();
		XPath xpath = factory.newXPath();

		XPathExpression idExpr;
		XPathExpression docTypeExpr;
		try {
			idExpr = xpath.compile("//IdentifiersHolder[@version = '" + version
					+ "']/AvailableProcessIDs/processId/text()");

			docTypeExpr = xpath.compile("//IdentifiersHolder[@version = '"
					+ version + "']/AvailableProcessIDs/processId/@docType");

			NodeList idNodes = (NodeList) idExpr.evaluate(xml,
					XPathConstants.NODESET);

			NodeList docTypeNodes = (NodeList) docTypeExpr.evaluate(xml,
					XPathConstants.NODESET);

			if (docTypeNodes != null) {
				for (int i = 0; i < docTypeNodes.getLength(); i++) {
					out.put(docTypeNodes.item(i).getNodeValue(), idNodes
							.item(i).getNodeValue());
				}
			}

		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return out;
	}

	public static List<IdentifiersHolder> readFromDocument(Document xml) {
		List<IdentifiersHolder> out = new ArrayList<IdentifiersHolder>();

		List<String> versions = getVersions(xml);
		if (versions != null)
			for (String version : versions) {
				IdentifiersHolder current = new IdentifiersHolder();
				current.setVersion(version);
				current.setBusinessIdScheme(getBusinessScheme(xml, version));
				current.setDocumentIdScheme(getDocumentScheme(xml, version));
				current.setProcessIdScheme(getProcessScheme(xml, version));
				current.setDocumentIdValues(getDocumentIDs(xml, version));
				current.setProcessIdValues(getProcessIDs(xml, version));
				out.add(current);
				System.out.println(version);
				System.out.println("\t" + current.getBusinessIdScheme());
				System.out.println("\t" + current.getDocumentIdScheme());
				System.out.println("\t" + current.getProcessIdScheme());
				if ((current.getDocumentIdValues() != null)
						&& (current.getDocumentIdValues().keySet() != null))
					for (String doc : current.getDocumentIdValues().keySet()) {
						System.out.println("\t\t " + doc + " - "
								+ current.getDocumentIdValues().get(doc));
					}
				if ((current.getProcessIdValues() != null)
						&& (current.getProcessIdValues().keySet() != null))
					for (String doc : current.getProcessIdValues().keySet()) {
						System.out.println("\t\t " + doc + " - "
								+ current.getProcessIdValues().get(doc));
					}
			}

		return out;
	}

	public static void main(String[] args) {
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		domFactory.setNamespaceAware(true); // never forget this!
		DocumentBuilder builder;
		try {
			builder = domFactory.newDocumentBuilder();
			Document doc = builder
					.parse("/home/stefano/WorkspaceVari/NuovoWorkspace/FatturazioneElettronica_trunk/web/portlets/peppol-outgoing/src/main/webapp/validation/identifiers/configurations.xml");

			readFromDocument(doc);

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
