/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.web.common.JSFUtils;
import it.fe.web.controller.invoiceUpload.UBLUploadHandler;
import it.fe.web.controller.peppol.limeclient.MessageUploader;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.xml.parsers.ParserConfigurationException;

import org.busdox.transport.lime.api.interfaces.MessageException;
import org.xml.sax.SAXException;

/**
 * Questo handler gestisce una form utente che consente di inserire i dati
 * necessari alla creazione di una fattura UBL (questa sezione di funzionalità è
 * ereditata dalla classe UBLUploadHandler.
 * 
 * Una volta creato il doc UBL corretto, l'handler gestisce anche l'invio ad un
 * server LIME
 * 
 * @author stefano
 * 
 */
public class LIMEUploadHandler extends UBLUploadHandler {

	public String dropFolder = System.getProperty("user.home")
			+ "/FE-SERVER/invoices/outgoing";
	private static final String client_truststore = "client-truststore.jks";
	private String senderID = "0010:5798000000303";
	private boolean leaveMessages = false;
	public String serverAddress = "https://193.178.246.22/busdox-transport-lime-server/wstransferService";
	public String receiverID = "0010:02249070356";

	public String username = "admin";
	public String password = "admin";

	public LIMEUploadHandler() {
		super();
		super.setSupplierId(senderID);
		super.setClientId(receiverID);
		File dropFolderFile = new File(dropFolder);
		if (!dropFolderFile.exists()) {
			dropFolderFile.mkdirs();
		}
	}

	public String getServerAddress() {
		return serverAddress;
	}

	public void setServerAddress(String serverAddress) {
		this.serverAddress = serverAddress;
	}

	public String getReceiverID() {
		return receiverID;
	}

	public void setReceiverID(String receiverID) {
		this.receiverID = receiverID;
	}

	public String getSenderID() {
		return senderID;
	}

	public void setSenderID(String senderID) {
		this.senderID = senderID;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String saveLocally() {
		byte[] UBLFile = createUBL();

		try {

			String fileName = new Date().getTime() + ".xml";
			String filePath = dropFolder + File.separator + fileName;
			
			FileOutputStream fos = new FileOutputStream(new File(filePath));
			fos.write(UBLFile);
			fos.close();

			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"savedFile");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg));

			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, fileName,
							fileName));

			OriginalDocument document = new OriginalDocument();
			document.setReceiptDateTime(new Date());
			document.setSender(this.receiverID);
			document.setAdditionalInfo(fileName);
			document.setTransmissionType(JNDIUtils.getTransmissionTypeManager()
					.getTransmissionTypeByDescription("PEPPOL_DRAFT"));

			document.setReceiptDocuments(new ArrayList<FEFile>());
			
			FEFile receiptDocument = new FEFile();
			receiptDocument.setFileMimeType("application/xml");
			receiptDocument.setFileName(fileName);
			receiptDocument.setFileBlob(UBLFile);
			document.getReceiptDocuments().add(receiptDocument);

			long out = JNDIUtils.getOriginalDocumentManager()
					.insertOriginalDocument(document);

			// Un file su Draft viene sempre caricato validando contro tuttte
			// gli schematron,
			// compresi quelli domestic
			applyValidations(out, true);

			super.clear();

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSaved");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSaved");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));

		}
		return "";
	}

	public String send() {
		PeppolIdentifierHandler peppolIdentifierHandler = (PeppolIdentifierHandler) JSFUtils
				.getManagedBean("peppolIdentifierHandler");

		MessageUploader uploader = new MessageUploader(serverAddress, username,
				password, receiverID, senderID, peppolIdentifierHandler
						.getIdHolder());
		byte[] UBLFile = createUBL();
		try {
			String fileName = dropFolder + File.separator
					+ new Date().getTime() + ".xml";
			FileOutputStream fos = new FileOutputStream(new File(fileName));
			fos.write(UBLFile);
			fos.close();

			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"savedFile");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg));

			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, fileName,
							fileName));

			uploader.sendMessage(fileName);

			msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"sentFile");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg));

			super.clear();

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSaved");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSaved");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));

		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSent");
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSent");
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
		} catch (MessageException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSent");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
		}
		return "";
	}

}
