/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol;

import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.PeppolValidation;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.core.par.user.Organization;
import it.fe.core.par.user.OrganizationProperty;
import it.fe.core.par.user.ProfileDetail;
import it.fe.core.par.user.Role;
import it.fe.core.par.user.User;
import it.fe.web.common.SessionUtils;

import java.util.List;

import javax.faces.context.FacesContext;
import javax.portlet.PortletContext;

import net.sf.saxon.expr.CurrentItemExpression;

import org.apache.log4j.Logger;


public abstract class PeppolHandler {

	public String dropFolder = "/tmp/FE";
	public String serverAddress = "https://peppol-ap.regione.emilia-romagna.it/transport-lime-server-1.0.2-SNAPSHOT/wstransferService";
	//EPOCA
//	public String receiverID = "9906:02249070356";

	//Intercent-ER
	public String receiverID = "9906:91252510374";	
	
	public static final String ADMIN_ROLE = "AMMINISTRATORE";
	
	protected String username = "unused";
	protected String password = "unused";

	protected static final String InvoiceUBLSchemaFile = "/validation/xsd/UBL-2.0/UBL-Invoice-2.0.xsd";
	
	protected static final String InvoiceSCHEMATRON_BIICORE_SCH = "/validation/schematron/artefactsv2_2/biicore/schematron/BIICORE-UBL-T10.sch";
	protected static final String InvoiceSCHEMATRON_BIIPROFILES_SCH = "/validation/schematron/artefactsv2_2/biiprofiles/schematron/BIIPROFILES-UBL-T10.sch";
	protected static final String InvoiceSCHEMATRON_BIIRULES_SCH = "/validation/schematron/artefactsv2_2/biirules/schematron/BIIRULES-UBL-T10.sch";
	protected static final String InvoiceSCHEMATRON_EU_SCH = "/validation/schematron/artefactsv2_2/eugen/schematron/EUGEN-UBL-T10.sch";

	protected static final String InvoiceSCHEMATRON_IT_SCH = "/validation/schematron/artefactsv2_2/itnat/schematron/ITNAT-UBL-T10.sch";
	
	protected static final String IT_STYLESHEET = "/validation/schematron/schematron4ITBusinessRules/validation-xslt/XSLT4SVRL_perVisualizzazioneReportDiValidazione.xsl";
	protected static final String BII_STYLESHEET = "/validation/schematron/WP5coreinvoicepackage-v0.95/XSLT4SVRL_ReportVisualization_BII.xsl";
	protected static final String EU_STYLESHEET = "/validation/schematron/WP5coreinvoicepackage-v0.95/XSLT4SVRL_ReportVisualization_EU.xsl";

	protected String ublSchemaFileName;
	protected String biicore_schematronFileName;
	protected String biiprofile_schematronFileName;
	protected String biirules_schematronFileName;
	protected String eu_schematronFileName;
	protected String it_schematronFileName;
	protected String it_stylesheet;
	protected String bii_stylesheet;
	protected String eu_stylesheet;

	protected List<PeppolDocument> peppolDocuments = null;

	/**
	 * Variabile che abilita/disabilita la validazione delle fatture mediante
	 * schematron (BII-CORE, PEPPOL, e nazionali (IT)) La validazione viene
	 * automaticamente disabilitata in caso non siano presenti schematron nel
	 * package war deployato (vd. costruttore default di questa classe)
	 * 
	 */
	protected boolean schematronValidationEnabled = false;

	/**
	 * Variabile che abilita/disabilita la validazione delle fatture mediante
	 * schema XSD UBL La validazione viene automaticamente disabilitata in caso
	 * non siano presenti schematron nel package war deployato (vd. costruttore
	 * default di questa classe)
	 * 
	 */
	protected boolean xsdValidationEnabled = false;

	private static final Logger logger = Logger.getLogger(PeppolHandler.class);

	protected Boolean isAdmin = null;
	
	public PeppolHandler() {
		try {
			this.ublSchemaFileName = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(InvoiceUBLSchemaFile);
			if ((this.ublSchemaFileName == null)
					|| (this.ublSchemaFileName.equals("")))
				xsdValidationEnabled = false;

		} catch (Exception e) {
			logger
					.warn("NO UBL XSD provided: XSD invoice validation has been disabled!");
			xsdValidationEnabled = false;
		}

		try {
			
			
			this.biicore_schematronFileName = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(InvoiceSCHEMATRON_BIICORE_SCH);
			
			this.biiprofile_schematronFileName = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(InvoiceSCHEMATRON_BIIPROFILES_SCH);

			this.biirules_schematronFileName = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(InvoiceSCHEMATRON_BIIRULES_SCH);
			
			this.eu_schematronFileName = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(InvoiceSCHEMATRON_EU_SCH);

			this.it_schematronFileName = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(InvoiceSCHEMATRON_IT_SCH);

			this.it_stylesheet = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(IT_STYLESHEET);

			this.bii_stylesheet = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(BII_STYLESHEET);

			this.eu_stylesheet = ((PortletContext) FacesContext
					.getCurrentInstance().getExternalContext().getContext())
					.getRealPath(EU_STYLESHEET);

			if ((this.biicore_schematronFileName == null)
					|| (this.biicore_schematronFileName.equals(""))
					|| (this.eu_schematronFileName == null)
					|| (this.eu_schematronFileName.equals(""))
					|| (this.it_schematronFileName == null)
					|| (this.eu_schematronFileName.equals("")))

				xsdValidationEnabled = false;

		} catch (Exception e) {
			logger
					.warn("Some Schematrons missing: schematron invoice validation has been disabled!");

			schematronValidationEnabled = false;
		}
	}

	

	public boolean applyValidations(long docId, boolean domesticRequired) {

		OriginalDocument document = JNDIUtils.getOriginalDocumentManager()
				.fetchOriginalDocument(docId);
		if (document == null)
			return true;

		List<FEFile> receipts = document.getReceiptDocuments();
		FEFile doc = null;
		if (receipts != null)
			for (FEFile feFile : receipts) {
				if ("application/xml".equals(feFile.getFileMimeType())) {
					doc = feFile;
				}

			}
		if (doc == null)
			return true;

		byte[] UBLFile = doc.getFileBlob();

		if (xsdValidationEnabled == true) {
			String result = ValidationFactory.validateXml(ublSchemaFileName,
					UBLFile);
			PeppolValidation xsd_validation = new PeppolValidation();
			xsd_validation.setOriginalDocumentId(docId);
			xsd_validation.setDocumentName(doc.getFileName());
			xsd_validation.setXmlFile(result.getBytes());
			xsd_validation.setHtmlFile(result.getBytes());
			xsd_validation.setValidationType("XSD");
			xsd_validation.setValidationStatus(result);

			JNDIUtils.getCrudManager().create(xsd_validation);
		}

		if (schematronValidationEnabled == true) {

			byte[] bii_core_result = ValidationFactory.validateSchematronSCH(
					biicore_schematronFileName, UBLFile);
			byte[] bii_profile_result = ValidationFactory.validateSchematronSCH(
					biiprofile_schematronFileName, UBLFile);
			byte[] bii_rules_result = ValidationFactory.validateSchematronSCH(
					biirules_schematronFileName, UBLFile);
			byte[] eu_result = ValidationFactory.validateSchematronSCH(
					eu_schematronFileName, UBLFile);
			byte[] it_result = ValidationFactory.validateSchematronSCH(
					it_schematronFileName, UBLFile);

			PeppolValidation bii_validation = new PeppolValidation();
			bii_validation.setOriginalDocumentId(docId);
			bii_validation.setDocumentName(doc.getFileName());
			bii_validation.setXmlFile(bii_core_result);
			bii_validation.setHtmlFile(ValidationFactory
					.applyXSLTransformation(bii_stylesheet, bii_core_result));
			bii_validation.setValidationType("BII-CORE-SCH");
			bii_validation.setValidationStatus(ValidationFactory
					.determineValidationStatus(bii_core_result));

			
			PeppolValidation bii_profiles_validation = new PeppolValidation();
			bii_profiles_validation.setOriginalDocumentId(docId);
			bii_profiles_validation.setDocumentName(doc.getFileName());
			bii_profiles_validation.setXmlFile(bii_profile_result);
			bii_profiles_validation.setHtmlFile(ValidationFactory
					.applyXSLTransformation(bii_stylesheet, bii_profile_result));
			bii_profiles_validation.setValidationType("BII-PROFILES-SCH");
			bii_profiles_validation.setValidationStatus(ValidationFactory
					.determineValidationStatus(bii_profile_result));

			
			PeppolValidation bii_rules_validation = new PeppolValidation();
			bii_rules_validation.setOriginalDocumentId(docId);
			bii_rules_validation.setDocumentName(doc.getFileName());
			bii_rules_validation.setXmlFile(bii_rules_result);
			bii_rules_validation.setHtmlFile(ValidationFactory
					.applyXSLTransformation(bii_stylesheet, bii_rules_result));
			bii_rules_validation.setValidationType("BII-RULES-SCH");
			bii_rules_validation.setValidationStatus(ValidationFactory
					.determineValidationStatus(bii_rules_result));
			
			
			PeppolValidation eu_validation = new PeppolValidation();
			eu_validation.setOriginalDocumentId(docId);
			eu_validation.setDocumentName(doc.getFileName());
			eu_validation.setXmlFile(eu_result);
			eu_validation.setHtmlFile(ValidationFactory.applyXSLTransformation(
					eu_stylesheet, eu_result));
			eu_validation.setValidationType("EU-SCH");
			eu_validation.setValidationStatus(ValidationFactory
					.determineValidationStatus(eu_result));
			JNDIUtils.getCrudManager().create(bii_validation);
			JNDIUtils.getCrudManager().create(bii_profiles_validation);
			JNDIUtils.getCrudManager().create(bii_rules_validation);
			JNDIUtils.getCrudManager().create(eu_validation);

			if (domesticRequired) {
				PeppolValidation it_validation = new PeppolValidation();
				it_validation.setOriginalDocumentId(docId);
				it_validation.setDocumentName(doc.getFileName());
				it_validation.setXmlFile(it_result);
				it_validation.setHtmlFile(ValidationFactory
						.applyXSLTransformation(it_stylesheet, it_result));
				it_validation.setValidationType("IT-SCH");
				it_validation.setValidationStatus(ValidationFactory
						.determineValidationStatus(it_result));
				JNDIUtils.getCrudManager().create(it_validation);

			}
		}

		// TODO
		return true;
	}

	public abstract void updatePeppolDocuments();

	public boolean determineDomesticValidationRequired(Long message) {
		// TODO Auto-generated method stub
		return true;
	}

	public List<PeppolDocument> getPeppolDocuments() {
//		if (peppolDocuments == null)
			updatePeppolDocuments();
		return peppolDocuments;
	}

	public void setPeppolDocuments(List<PeppolDocument> peppolDocuments) {
		this.peppolDocuments = peppolDocuments;
	}

	public String getDropFolder() {
		return dropFolder;
	}

	public void setDropFolder(String dropFolder) {
		this.dropFolder = dropFolder;
	}

	public String getServerAddress() {
		return serverAddress;
	}

	public void setServerAddress(String serverAddress) {
		this.serverAddress = serverAddress;
	}

	public String getReceiverID() {
		return receiverID;
	}

	public void setReceiverID(String receiverID) {
		this.receiverID = receiverID;
	}

	public boolean isXsdValidationEnabled() {
		return xsdValidationEnabled;
	}

	public void setXsdValidationEnabled(boolean xsdValidationEnabled) {
		this.xsdValidationEnabled = xsdValidationEnabled;
	}

	public boolean isSchematronValidationEnabled() {
		return schematronValidationEnabled;
	}

	public void setSchematronValidationEnabled(
			boolean schematronValidationEnabled) {
		this.schematronValidationEnabled = schematronValidationEnabled;
	}



	public String getUsername() {
		return username;
	}



	public void setUsername(String username) {
		this.username = username;
	}



	public String getPassword() {
		return password;
	}



	public void setPassword(String password) {
		this.password = password;
	}
	
	public Boolean getIsAdmin() {
		isAdmin = false;
		if (isAdmin == null) {
			Role r = JNDIUtils.getRoleManager()
					.getRoleByDescription(ADMIN_ROLE);
			for (ProfileDetail pd : SessionUtils.getCurrentUserProfileDetail()) {
				if (pd.getRole().getRoleId() == r.getRoleId()) {
					isAdmin = true;
				}
			}
		}
		return isAdmin;
	}
}
