/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol;

import java.util.List;

import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.portlet.PortletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;

/**
 * Handler per la gestione degli identificativi Peppol.
 * 
 * Fornisce funzionalità per reperire le differenti configurazioni a partire da
 * un file di config deployato nella applicazione e le relative funzionalità di
 * selezione lato utente.
 * 
 * NB: richiede che il file di configurazione sia deployato all'interno del war,
 * con il seguente path:
 * 
 * /validation/identifiers/configurations.xml
 * 
 * @author stefano
 * 
 */
public class PeppolIdentifierHandler {

	private static final String HolderConfigFile = "/validation/identifiers/configurations.xml";
	public IdentifiersHolder idHolder;
	public String identifiersVersion;
	public SelectItem[] availableHolderItems = new SelectItem[] {};
	public List<IdentifiersHolder> availableHolders;

	public PeppolIdentifierHandler() {
		super();
		initHolders();
	}

	protected void initHolders() {
		PortletContext context = (PortletContext) FacesContext
				.getCurrentInstance().getExternalContext().getContext();

		String filename = context.getRealPath(HolderConfigFile);
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		domFactory.setNamespaceAware(true); // never forget this!
		DocumentBuilder builder;
		try {
			builder = domFactory.newDocumentBuilder();
			Document doc = builder.parse(filename);
			availableHolders = IdentifiersHolder.readFromDocument(doc);
			if ((availableHolders != null) && (availableHolders.size() > 0)) {
				idHolder = availableHolders.get(0);
				int i = 0;
				availableHolderItems = new SelectItem[availableHolders.size()];
				for (IdentifiersHolder h : availableHolders) {
					availableHolderItems[i] = new SelectItem(h.getVersion(), h
							.getVersion());
					i++;
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public IdentifiersHolder getIdHolder() {
		return idHolder;
	}

	public void setIdHolder(IdentifiersHolder idHolder) {
		this.idHolder = idHolder;
	}

	public String getIdentifiersVersion() {
		return identifiersVersion;
	}

	public void setIdentifiersVersion(String identifiersVersion) {
		this.identifiersVersion = identifiersVersion;
		if ((identifiersVersion != null) && (!identifiersVersion.equals(""))) {
			for (IdentifiersHolder h : this.availableHolders) {
				if (identifiersVersion.equals(h.getVersion())) {
					this.idHolder = h;
				}
			}
		}
	}

	public SelectItem[] getAvailableHolderItems() {
		return availableHolderItems;
	}

	public void setAvailableHolderItems(SelectItem[] availableHolderItems) {
		this.availableHolderItems = availableHolderItems;
	}

	public List<IdentifiersHolder> getAvailableHolders() {
		return availableHolders;
	}

	public void setAvailableHolders(List<IdentifiersHolder> availableHolders) {
		this.availableHolders = availableHolders;
	}

}
