/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.par.PeppolValidation;
import it.fe.core.par.common.FEFile;
import it.fe.core.par.invoice.OriginalDocument;
import it.fe.web.common.JSFUtils;
import it.fe.web.controller.peppol.limeclient.MessageUploader;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.busdox.transport.lime.api.interfaces.MessageException;
import org.primefaces.event.FileUploadEvent;
import org.primefaces.model.UploadedFile;
import org.richfaces.event.UploadEvent;
import org.richfaces.model.UploadItem;
import org.xml.sax.SAXException;

public class SupplierOutboxHandler extends PeppolHandler {

	public String dropFolder = System.getProperty("user.home")
			+ "/FE-SERVER/invoices/outgoing";

	private static final Logger logger = Logger
			.getLogger(SupplierOutboxHandler.class);

	protected Long draftId;
	protected String destination;
	protected String outgoingFile;
	protected String outgoingFileName;

	public SupplierOutboxHandler() {
		super();
		File dropFolderFile = new File(dropFolder);
		if (!dropFolderFile.exists()) {
			dropFolderFile.mkdirs();
		}
	}

	protected List<PeppolDocument> sentPeppolDocuments = null;

	public String refresh() {
		updatePeppolDocuments();
		updateSentPeppolDocuments();
		return "";
	}

	public String send() {
		PeppolIdentifierHandler peppolIdentifierHandler = (PeppolIdentifierHandler) JSFUtils
				.getManagedBean("peppolIdentifierHandler");

		MessageUploader uploader = new MessageUploader(serverAddress, username,
				password, destination, super.receiverID,
				peppolIdentifierHandler.getIdHolder());

		byte[] UBLFile = getOutgoingFile().getBytes();
		long docId = 0;

		try {
			String fileName = dropFolder + File.separator
					+ new Date().getTime() + ".xml";
			FileOutputStream fos = new FileOutputStream(new File(fileName));
			fos.write(UBLFile);
			fos.close();

			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"savedFile");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg));

			FacesContext.getCurrentInstance().addMessage(
					null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, fileName,
							fileName));

			OriginalDocument document = new OriginalDocument();
			document.setReceiptDateTime(new Date());
			document.setSender(this.receiverID);
			document.setReceiveStatus("SENT");
			document.setAdditionalInfo(this.destination); // se il documento è
			// un doc peppol in
			// uscita, uso
			// additionalInfo
			// per salvare il
			// destinatario
			document.setTransmissionType(JNDIUtils.getTransmissionTypeManager()
					.getTransmissionTypeByDescription("SUPPLIER_OUTGOING"));

			document.setReceiptDocuments(new ArrayList<FEFile>());

			FEFile receiptDocument = new FEFile();
			receiptDocument.setFileMimeType("application/xml");
			receiptDocument.setFileName(this.outgoingFileName);
			receiptDocument.setFileBlob(this.outgoingFile.getBytes());
			document.getReceiptDocuments().add(receiptDocument);

			docId = JNDIUtils.getOriginalDocumentManager()
					.insertOriginalDocument(document);

			applyValidations(docId, determineDomesticValidationRequired(docId));

			uploader.sendMessage(fileName);

			msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"sentFile");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, msg, msg));

			destination = null;
			outgoingFile = null;
			outgoingFileName = null;

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSaved");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
			if (docId > 0)
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						docId, "ERROR", e.getLocalizedMessage());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSaved");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
			if (docId > 0)
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						docId, "ERROR", e.getLocalizedMessage());

		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSent");
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
			if (docId > 0)
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						docId, "ERROR", e.getLocalizedMessage());

		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSent");
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
			if (docId > 0)
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						docId, "ERROR", e.getLocalizedMessage());
		} catch (MessageException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = JSFUtils.getBundledMessage("it.fe.bundle.Messages",
					"notSent");

			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msg));
			if (docId > 0)
				JNDIUtils.getOriginalDocumentManager().updateReceiveStatus(
						docId, "ERROR", e.getLocalizedMessage());
		}
		refresh();
		return "home";
	}

	public String back() {
		return "home";
	}

	public String prepareSend() {

		for (PeppolDocument doc : peppolDocuments) {
			if (doc.getReceivedDocument().getFileId() == draftId.longValue()) {
				this.outgoingFile = new String(doc.getReceivedDocument()
						.getFileBlob());
				this.outgoingFileName = doc.getReceivedDocument().getFileName();
			}
		}
		return "send";
	}

	public String delete() {
		for (PeppolDocument doc : peppolDocuments) {
			if (doc.getReceivedDocument().getFileId() == draftId.longValue()) {

				JNDIUtils.getCrudManager().delete(doc.getOriginalDocument());
				JNDIUtils.getCrudManager().delete(doc.getReceivedDocument());
				if (doc.getValidations() != null) {
					for (PeppolValidation validation : doc.getValidations()) {
						JNDIUtils.getCrudManager().delete(validation);
					}
				}

			}
		}
		return "";
	}

	public void updatePeppolDocuments() {
		peppolDocuments = new ArrayList<PeppolDocument>();

		List<OriginalDocument> originalDocuments = JNDIUtils
				.getCrudManager()
				.findByQuery(
						"from OriginalDocument r where r.transmissionType.description = 'SUPPLIER_DRAFT' order by r.originalDocumentId DESC");

		if (originalDocuments != null) {
			for (OriginalDocument originalDocument : originalDocuments) {
				List<FEFile> receiptDocuments = JNDIUtils
						.getOriginalDocumentManager().fetchOriginalDocument(
								originalDocument.getOriginalDocumentId())
						.getReceiptDocuments();

				List<PeppolValidation> validations = JNDIUtils.getCrudManager()
						.findByQuery(
								"from PeppolValidation v where v.originalDocumentId = "
										+ originalDocument
												.getOriginalDocumentId());
				if (validations == null)
					validations = new ArrayList<PeppolValidation>();
				
				PeppolDocument pd = new PeppolDocument(originalDocument,
						receiptDocuments.get(0), validations);
				peppolDocuments.add(pd);
			}
		}
	}

	public void updateSentPeppolDocuments() {
		sentPeppolDocuments = new ArrayList<PeppolDocument>();

		List<OriginalDocument> originalDocuments = JNDIUtils
				.getCrudManager()
				.findByQuery(
						"from OriginalDocument r where r.transmissionType.description = 'SUPPLIER_OUTGOING' order by r.originalDocumentId DESC");

		if (originalDocuments != null) {
			for (OriginalDocument originalDocument : originalDocuments) {

				List<FEFile> receiptDocuments = JNDIUtils
						.getOriginalDocumentManager().fetchOriginalDocument(
								originalDocument.getOriginalDocumentId())
						.getReceiptDocuments();

				List<PeppolValidation> validations = JNDIUtils.getCrudManager()
						.findByQuery(
								"from PeppolValidation v where v.originalDocumentId = "
										+ originalDocument
												.getOriginalDocumentId());
				if (validations == null)
					validations = new ArrayList<PeppolValidation>();
				
				PeppolDocument pd = new PeppolDocument(originalDocument,
						receiptDocuments.get(0), validations);
				sentPeppolDocuments.add(pd);
			}
		}
	}

	private int uploadsAvailable = 4;

	public Long getDraftId() {
		return draftId;
	}

	public void setDraftId(Long draftId) {
		this.draftId = draftId;
	}

	private ArrayList<File> files = new ArrayList<File>();

	private static final char extensionSeparator = '.';
	private static final File outputDir = new File(FEPropertiesUtil
			.getInvoiceManualUploadDir());
	private static final String DATE_FORMAT = "yyyyMMdd";

	public int getSize() {
		if (getFiles().size() > 0) {
			return getFiles().size();
		} else {
			return 0;
		}
	}


	public String getDestination() {
		return destination;
	}

	public void setDestination(String destination) {
		this.destination = destination;
	}

	public String getOutgoingFile() {
		return outgoingFile;
	}

	public void setOutgoingFile(String outgoingFile) {
		this.outgoingFile = outgoingFile;
	}

	/**
	 * @return the uploadsAvailable
	 */
	public int getUploadsAvailable() {
		return uploadsAvailable;
	}

	/**
	 * @param uploadsAvailable
	 *            the uploadsAvailable to set
	 */
	public void setUploadsAvailable(int uploadsAvailable) {
		this.uploadsAvailable = uploadsAvailable;
	}

	/**
	 * @return the files
	 */
	public ArrayList<File> getFiles() {
		return files;
	}

	/**
	 * @param files
	 *            the files to set
	 */
	public void setFiles(ArrayList<File> files) {
		this.files = files;
	}

	public List<PeppolDocument> getSentPeppolDocuments() {
		if (sentPeppolDocuments == null)
			updateSentPeppolDocuments();
		return sentPeppolDocuments;
	}

	public void setSentPeppolDocuments(List<PeppolDocument> sentPeppolDocuments) {
		this.sentPeppolDocuments = sentPeppolDocuments;
	}

}
