/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.peppol.limeclient;

import it.fe.web.controller.peppol.IdentifiersHolder;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.busdox.transport.lime.api.impl.Factory;
import org.busdox.transport.lime.api.interfaces.CredentialsInterface;
import org.busdox.transport.lime.api.interfaces.EndpointReferenceInterface;
import org.busdox.transport.lime.api.interfaces.MessageException;
import org.busdox.transport.lime.api.interfaces.MessageInterface;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import eu.peppol.common.IReadonlyUsernamePWCredentials;

public class MessageUploader {

	// private final String ublDocumentName = "invoice";
	// private final String documentCode = "BiiCoreTrdm010";
	// private final String businessIdScheme = "busdox-actorid-upis";
	// private final String documentIdScheme =
	// "cenbii_docid_fci::urn:oasis:names:specification:ubl:schema:xsd:" +
	// ublDocumentName + "-2";
	// private final String documentIdValue = ublDocumentName +
	// "##urn:www.cenbii.eu:transaction:"+documentCode+"::ver1.0" ;
	// private final String processIdScheme = "cenbii-procid-ubl";
	// private final String processIdValue = "BII07";

	private final String businessIdScheme = "busdox-actorid-upis";
	private final String documentIdScheme = "busdox-docid-qns";
	private final String documentIdValue = "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2::SubmitInvoice##UBL-2.0";
	private final String processIdScheme = "cenbii-procid-ubl";
	private final String processIdValue = "BII04";

	private final static Logger logger = Logger
			.getLogger(MessageUploader.class);
	//private CredentialsInterface credentials;
	private IReadonlyUsernamePWCredentials credentials;
	private String limeServer;
	private String receiverChannel;
	private String senderChannel;
	private IdentifiersHolder holder;

	/**
	 * @param limeServer
	 *            The URL of the limeserver
	 * @param username
	 *            Username
	 * @param password
	 *            User's password
	 * @param receiverChannel
	 *            Participant Identifier of the receiver
	 * @param uploadDirectory
	 *            Directory in which the file will be stored for the uploading
	 *            in the system
	 */
	public MessageUploader(String limeServer, String username, String password,
			String receiverChannel, String senderChannel,
			IdentifiersHolder idHolder) {
		System.setProperty(
				"com.sun.xml.ws.transport.http.client.HttpTransportPipe.dump",
				"true");
//		Factory factory = new Factory();
		this.credentials = Factory.createCredentials(username,password);
//		credentials.setUsername(username);
//		credentials.setPassword(password);
		this.limeServer = limeServer;
		this.receiverChannel = receiverChannel;
		this.senderChannel = senderChannel;
		if (idHolder != null)
			this.holder = idHolder;
		else {
			initDefaultHolder();
		}

	}

	private void initDefaultHolder() {
		this.holder = new IdentifiersHolder();
		this.holder.setBusinessIdScheme(businessIdScheme);
		this.holder.setDocumentIdScheme(documentIdScheme);
		this.holder.setProcessIdScheme(processIdScheme);

		this.holder.setDocumentIdValues(new HashMap<String, String>());
		this.holder.getDocumentIdValues()
				.put(IdentifiersHolder.Documents.Invoice.toString(),
						documentIdValue);

		this.holder.setProcessIdValues(new HashMap<String, String>());
		this.holder.getProcessIdValues().put(
				IdentifiersHolder.Documents.Invoice.toString(), processIdValue);

	}

	public String sendMessage(String xmlFile) throws SAXException,
			ParserConfigurationException, IOException, MessageException {
		DocumentBuilderFactory domFactory = DocumentBuilderFactory
				.newInstance();
		DocumentBuilder parser = domFactory.newDocumentBuilder();

		// BUG FIX: l'implementazione corrente dello stack WS sembra aggiungere
		// automaticamente la dichiarazione del namespace di default anche se
		// già presente.
		// Rimuovo quella già presente per evitare duplicati e conseguenti file
		// XML non validi
		String fileString = FileUtils
				.readFileToString(new File(xmlFile), "UTF-8");
//				.replace(
//						"xmlns=\"urn:oasis:names:specification:ubl:schema:xsd:Invoice-2\"",
//						"");
		File temp = File.createTempFile("tmpinvoice", ".xml");
		FileUtils.writeStringToFile(temp, fileString);
		Document document = parser.parse(temp);
		return sendMessage(document);
	}

	public String sendMessage(Document document) throws MessageException {
//		Factory factory = new Factory();

		MessageInterface message = Factory.createMessage();

		message.setDocument(document);
		message.setDocumentType(Factory.createDocumentType());
		message.setSender(Factory.createBusinessIdentifier());
		message.getSender().setValue(this.senderChannel);
		message.getSender().setScheme(holder.getBusinessIdScheme());
		message.setReciever(Factory.createBusinessIdentifier());
		message.getReciever().setValue(this.receiverChannel);
		message.getReciever().setScheme(holder.getBusinessIdScheme());
		message.getDocumentType().setValue(
				holder.getDocumentIdValues().get(
						IdentifiersHolder.Documents.Invoice.toString()));
		message.getDocumentType().setScheme(holder.getDocumentIdScheme());
		message.setProcessType(Factory.createProcessType());
		message.getProcessType().setValue(
				holder.getProcessIdValues().get(
						IdentifiersHolder.Documents.Invoice.toString()));
		message.getProcessType().setScheme(holder.getProcessIdScheme());

		EndpointReferenceInterface endpointReference = Factory
				.createEndpointReference();
		endpointReference.setAddress(this.limeServer);
		endpointReference.setChannelID(this.receiverChannel);

		
		logger.info("Receiver:"+this.receiverChannel);
		logger.info("Sender:"+this.senderChannel);
		logger.info("Sender:"+this.senderChannel);
		logger.info("Business ID Scheme:"+holder.getBusinessIdScheme());
		logger.info("Document ID Scheme:"+holder.getDocumentIdScheme());
		logger.info("Document ID:"+holder.getDocumentIdValues().get(
				IdentifiersHolder.Documents.Invoice.toString()));
		
		logger.info("Process ID Scheme:"+holder.getProcessIdScheme());
		logger.info("Process ID:"+holder.getProcessIdValues().get(
				IdentifiersHolder.Documents.Invoice.toString()));
		
		String messageid = Factory.createOutbox().sendMessage(
				this.credentials, message, endpointReference);
		logger.info("OUTBOX - MESSAGE DELIVERED: " + messageid);

		return messageid;

	}

}
