/**
 * This file is part of FatturazioneElettronica - Web - Login Modules - Generic
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Login Modules - Generic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Login Modules - Generic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Login Modules - Generic.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * 
*
 *
 */
package it.fe.web.login.generic;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.common.utils.JNDIUtils;
import it.fe.core.common.utils.PasswordUtils;
import it.fe.core.ejb.user.UserManager;
import it.fe.core.par.user.ProfileDetail;
import it.fe.core.par.user.User;
import it.fe.core.par.user.UserPassword;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;

import com.liferay.portal.DuplicateUserScreenNameException;
import com.liferay.portal.NoSuchUserException;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.Company;
import com.liferay.portal.model.Role;
import com.liferay.portal.security.auth.AutoLogin;
import com.liferay.portal.security.auth.AutoLoginException;
import com.liferay.portal.service.RoleLocalServiceUtil;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.util.PortalUtil;


/**
 * @author fulvio di marco
 * 
 * Generic FE login service: it simply authenticates and authorizes users on local db.
 * After this, creates a new user (or updates an existing one) within Liferay portal in order to
 * make use of portal roles/permissions mechanisms.
 * 
 * On logout, UserDeleteLogoutAction hook is automatically called in order to delete user profile, without 
 * leaving traces on portal's database.
 * 
 *
 */
public class FEGenericAutoLogin implements AutoLogin {
	
	private Logger logger = Logger.getLogger(FEGenericAutoLogin.class);
	
	private final static boolean SUCCESS = true;
	private final static boolean FAILURE = false;
	private static final String _LOGIN_PARAM = "_58_login";
	private static final String _PASSWORD_PARAM = "_58_password";
	
	private long FEorganizationId = Long.valueOf(FEPropertiesUtil.getFELiferayOrganizationId());
	private long FEgroupId = Long.valueOf(FEPropertiesUtil.getFELiferayGroupId());
	
	private UserManager userManager;
	private User user;
	private String clearTxtValidPassword;
	private Set<ProfileDetail> profileDetails;

	/* (non-Javadoc)
	 * @see com.liferay.portal.security.auth.AutoLogin#login(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	public String[] login(HttpServletRequest request, HttpServletResponse response) throws AutoLoginException {
		
		boolean authenticationResult = false;
		
		// credentials[0]: user id;
		// credentials[1]: password in clear text
		// credentials[2]: true/false (String) if password is encrypted or not. eg. Boolean.TRUE.toString().
		String[] credentials = null;
		
		String formUsername = ParamUtil.getString(request, getLoginParam());

		if (Validator.isNull(formUsername)) {
			return null;
		}

		String formPassword = ParamUtil.getString(request, getPasswordParam());

		if (Validator.isNull(formPassword)) {
			return null;
		}

		Company company = null;
		try {
			company = PortalUtil.getCompany(request);
		} catch (PortalException e) {
			throw new AutoLoginException(e);
		} catch (SystemException e) {
			throw new AutoLoginException(e);
		}
		
		if (formUsername !=null && formPassword !=null) {
			if (formUsername.equals("")) {
				return credentials;
			}
		}
		
		// verifico la presenza dell'utente sul DB epoca
		
		logger.info("Checking FE database.");
		try {
			userManager = JNDIUtils.getUserManager();
			user = userManager.getUserByUsername(formUsername);
		} catch(Exception e) {
			logger.error(e.getMessage());
			throw new AutoLoginException(e);
		}
		
		if (user != null) {
			
			logger.info("User "+formUsername+" exists in the FE database. Verifying password...");
			
			// Getting user passwords from database
			List<UserPassword> retrievedPasswords = user.getPasswords();
			
			for(UserPassword each:retrievedPasswords) {
				// verifying if stored date is before current date a.k.a password is valid.
				if ( each.getEndDate() == null || each.getEndDate().compareTo(new Date()) > 0 ) {
					// now comparing retrieved and provided
					try {
						if (each.getPassword().equals(PasswordUtils.encrypt(PasswordUtils.TYPE_SHA, formPassword, null))) {
							authenticationResult = FEGenericAutoLogin.SUCCESS;
							clearTxtValidPassword = formPassword;
							logger.info("Password match for user "+formUsername);
						}
						// user exists but the passwords do not match 
						else {
							logger.info("Password does not match for user "+formUsername);
							authenticationResult = FEGenericAutoLogin.FAILURE;
						} 
					} catch (Exception e) {
						logger.info(e.getMessage());
						authenticationResult = FEGenericAutoLogin.FAILURE;
					}
					
				}
			}
		}
		else {
			// user does not exist on the system
			logger.info("User "+formUsername+" does not exist in the FE database.");
			authenticationResult = FEGenericAutoLogin.FAILURE;
		}
		
		// autenticazione corretta
		if (authenticationResult) {
			
			logger.info("Retrieving user profiles");
			profileDetails = user.getProfile();
			if (profileDetails != null && !profileDetails.isEmpty()) {
				// user profile must be set in Shared Session
				setProfileSessionAttributes(profileDetails, request);
			}
			
			//user must be added to Liferay database in order to access the portal
			com.liferay.portal.model.User liferayUser = null;
			try {
				logger.info("Querying Liferay database by screen name for user "+formUsername);
				liferayUser = UserLocalServiceUtil.getUserByScreenName(company.getCompanyId(), formUsername);
			} catch (NoSuchUserException e) {
				logger.info("User "+formUsername+" does not exist in the Liferay database.");
				authenticationResult = FEGenericAutoLogin.FAILURE;
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
				throw new AutoLoginException(e);
			}
			
		
			// a new user must be created
			if (liferayUser == null) {
				logger.info("Adding user "+formUsername+" to Liferay database.");
				
				long[] roleIds;
				// if a FE ProfileDetail is found, user gets FE roles
				if (profileDetails != null) {
					roleIds = getRoleIds(profileDetails);
				}
				// otherwise user gets a Liferay Role
				else {
					roleIds = new long[]{10122};
				}
				
				try {
	                   
                    liferayUser = UserLocalServiceUtil.addUser(
                    		0, // long creatorUserId
                    		//CompanyThreadLocal.getCompanyId(), // long companyId
                    		PortalUtil.getCompany(request).getCompanyId(), // long companyId
                    		false, // boolean autoPassword 
                    		clearTxtValidPassword, // String password1 
                    		clearTxtValidPassword, // String password2
                    		false, // boolean autoScreenName
                    		formUsername, // String screenName
                    		user.getEmail(), // String emailAddress
                    		StringPool.BLANK, // String openId
                    		Locale.ITALIAN, // Locale locale
                    		user.getName(), // String firstName
                    		StringPool.BLANK, // String middleName
                    		user.getSurname(), // String lastName
                    		0, // int prefixId
                    		0, // int suffixId
                    		true, // boolean male
                    		Calendar.JANUARY, // int birthdayMonth
                    		1, // int birthdayDay
                    		1970, // int birthdayYear
                    		StringPool.BLANK, // String jobTitle
                    		new long[]{FEgroupId}, // long[] groupIds
                    		new long[]{FEorganizationId}, // long[] organizationIds
                    		roleIds, // long[] roleIds
                    		new long[]{}, // long[] userGroupIds
                    		false, // boolean sendEmail
                    		new ServiceContext() // ServiceContext serviceContext
                    		);

                    //RoleLocalServiceUtil.addUserRoles(liferayUser.getUserId(), new long[]{10122});
                    logger.info("Granting access to user "+formUsername);
                    authenticationResult = FEGenericAutoLogin.SUCCESS;
                    credentials = new String[3];
                    credentials[0] = String.valueOf(liferayUser.getUserId());
                    credentials[1] = clearTxtValidPassword;
                    credentials[2] = Boolean.FALSE.toString();               
				} catch (DuplicateUserScreenNameException e) {
					logger.error("Duplicate user Screen name on liferay database.");
					authenticationResult = FEGenericAutoLogin.FAILURE;
					throw new AutoLoginException(e);
				} catch (PortalException e) {
					logger.info(e.getMessage());
					authenticationResult = FEGenericAutoLogin.FAILURE;
					throw new AutoLoginException(e);
				} catch (SystemException e) {
					logger.info(e.getMessage());
					authenticationResult = FEGenericAutoLogin.FAILURE;
					throw new AutoLoginException(e);
				}
				
				
				
			}
			// user information must be updated
			else {
				logger.info("Updating user "+formUsername+" profile.");
				
				long[] roleIds;
				// if a FE ProfileDetail is found, user gets FE roles
				if (profileDetails != null) {
					roleIds = getRoleIds(profileDetails);
				}
				// otherwise user gets a Liferay Role
				else {
					roleIds = new long[]{10122};
				}
				
				try {
					liferayUser = UserLocalServiceUtil.updateUser(
							liferayUser.getUserId(), // long userId
							liferayUser.getPassword(), // String oldPassword
							StringPool.BLANK, // String newPassword1
							StringPool.BLANK, // String newPassword2
							false, // boolean passwordReset 
							liferayUser.getReminderQueryQuestion(), // String reminderQueryQuestion 
							liferayUser.getReminderQueryAnswer(), // String reminderQueryAnswer
							user.getUsername(), // String screenName 
							user.getEmail(), // String emailAddress  
							StringPool.BLANK, // String openId 
							liferayUser.getLanguageId(), // String languageId
							liferayUser.getTimeZoneId(), // String timeZoneId 
							liferayUser.getGreeting(), // String greeting 
							liferayUser.getComments(), // String comments
							user.getName(), // String firstName
							liferayUser.getMiddleName(), // String middleName
							user.getSurname(), // String lastName
							0, // int prefixId
                    		0, // int suffixId
							true, // boolean male
							Calendar.JANUARY, // int birthdayMonth
							1, // int birthdayDay
							1970, // int birthdayYear
							liferayUser.getContact().getSmsSn(), // String smsSn
							liferayUser.getContact().getAimSn(), // String aimSn
							liferayUser.getContact().getFacebookSn(), // String facebookSn
							liferayUser.getContact().getIcqSn(), // String icqSn
							liferayUser.getContact().getJabberSn(), // String jabberSn
							liferayUser.getContact().getMsnSn(), // String msnSn
							liferayUser.getContact().getMySpaceSn(), // String mySpaceSn
							liferayUser.getContact().getSkypeSn(), // String skypeSn
							liferayUser.getContact().getTwitterSn(), // String twitterSn
							liferayUser.getContact().getYmSn(), // String ymSn
							liferayUser.getJobTitle(), // String jobTitle
							new long[]{FEgroupId}, // long[] groupIds
							new long[]{FEorganizationId}, // long[] organizationIds
							roleIds, // long[] roleIds
							null, // List<UserGroupRole> userGroupRoles
							new long[]{}, // long[] userGroupIds
							new ServiceContext() // ServiceContext serviceContext
						);
					
					logger.info("Granting access to user "+formUsername);
	                authenticationResult = FEGenericAutoLogin.SUCCESS;
	                credentials = new String[3];
	                credentials[0] = String.valueOf(liferayUser.getUserId());
	                credentials[1] = clearTxtValidPassword;
	                credentials[2] = Boolean.FALSE.toString();
				} catch (SystemException e) {
					logger.info(e.getMessage());
					authenticationResult = FEGenericAutoLogin.FAILURE;
					throw new AutoLoginException(e);
				} catch (PortalException e) {
					logger.info(e.getMessage());
					authenticationResult = FEGenericAutoLogin.FAILURE;
					throw new AutoLoginException(e);
				}
				
				
			}
			
		}
		
		// authenticationResult == FEUniMoReAutoLogin.FAILURE
		else {
			logger.info("Authentication failed. User does not exist in FE.");
		}		
		
		
		return credentials;
	}
	
	/**
	 * Sets profile details in user's http session
	 * 
	 * Note that session.enable.phishing.protection=false must be set in portal-ext.properties, otherwise
	 * Liferay ordinary login process invalidates session after login deleting all previously set attributes.
	 * 
	 * In order to share attributes from portal to portlets the parameter <private-session-attributes> must be 
	 * set to true in the liferay-portlet.xml of each WAR
	 * 
	 * @param profileDetail
	 * @param request
	 */
	private void setProfileSessionAttributes(Set<ProfileDetail> profileDetail, HttpServletRequest request) {
		
		if (request != null && profileDetail != null && !profileDetail.isEmpty()) {
						
			HttpSession session = request.getSession();
			
			if (session != null) {
				logger.info("Setting user session attributes");
				session.setAttribute("FE_SHARED_USERPROFILEDETAIL", profileDetail);
			}
		
		}
	}
	
	/**
	 * @author fulvio di marco
	 * 
	 * Returns Liferay's role ids matching authenticated user roles
	 * 
	 * @param profileDetails
	 * @return
	 */
	private long[] getRoleIds(Set<ProfileDetail> profileDetails) {
		List<Long> result = new ArrayList<Long>();
		if (profileDetails != null) {
			for (ProfileDetail pd : profileDetails) {
				String feRoleName = pd.getRole().getDescription();
				try {
					List<Role> portalRoles = RoleLocalServiceUtil.getRoles(0,RoleLocalServiceUtil.getRolesCount());
					for (Role portalRole : portalRoles) {
						if (portalRole.getName().equals(feRoleName)) {
							result.add(portalRole.getRoleId());
						}
					}
				} catch (SystemException e) {
					logger.error(e.getMessage());
				}
			}
		}
		
		long[] resultAsArray = new long[result.size()];
		for (int i=0; i < result.size(); i++) {
			resultAsArray[i] = result.get(i);
		}
		
		return resultAsArray;
		
	}
	
	protected String getLoginParam() {
		return _LOGIN_PARAM;
	}

	protected String getPasswordParam() {
		return _PASSWORD_PARAM;
	}

}
